#include "Analyzer.h"

#include "../worldmodel/WorldModelInterface.h"
#include "../worldmodel/Situation.h"

#include "../basic/Timer.h"
#include "../intentions/conditions/KickConditions.h"
#include "../intentions/conditions/AttackConditions.h"


Analyzer *globalAnalyzer;

Analyzer::Analyzer():
  theIdString( "Analyzer" )
{
  if ( globalWorldModel->player().isGoalie() == YES ) {
    // der Goalie bekommt etwas andere Analyzerobjekte
    thePositionAnalyzer = new PositionAnalyzer();
    thePassAnalyzer = new PassAnalyzer();
    theGoalKickAfterCatchPositionAnalyzer = new GoalKickAfterCatchPositionAnalyzer();
    theInterceptAnalyzer = new InterceptAnalyzer();
  }
  else {
    // die Analyzerobjekte fuer die normalen Spieler
    thePassAnalyzer = new PassAnalyzer();
    theScoreGoalAnalyzer = new ScoreGoalAnalyzer();
    theDribbleAnalyzer = new DribbleAnalyzer();
    thePositionAnalyzer = new PositionAnalyzer();
    theInterceptAnalyzer = new InterceptAnalyzer();
  }

  // Conditions anlegen
  theIsKickPossible = new KickPossibleCondition;
  theIsGoalReachable = new GoalReachableCondition;
 
//   // Positionierungskomponente anlegen
//   globalDelib = new Delib( dynamic_formation_442b );
//   thePositionCalculator = globalDelib;
  thePositionTimeStamp = -10;

  simStepOfLastAnalyze = 0;
  haveToAnalyze = NO;
}

Analyzer::~Analyzer() {};

void Analyzer::analyze() {
  INFO( Log::CONDITIONS,  6,  idString() , globalLog->infoStream << "ENTER "
	<< idString() << " analyze\n");  

	if( (globalWorldModel->simStep() > 0) &&
	    ( ( simStepOfLastAnalyze < globalWorldModel->simStep() ) ||
	      ( haveToAnalyze == YES) ) ) {
	//cerr << "Analyzer: sS: " << globalWorldModel->simStep() 
	//	<< " la: " << simStepOfLastAnalyze << " ls: " << globalWorldModel->simStepOfLastSee() << endl;

    simStepOfLastAnalyze = globalWorldModel->simStep();
    haveToAnalyze = NO;
	 /* die einzelnen Grids werden mit Werten belegt
	    --> calculateValues
	    der Aufruf ist abhngig vom timestamp
	*/
	  int duration = globalTimer->getAgentTime();
	  if( globalWorldModel->player().isGoalie() == YES ) {
	    if( globalWorldModel->gameState() & WorldModelInterface::CATCH_BALL_AND_FREE_KICK ) {
			// nur wenn der Goalie gecatched hat, soll er einen Abstoss berechnen 
			theGoalKickAfterCatchPositionAnalyzer->calculateValues();	
		    globalTimer->controller();
	    }	
	    else if( theIsKickPossible->check() == YES ) {    
	      // nur wenn der Goalie am Ball ist sollen die Passvarianten berechnet werden
	      thePassAnalyzer->calculateValues();
	      globalTimer->controller();
	    }
	    else {
	      theInterceptAnalyzer->calculateValues();
  		  thePositionAnalyzer->calculateGoalieValues();
	      globalTimer->controller();
	    }
	  }
	  else {
	    // der Analyzer fuer normale Spieler
	    if( theIsKickPossible->check() == YES ) {
	      thePassAnalyzer->calculateValues();
//	      globalTimer->controller();
	      theDribbleAnalyzer->calculateValues();
	      globalTimer->controller();
	      //if( theIsGoalReachable->check() ) {
	        theScoreGoalAnalyzer->calculateValues();
	        globalTimer->controller();
	      //}
	    }
	    else{
	      theInterceptAnalyzer->calculateValues();
	      globalTimer->controller();
	      if( ( thePositionTimeStamp < globalWorldModel->simStep() - globalConf->calculatePositionInterval ) ||
	          ( globalWorldModel->simStepOfLastRefereeDecision() < globalWorldModel->simStep()  ) ) {
	        if( globalWorldModel->simStep() > 1 ) {
//	          calculatePositionTarget();
	          thePositionAnalyzer->calculateValues();
	          globalTimer->controller();
	        }
	      }
	    }
	  }
	  duration = globalTimer->getAgentTime() - duration;
	  if( duration > 40 ) cerr << "Analyzer brauchte : " << duration << " ms !!!!!\n";
	}
  INFO( Log::CONDITIONS,  6,  idString() , globalLog->infoStream << "LEAVE "
	<< idString() << " analyze\n");  
}

const Analyzer::SituationClass Analyzer::classifySituation(){
  static int timeOfLastClassification = 0;
  static SituationClass sitClass = NOTHING;
  long aTime = globalTimer->getAgentTime();

  if( timeOfLastClassification < aTime - 75 ){
    timeOfLastClassification = aTime;
    return sitClass = situationClassifcation();
  }
  return sitClass;
}



const Analyzer::SituationClass Analyzer::situationClassifcation(){
  INFO( Log::CONDITIONS,  6,  idString() , globalLog->infoStream << "ENTER "
	<< idString() <<	"::classifySituation \n");
  // Initialisierung mit NOTHING
  SituationClass sitClass = NOTHING;

  // Pruefen, ob InitalSituation
  INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	"SituationClass:\nNOTHING");
  if( globalWorldModel->gameState() &
      ( WorldModelInterface::GOAL | WorldModelInterface::BEFORE_KICK_OFF ) ) {
    INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	  "GOAL | BEFORE_KICK_OFF");

    if( globalWorldModel->initiative() == OUR_TEAM_ID ){
      // wir haben Anstoss
      INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	  " | INITIAL_SITUATION_OUR_TEAM");
      sitClass |= INITIAL_SITUATION_OUR_TEAM;
    }
    else{
      // Gegner hat Anstoss
      INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	    " | INITIAL_SITUATION_THEIR_TEAM");
      sitClass |= INITIAL_SITUATION_THEIR_TEAM;
    }
  }
  if( globalWorldModel->gameState() & WorldModelInterface::KICK_OFF &&
      globalWorldModel->initiative() == OUR_TEAM_ID ){
    int index = globalWorldModel->
	situation().indexOfNearestPlayerToPos( Vector( 0, 0 ) );
      if( globalWorldModel->
	  situation().index2Id( index ) ==
	  globalWorldModel->id() ){
	// ich mache Anstoss
	INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	      " | INITIAL_SITUATION_ACTIVE" );
	sitClass |= INITIAL_SITUATION_ACTIVE;
      }
  }

  // Pruefen, ob neue Schiedsrichterendscheidung erfolgte
  // Init mit -1, damit auch die erste Entscheidung des
  // Schiedsrichters mitbekommen wird
  static int simStepOfLastRefereeDecisionNotify = -1;
  if( globalWorldModel->simStepOfLastRefereeDecision() >=
      simStepOfLastRefereeDecisionNotify ){
    simStepOfLastRefereeDecisionNotify = globalWorldModel->simStep();
    INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	  " | GAME_STATE_CHANGE" );
    sitClass |= GAME_STATE_CHANGE;
    if( globalWorldModel->gameState() == WorldModelInterface::PLAY_ON ){
      INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	  " | NEW_NORMAL_SITUATION" );
      sitClass |= NEW_NORMAL_SITUATION;
    }
  }
  // Freekick-Situation testen
  if( globalWorldModel->gameState() &
      ( WorldModelInterface::FREE_KICK | WorldModelInterface::GOAL_KICK |
	WorldModelInterface::CORNER_KICK | WorldModelInterface::KICK_IN |
	WorldModelInterface::OFFSIDE | WorldModelInterface::FOUL )/* &&
      globalWorldModel->gameState() != WorldModelInterface::CATCH_BALL_AND_FREE_KICK */){
// HACK  !!!!      
    if( globalWorldModel->initiative() == OUR_TEAM_ID ){
      sitClass |= FREEKICK_SITUATION_OUR_TEAM;
      INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	    " | FREEKICK_SITUATION_OUR_TEAM" );
	  // HACK !  
      // beim Neuschreiben von estimateStepsToBall ist stepsOfPlayerWithId() rausgefallen...
      // deshalb setzen wir mysteps jetzt einfach auf 0 
      int mySteps =	0; //globalWorldModel->situation().stepsOfPlayerWithId( globalWorldModel->id() );

      if( !( globalWorldModel->situation().stepsOfNearestPlayer( OUR_TEAM_ID ) <
	     mySteps - 1 ) ){
	sitClass |= FREEKICK_SITUATION_ACTIVE;
	INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	      " | FREEKICK_SITUATION_ACTIVE" );
      }
      if( globalWorldModel->ball().pos().x() > -15 ){
	sitClass |= ATTACK_SITUATION;
      }
    }
    else{
      sitClass |= FREEKICK_SITUATION_THEIR_TEAM;
      INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream <<
	    " | FREEKICK_SITUATION_THEIR_TEAM" );
    }

  }
  if( globalWorldModel->gameState() & WorldModelInterface::PLAY_ON ){
    int ourSteps = globalWorldModel->
      situation().stepsOfNearestPlayer( OUR_TEAM_ID );
    int theirSteps =
      globalWorldModel->
      situation().stepsOfNearestPlayer( THEIR_TEAM_ID );

    if( ( (  ourSteps > theirSteps + 2 ) &&
	  ( globalWorldModel->ball().pos().x() < 5 ) ) ||
	( (  ourSteps > theirSteps + 3 ) &&
	  ( globalWorldModel->ball().pos().x() < 20 ) ) ){
      sitClass |= DEFENSE_SITUATION;
    }
    else if( ( ( ourSteps < theirSteps - 2 ) &&
	       ( globalWorldModel->ball().pos().x() > 5 ) ) ||
	     ( ( ourSteps < theirSteps - 3 ) &&
	       ( globalWorldModel->ball().pos().x() > 0 ) )  ||
	     ( ( ourSteps < theirSteps - 4 ) &&
	       ( globalWorldModel->ball().pos().x() > -15 ) ) ){
      sitClass |= ATTACK_SITUATION;
    }
  }
  INFO( Log::CONDITIONS, 10,  idString() , globalLog->infoStream << '\n');
  INFO( Log::CONDITIONS,  6,  idString() , globalLog->infoStream << "LEAVE "
	<< idString() <<	"::classifySituation \n");
  return sitClass;
}
