#ifndef _GRID_H_
#define _GRID_H_

#include "../basic/Vector.h"
#include "../worldmodel/Situation.h"
#include "../worldmodel/WorldModelInterface.h"

const int GRID_RANGE_X = 105;
const int GRID_RANGE_Y = 68;
const int MINPOSX = 0;
const int MAXPOSX = 105;
const int MINPOSY = 0;
const int MAXPOSY = 68;
const int GRID_TYPE_COUNT = 1;


struct Point { double x; double y; };

/** struct fr den Logger */
struct PlayerForGrid {
  int theNumber;
  int theRole;
  int theId;
  int theX;
  int theY;
  int theHomeX;
  int theHomeY;
};

struct ToGrid {
  int indexOfPlayerToShow;
  PlayerForGrid thePlayers[23];
};


/**
  Ein beschreibender Satz.
  Hier die komplette Beschreibung der Idee/Aufgabe der Klasse.

  In den beiden URL doc++-Befehlen nicht vergessen, "Filename" durch den
  Namen der Datei zu ersetzen.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Filename.h} | [\URL[Implementierung]{src/Filename.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Grid.h,v $
  *   $Revision: 1.1.1.1 $
  *   $Date: 2001/07/31 22:23:08 $
  *   $Author: rami $
  *
  **************************************************
  \end{verbatim} }*/

class Grid {

public:
  /** Konstruktor, Destruktor */
  Grid();
  virtual ~Grid();

  /** Berechnung der Werte in dem Grid */
  virtual void calculateValues( int gridType = 0) = 0;

  /** setzt die Werte in dem Grid auf 0.0 */
  virtual void resetValues( int gridType = 0);

  /** liefert den Grid - Wert an der Position pos */
  double getValue(Vector pos, int gridType = 0);

  /** fasst Grid zu einem kleineren Grid zusammen
      Bsp.: standardmig wird ein 3x3 Feld zu einem Punkt zusammengefasst
      (dabei wird der Mittelwert der 9 Zahlen berechnet */
  double* getLessDiscreteGrid( int newX = GRID_RANGE_X/3, int newY = GRID_RANGE_Y/3, int gridType = 0 );

  /** sucht den Punkt mit dem gobalen Maximum */
  Vector* getGlobalMaximum( int gridType = 0);

  /** sucht den Punkt mit dem Maximum um den Spieler, wobei radius
      die Kstchenanzahl um den Spieler angibt */
  Vector* getLocalMaximum( int theRadius = 1, int gridType = 0 );

  /** sucht den Punkt mit dem Maximum um den Spieler, wobei radius
      die Kstchenanzahl um den Spieler angibt
      wird vor allem von ForwardPass benutzt */
  Vector* getMaximumAroundPos( Vector pos, int theRadius = 15, int gridType = 0 );

 /** setzt Zahl, die angibt nach wie vielen Takten eine neue Berechnung
     des Grids erfolgt */
  void setPause( int paused );

  /** Zeit der letzten Berechnung setzen. */
  void setTimeStamp();

  /** Zeit der letzten Berechnung. */
  int timeStamp() const;

  /** Ausgabe des gesamten Grids fuer Gnuplot
      die Ausgabe mu in eine Datei umgeleitet werden */
  void getGridPoints( int gridType = 0 ) const;

  /** liefert Feld mit den Zeilensummen */
  double* getLineSum( int gridType = 0);

  /** liefert Feld mit den Spaltensummen */
  double* getColumnSum( int gridType = 0);

   /** berechnet RGB - Werte des Feldes, wobei die Werte auf 255 bzw. -255 normiert werden
       (ein Grnwert von 255 heisst also nicht, dasss der Wert dort 255 betrgt, sondern dass
       dieser insgesamt ber das Feld betrachtet den hchsten Wert darstellt
       *Rckgabe: Zeiger auf das Feld */
  unsigned long *getGridRGB( int gridType = 0);
  /** hier wird nur ein Zeiger auf die Agenten gesetzt
   *players: Zeiger auf ein "ToGrid" - struct */

  void init( ToGrid *players );
  /** berechnet RGB - Werte des Feldes, wobei die Werte auf 255 bzw. -255 normiert werden
      (ein Grnwert von 255 heisst also nicht, dasss der Wert dort 255 betrgt, sondern dass
      dieser insgesamt ber das Feld betrachtet den hchsten Wert darstellt
      *Rckgabe: Zeiger auf das Feld */
  unsigned long *calculateLoggerRGB( int gridType = 0);

protected:
  double theGrid[GRID_TYPE_COUNT][GRID_RANGE_X + 1][GRID_RANGE_Y + 1];  
  double f[4][2 * GRID_RANGE_X + 1][2 * GRID_RANGE_Y + 1];

  int theTimeStamp, thePause;

  /** addiert Funktion auf das Grid
      *Vector: Position des Spielers (dieser wurde mit rePos bereits korrekt verschoben
      *weight: Gewichtung der Funktion
      *fType: welche Funtkion soll benutzt werden
  */
  void addValues( Vector pos, double weight, int fType, int gridType = 0 );

  /** diese Funktion hebt das durch die uebergebenen Koordinaten
      definierte Rechteck um den Wert value an
  */
  void liftArea( double lowerXPos, double lowerYPos,
                 double upperXPos, double upperYPos,
                 double value, int gridType = 0 );

  /** addiert Werte zum Spielfeldrand
      ACHTUNG: das gegnerische Tor wird ausgelassen
      Parameter 1: Wert der auf die Seitenauslinien und Grundlinien addiert wird
      Parameter 2: Wert der auf die Linien (1m zur Spielfeldmitte) neben den Seitenauslinien
                   und Grundlinien addiert wird
  */
  void addValuesToFieldSides( double onLine, double nextToLine, int gridType = 0 );

  /** setzt Spieler auf Kantenkreuzung */
  Point rePos( Point thePos );

  /** Hilfsfunktionen */
  int round( double val );
  Point makePoint( int x, int y );

  /** functions */
  /** abstossende Funktion fr die Mitspieler */
  double f0( double x, double y );
  /** falls man Angreifer ist, werden durch diese Funktion die Gegner
      abgestossen */
  double f1( double x, double y );
  /** nur zum Test */
  double f2( double x, double y );
  /** Ballfunktion (fllt schnell ab aber sehr starker Anstieg)*/
  double f3( double x, double y );

  /** logging functions */
  /** addiert Funktion auf das Grid
      *Vector: Position des Spielers (dieser wurde mit rePos bereits korrekt verschoben
      *weight: Gewichtung der Funktion
      *fType: welche Funtkion soll benutzt werden */
  void addValuesLogger( Point pos, double weight, int fType, int gridType = 0 );

  /** geht alle 23 Spieler durch und legt entsprechend Funktionen auf des Grid
      *WorldModel: struct mit allen Agenten
      *number: Nummer des Spielers */
  void calculateValuesLogger( ToGrid* WorldModel, int number, int gridType = 0 );
  ToGrid *playersLogger;
};


///////////////////
// Hilfsfunktionen

inline int Grid::round( double val ) {
	int d = (int)val;
	val = val - d;
	return (val>=0.5)? d+1:d;
}

inline Point Grid::makePoint (int x, int y) {
  Point p;

  p.x = x; p.y = y;
  return (p);
}

inline void Grid::setTimeStamp(){
  theTimeStamp = globalWorldModel->simStep();
}

inline int Grid::timeStamp() const{
  return theTimeStamp;
}

#endif
