#ifndef _INTERCEPT_ANALYZER_H_
#define _INTERCEPT_ANALYZER_H_

#include "../intentions/targets/Target.h"
#include "../intentions/skills/Skills.h"
#include "../worldmodel/Situation.h"


class InterceptAnalyzer {
  public:
    InterceptAnalyzer();
    void calculateValues();

    int getMyStepsToBall();
    int getOurGoalieStepsToBall();
    int getNearestEnemyStepsToBall();
    int getNearestTeammateStepsToBall();
    int get2ndNearestTeammateStepsToBall();

    const Vector& getMyInterceptPosition();
    const Vector& getOurGoalieInterceptPosition();
    const Vector& getNearestEnemyInterceptPosition();
    const Vector& getNearestTeammateInterceptPosition();
    const Vector& get2ndNearestTeammateInterceptPosition();

    const bool getGoalieInterceptRunBehaviour();
    
  protected:

  private:

    int myStepsToBall;
    int ourGoalieStepsToBall;
    int nearestEnemyStepsToBall;
    int nearestTeammateStepsToBall;
    int secondNearestTeammateStepsToBall;

    Vector myInterceptPosition;
    Vector ourGoalieInterceptPosition; 
    Vector nearestEnemyInterceptPosition;
    Vector nearestTeammateInterceptPosition;
    Vector secondNearestTeammateInterceptPosition;

    bool goalieInterceptRunBehaviour;

    int calculatedSteps;
    Vector calculatedTarget;

    void calculateMinInterceptTarget(const ModelledPlayer& player, double targetRadius = -1.0);
    void calculateOptInterceptTarget(const ModelledPlayer& player, double targetRadius = -1.0);
    void calculateGoalieInterceptTarget(const ModelledPlayer& player, double targetRadius = -1.0);

    int simStepOfLastCalc;

};

inline int InterceptAnalyzer::getMyStepsToBall() {
  return myStepsToBall;
}

inline int InterceptAnalyzer::getOurGoalieStepsToBall() {
  return ourGoalieStepsToBall;
}

inline int InterceptAnalyzer::getNearestEnemyStepsToBall() {
  return nearestEnemyStepsToBall;
}

inline int InterceptAnalyzer::getNearestTeammateStepsToBall() {
  return nearestTeammateStepsToBall;
}

inline int InterceptAnalyzer::get2ndNearestTeammateStepsToBall() {
  return secondNearestTeammateStepsToBall;
}

inline const Vector& InterceptAnalyzer::getMyInterceptPosition() {
  return myInterceptPosition;
}

inline const Vector& InterceptAnalyzer::getOurGoalieInterceptPosition() {
  return ourGoalieInterceptPosition;
}

inline const Vector& InterceptAnalyzer::getNearestEnemyInterceptPosition() {
  return nearestEnemyInterceptPosition;
}

inline const Vector& InterceptAnalyzer::getNearestTeammateInterceptPosition() {
  return nearestTeammateInterceptPosition;
}

inline const Vector& InterceptAnalyzer::get2ndNearestTeammateInterceptPosition() {
  return secondNearestTeammateInterceptPosition;
}

inline const bool InterceptAnalyzer::getGoalieInterceptRunBehaviour() {
  return goalieInterceptRunBehaviour;
}


#endif
