#include "PositionAnalyzer.h"

#include "Analyzer.h"

#include "../basic/Timer.h"
#include "../worldmodel/WorldModelInterface.h"
#include "../worldmodel/Situation.h"

PositionAnalyzer::PositionAnalyzer()
{
}

const Vector& PositionAnalyzer::getPositionTarget()
{
  return positionTarget;
}

const Vector& PositionAnalyzer::getGoaliePositionTarget()
{
  return goaliePositionTarget;
}

const Vector& PositionAnalyzer::getGoaliePositionTurnVector()
{
  return goaliePositionTurnVector;
}


void PositionAnalyzer::calculateGoalieValues(){
//cout <<	globalWorldModel->simStep() << "\nPositionAnalyzer::calculateGoalieValues\n";
 
  const ModelledBall& ball = globalWorldModel->ball();  
  const ModelledPlayer& mySelf = globalWorldModel->player();  
  Vector topToBall = ball.pos() - globalWorldModel->field().ourGoalTop();
  Vector bottomToBall = ball.pos() - globalWorldModel->field().ourGoalBottom();
  double dir = normalizeAngle( 0.5 * ( deviationBetween(0.0, topToBall.angle()) +  deviationBetween(0.0, bottomToBall.angle() ) ) );
//cout << "  Winkelhalbierende:  Winkel: " << degFromRadiant(dir);
  double intersectY = ball.pos().y()
                      + ( globalWorldModel->field().ourGoal().x() - ball.pos().x() )
                         * tan( dir );
//cout << "  von " << intersectY -  globalWorldModel->field().ourGoalBottom().y() << " m\n";
  double dist = 0.3 * (ball.pos() - globalWorldModel->field().ourGoal()).length();
  Vector target = Vector( globalWorldModel->field().ourGoal().x(), intersectY, CARTESIAN )
                  + Vector( dist, dir, POLAR, RADIAN );
//cout << "  tempTarget : " << target << endl;

	double maxX = -49.0;
	double minX = -52.0;

	if( target.x() < minX ) {
	  target.setX( minX );
  	target.setY( ball.pos().y() + ( minX - ball.pos().x() ) * tan( dir ) );		
  }
	if( target.x() > maxX ) {
	  target.setX( maxX );
  	target.setY( ball.pos().y() + ( maxX - ball.pos().x() ) * tan( dir ) );		
  }

	double yOffset =  0.8 + fabs( -51.5 - target.x() ) * 0.4;
	double maxY = globalWorldModel->field().ourGoalTop().y() - yOffset;
	double minY = globalWorldModel->field().ourGoalBottom().y() + yOffset;

	if( target.y() < minY ) {
	  target.setX( target.x() + ( minY - target.y() ) / tan( dir ) );
  	target.setY( minY );		
  }
	if( target.y() > maxY ) {
	  target.setX( target.x() + ( maxY - target.y() ) / tan( dir ) );
  	target.setY( maxY );		
  }


  // Target mglichst stabil halten
  if( (mySelf.pos() - target).length() < 0.20 ) {
//cout <<	"  Target = " << mySelf.pos() << " --- OLD\n";
    goaliePositionTarget =  mySelf.pos();
  }
  else {
//cout <<	"Target = " << target << endl;
    goaliePositionTarget = target;
  }

  Vector turnVector = Vector(mySelf.pos().x(), 
                             0.5 * globalWorldModel->field().fieldWidth() * ( mySelf.pos().y() < ball.pos().y() ? 1.0 : -1.0 ), 
                             CARTESIAN );
  goaliePositionTurnVector = turnVector;
//cout <<	globalWorldModel->simStep() << " end of PositionAnalyzer::calculateGoalieValues\n";
}




void PositionAnalyzer::calculateValues(){
  INFO( Log::CONDITIONS,  1,  idString() , globalLog->infoStream
    << "PositionAnalyzer::calculateValues started at " << globalTimer->getAgentTime() << endl );

  const ModelledPlayer& myself = globalWorldModel->player();
  const ModelledBall& ball = globalWorldModel->ball();
  const Situation& situation = globalWorldModel->situation();

  // es wird hier die offensive Homeposition verwendet
//  Vector homePosition = globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, OUR_TEAM_ID);
  Vector homePosition = globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, THEIR_TEAM_ID);

  const double homeRange = 15.0;
  Vector offsetToBallPos = ball.pos() + Vector( -10.0, 0.0, CARTESIAN );

  const double minX = -37.0;
  const double maxX = 12.0;
  const double minY = -16.0;
  const double maxY = 16.0;

  if( offsetToBallPos.x() < minX ) offsetToBallPos.setX( minX );
  if( offsetToBallPos.x() > maxX ) offsetToBallPos.setX( maxX );
  if( offsetToBallPos.y() < minY ) offsetToBallPos.setY( minY );
  if( offsetToBallPos.y() > maxY ) offsetToBallPos.setY( maxY );

  // Sind wir im Angriff oder in der Verteidigung ?
  bool offense = YES;

  // HACK - in Abhngigkeit von der HomePosition wird Rolle definiert
  if( homePosition.x() < 0.0 ) offense = NO;    // Verteidiger
  else if( (homePosition.x() < 20.0) && (ball.pos().x() < -5.0) ) offense = NO;  // Mittelfeld
  // sonst Strmer --> immer offensiv

  if( ( ( (globalWorldModel->gameState() == WorldModelInterface::FREE_KICK)
       || (globalWorldModel->gameState() == WorldModelInterface::GOAL_KICK) )
     && ( globalWorldModel->initiative() == THEIR_TEAM_ID ) ) )
  {
    offense = NO;
  }

  homePosition = homePosition + offsetToBallPos;


  Vector homeForce = Vector( 0.0, 0.0, CARTESIAN );

  // Homeforce berechnen
  if( myself.distTo( homePosition ) > homeRange ){
    homeForce = Vector( 35.0 + myself.distTo(homePosition),
                        (homePosition - myself.pos()).angle(), POLAR, RADIAN );
  }
  else{
    homeForce = Vector( myself.distTo(homePosition) * 2.0,
                        (homePosition - myself.pos()).angle(), POLAR, RADIAN );
  }

  Vector ourTeamForce = Vector( 0.0, 0.0, CARTESIAN );
  Vector opponentTeamForce = Vector( 0.0, 0.0, CARTESIAN );

  // TeammateForce und EnemyForce
  for( int i = 0; i < situation.playerCount(); ++i ) {
    const ModelledPlayer& player = situation.playerWithIndex(i);

    // undefinierte Spieler haben keinen Einfluss
    if( (player.team() == OUR_TEAM_ID)
     && (myself.distTo(player.pos()) < 12.0)
     && (player.id() != globalWorldModel->id()) ) {
      ourTeamForce       += Vector( 10 * myself.distTo(player.pos()) - 150.0,
                                   (player.pos() - myself.pos()).angle(), POLAR, RADIAN );

    }
    else if( player.team() == THEIR_TEAM_ID ) {
      if( offense == YES ) { // Offensives Verhalten, Freilaufen
        if( myself.distTo(player.pos()) < 10.0 ){
          // zwei Teilkrfte berechnen -> Abstoung und Attraktionspunkt
          opponentTeamForce  += Vector(  myself.distTo(player.pos()) - 10.0,
                                       (player.pos() - myself.pos()).angle(), POLAR, RADIAN );
	}
	if( myself.distTo(player.pos()) < 13.0 ){
          Vector attractionPoint = player.pos() + Vector( 8.0, (globalWorldModel->field().enemyGoal() - player.pos()).angle(), POLAR, RADIAN);
          opponentTeamForce  += Vector(  myself.distTo(attractionPoint),
                                       (attractionPoint - myself.pos()).angle(), POLAR, RADIAN );
        }
      }
      else { // Defensiv, Manndeckung
      	// in Abhngigkeit von der Entfernung des Gegners von der Torauslinie (Gefhrlichkeit)
      	// wird der Abstand der Manndeckung bestimmt
	//      	double enDist = globalWorldModel->field().ourGoal().x() - situation.playerWithIndex(i).pos().x();
	//      	double enemyAttraction = 0.2 * enDist - 2.0;
	//      	enemyAttraction = max( enemyAttraction , 1.0 );
        if( myself.distTo(situation.playerWithIndex(i).pos()) < 6.0 ){
          // zwei Teilkrfte berechnen -> Abstoung und Attraktionspunkt
          opponentTeamForce  += Vector( 5* myself.distTo(player.pos()) - 50.0,
                                       (player.pos() - myself.pos()).angle(), POLAR, RADIAN );
	}
	if( myself.distTo(situation.playerWithIndex(i).pos()) < homeRange ){
          Vector attractionPoint = player.pos() + Vector( min(.5,max(7,(48+player.pos().x())/3)), (globalWorldModel->field().ourGoal() - player.pos()).angle(), POLAR, RADIAN);
          opponentTeamForce  += Vector( myself.distTo(attractionPoint),
                                       (attractionPoint - myself.pos()).angle(), POLAR, RADIAN );
        }
      }
    }
  }

  Vector target = homeForce + ourTeamForce + opponentTeamForce;

  // Gegen einen Schwellwert prfen (Stamina sparen!)
  if( target.length() < 1.0 ){
    target = Vector( 0.0, 0.0 );
  }

  // Absoluten Vektor bilden
  target += myself.pos();

  // Abseitslinie vermeiden
  if( target.x() > situation.offsideLineX() ){
    target.setX( situation.offsideLineX() - 0.5 );
  }

  if( target.y() > 30.0 ){
    target.setY( 30.0 );
  }
  if( target.y() < -30.0 ){
    target.setY( -30.0 );
  }
  if( target.x() > 48.0 ){
    target.setX( 48.0 );
  }
  if( target.x() < -42.0 ){
    target.setX( -42.0 );
  }

  // das neue Target wird gesetzt
  setPositionTarget( target );	





/*

  INFO( Log::CONDITIONS,  1,  idString() , globalLog->infoStream
    << "PositionAnalyzer::calculateValues started at " << globalTimer->getAgentTime() << endl );

//int duration = globalTimer->getAgentTime();

  const ModelledPlayer& myself = globalWorldModel->player();
  const ModelledBall& ball = globalWorldModel->ball();
  const Situation& situation = globalWorldModel->situation();

  // es wird hier die offensive Homeposition verwendet
  Vector homePosition = globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, OUR_TEAM_ID);

  const double homeRange = 15.0;
  Vector offsetToBallPos = ball.pos() + Vector( -10.0, 0.0, CARTESIAN );

  const double minX = -37.0;
  const double maxX = 12.0;
  const double minY = -16.0;
  const double maxY = 16.0;

  if( offsetToBallPos.x() < minX ) offsetToBallPos.setX( minX );
  if( offsetToBallPos.x() > maxX ) offsetToBallPos.setX( maxX );
  if( offsetToBallPos.y() < minY ) offsetToBallPos.setY( minY );
  if( offsetToBallPos.y() > maxY ) offsetToBallPos.setY( maxY );

  // Sind wir im Angriff oder in der Verteidigung ?
  bool offense = YES;

  // HACK - in Abhngigkeit von der HomePosition wird Rolle definiert
  if( homePosition.x() < 0.0 ) offense = NO;    // Verteidiger
  else if( (homePosition.x() < 20.0) && (ball.pos().x() < -5.0) ) offense = NO;  // Mittelfeld
  // sonst Strmer --> immer offensiv

  if( ( ( (globalWorldModel->gameState() == WorldModelInterface::FREE_KICK)
       || (globalWorldModel->gameState() == WorldModelInterface::GOAL_KICK) )
     && ( globalWorldModel->initiative() == THEIR_TEAM_ID ) ) )
  {
    offense = NO;
  }
  
  homePosition = homePosition + offsetToBallPos;


  Vector homeForce = Vector( 0.0, 0.0, CARTESIAN );
  
  // Homeforce berechnen
  if( myself.distTo( homePosition ) > homeRange ){
    homeForce = Vector( 35.0 + myself.distTo(homePosition),
                        (homePosition - myself.pos()).angle(), POLAR, RADIAN );
  }
  else{
    homeForce = Vector( myself.distTo(homePosition) / 2.0,
                        (homePosition - myself.pos()).angle(), POLAR, RADIAN );
  }
  
  Vector ourTeamForce = Vector( 0.0, 0.0, CARTESIAN );
  Vector opponentTeamForce = Vector( 0.0, 0.0, CARTESIAN );

  // TeammateForce und EnemyForce
  for( int i = 0; i < situation.playerCount(); ++i ) {
    const ModelledPlayer& player = situation.playerWithIndex(i);
    
    // undefinierte Spieler haben keinen Einfluss
    if( (player.team() == OUR_TEAM_ID)
     && (myself.distTo(player.pos()) < 7.0)
     && (player.id() != globalWorldModel->id()) ) {
      ourTeamForce       += Vector( 5 * myself.distTo(player.pos()) - 50.0,
                                   (player.pos() - myself.pos()).angle(), POLAR, RADIAN );
                                   
    }
    else if( player.team() == THEIR_TEAM_ID ) {
      if( offense == YES ) { // Offensives Verhalten, Freilaufen
        if( myself.distTo(player.pos()) < 7.0 ){
          // zwei Teilkrfte berechnen -> Abstoung und Attraktionspunkt
          opponentTeamForce  += Vector( 5 * myself.distTo(player.pos()) - 50.0,
                                       (player.pos() - myself.pos()).angle(), POLAR, RADIAN );
          Vector attractionPoint = player.pos() + Vector( 7.0, (globalWorldModel->field().enemyGoal() - player.pos()).angle(), POLAR, RADIAN);   
          opponentTeamForce  += Vector( 3 * myself.distTo(attractionPoint),
                                       (attractionPoint - myself.pos()).angle(), POLAR, RADIAN );
        }    
      }
      else { // Defensiv, Manndeckung
      	// in Abhngigkeit von der Entfernung des Gegners von der Torauslinie (Gefhrlichkeit)
      	// wird der Abstand der Manndeckung bestimmt
      	double enDist = globalWorldModel->field().ourGoal().x() - situation.playerWithIndex(i).pos().x();
      	double enemyAttraction = 0.2 * enDist - 2.0;
      	enemyAttraction = max( enemyAttraction , 1.0 );
        if( myself.distTo(situation.playerWithIndex(i).pos()) < homeRange ){
          // zwei Teilkrfte berechnen -> Abstoung und Attraktionspunkt
          opponentTeamForce  += Vector( 2 * myself.distTo(player.pos()) - 20.0,
                                       (player.pos() - myself.pos()).angle(), POLAR, RADIAN );
          Vector attractionPoint = player.pos() + Vector( enemyAttraction, (globalWorldModel->field().ourGoal() - player.pos()).angle(), POLAR, RADIAN);   
          opponentTeamForce  += Vector( 5 * myself.distTo(attractionPoint),
                                       (attractionPoint - myself.pos()).angle(), POLAR, RADIAN );
        }                                      
      }
    }
  }

  Vector target = homeForce + ourTeamForce + opponentTeamForce;

  // Gegen einen Schwellwert prfen (Stamina sparen!)
  if( target.length() < 1.0 ){
    target = Vector( 0.0, 0.0 );
  }

  // Absoluten Vektor bilden
  target += myself.pos();

  // Abseitslinie vermeiden
  if( target.x() > situation.offsideLineX() ){
    target.setX( situation.offsideLineX() - 0.5 );
  }

  if( target.y() > 30.0 ){
    target.setY( 30.0 );
  }
  if( target.y() < -30.0 ){
    target.setY( -30.0 );
  }
  if( target.x() > 48.0 ){
    target.setX( 48.0 );
  }
  if( target.x() < -48.0 ){
    target.setX( -48.0 );
  }

  // das neue Target wird gesetzt
  setPositionTarget( target );	

*/

/*
  const ModelledPlayer& myself = globalWorldModel->player();
  const Situation& situation = globalWorldModel->situation();

  Vector ourTeamForce = Vector( 0.0, 0.0 );
  Vector opponentTeamForce = Vector( 0.0, 0.0 );

  // Berechnen der einzelnen Kraftvektoren des Potentialfeldes (erweiterbar)

  Vector homeForce = Vector( min( 45.0, 0.001 * pow( myself.distTo(globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, THEIR_TEAM_ID)) , 3.0) ),
                            (globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, THEIR_TEAM_ID) - myself.pos()).angle(), POLAR, RADIAN );
//cerr << "HOMEFORCE = " << homeForce << endl;
  Vector ballForce = Vector( min( 60.0, 200.0 * pow( myself.distTo(globalWorldModel->ball().pos()) , -0.6 ) ),
                            (globalWorldModel->ball().pos() - myself.pos()).angle(), POLAR, RADIAN );
                           
//cerr << "BALLFORCE = " << ballForce << endl;
  for( int i = 0; i < situation.playerCount(); ++i ) {
    if( (situation.playerWithIndex(i).team() == OUR_TEAM_ID) && (situation.playerWithIndex(i).id() != globalWorldModel->id()) ) {
      ourTeamForce       += Vector( max( -50.0, -200.0 * pow( myself.distTo(situation.playerWithIndex(i).pos()), -.7 ) +
                                   10.0 * pow( myself.distTo(situation.playerWithIndex(i).pos()), .1 ) ),
                                   (situation.playerWithIndex(i).pos() - myself.pos()).angle(), POLAR, RADIAN );
                                   
    }
    else if( situation.playerWithIndex(i).team() == THEIR_TEAM_ID ) {

      if( myself.pos().x() > - 10.0 ) { // Offensives Verhalten, Freilaufen
        opponentTeamForce  += Vector( max( -50.0, -100.0 / myself.distTo(situation.playerWithIndex(i).pos()) ),
                                    ( situation.playerWithIndex(i).pos() - myself.pos()).angle(), POLAR, RADIAN );
                                      
    
      }
      else { // Defensiv, Manndeckung
        opponentTeamForce  += Vector( min( 50.0, 150.0 / myself.distTo(
                                      situation.playerWithIndex(i).pos() - Vector( -4.0, 0.0)) ),
                                     (situation.playerWithIndex(i).pos() - myself.pos()).angle(), POLAR, RADIAN );
                                      
      }
    }
  }
//cerr << "OURTEAMFORCE = " << ourTeamForce << endl;
//cerr << "OPPONENTTEAMFORCE = " << opponentTeamForce << endl;
  // Aufsummieren der Komponenten
  Vector target = homeForce + ballForce + ourTeamForce + opponentTeamForce;

  // Gegen einen Schwellwert prfen (Stamina sparen!)
  if( target.length() < 1.0 ){
    target = Vector( 0.0, 0.0 );
  }

  // Absoluten Vektor bilden
  target += myself.pos();

  // Abseitslinie vermeiden
  if( target.x() > situation.offsideLineX() ){
    target.setX( situation.offsideLineX() - 0.5 );
  }

  if( target.y() > 30.0 ){
    target.setY( 30.0 );
  }
  if( target.y() < -30.0 ){
    target.setY( -30.0 );
  }
  if( target.x() > 48.0 ){
    target.setX( 48.0 );
  }
  if( target.x() < -48.0 ){
    target.setX( -48.0 );
  }


  if( globalWorldModel->field().isInsideField( target ) == NO ){
//    cerr << "desired position is outside of field!\n";
  }
  
  setPositionTarget( target );	
  
  if ( ((globalWorldModel->simStep() % 3)==0) && (globalWorldModel->id()== 0) ) { // 421 fr Ausgabe bei einem Player
  	// fuer Debugzwecke die Positionierungsmatrix ausgeben
  	
  	// fuer gnuplot
  	
//  	cout <<"set view 15, 220\n splot '-' with lines\n";
  	
  	for (double xpos=-50.0; xpos<50.0; xpos+=2.0 ) {
  		for (double ypos=-30.0; ypos<30.0; ypos+=2.0 ) {
  			
  			Vector ourTeamForce = Vector( 0.0, 0.0 );
  			Vector opponentTeamForce = Vector( 0.0, 0.0 );
			Vector mypos = Vector (xpos,ypos);

 			// Berechnen der einzelnen Kraftvektoren des Potentialfeldes (erweiterbar)

			  Vector homeForce = Vector( min( 45.0, 0.001 * pow( (mypos-globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, OUR_TEAM_ID)).length() , 3.0) ),
			                            (globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, OUR_TEAM_ID) - mypos).angle(), POLAR, RADIAN );
			//cerr << "HOMEFORCE = " << homeForce << endl;
			  Vector ballForce = Vector( min( 60.0, 200.0 * pow( (mypos -globalWorldModel->ball().pos()).length() , -0.6 ) ),
			                            (globalWorldModel->ball().pos() - mypos).angle(), POLAR, RADIAN );
			                           
			//cerr << "BALLFORCE = " << ballForce << endl;
			  for( int i = 0; i < situation.playerCount(); ++i ) {
			    if( (situation.playerWithIndex(i).team() == OUR_TEAM_ID) && (situation.playerWithIndex(i).id() != globalWorldModel->id()) ) {
			      ourTeamForce       += Vector( max( -50.0, -200.0 * pow( (mypos -situation.playerWithIndex(i).pos()).length(), -.7 ) +
			                                   10.0 * pow( (mypos -situation.playerWithIndex(i).pos()).length(), .1 ) ),
			                                   (situation.playerWithIndex(i).pos() - mypos).angle(), POLAR, RADIAN );
			                                   
			    }
			    else if( situation.playerWithIndex(i).team() == THEIR_TEAM_ID ) {
			
			      if( mypos.x() > - 10.0 ) { // Offensives Verhalten, Freilaufen
			        opponentTeamForce  += Vector( max( -50.0, -100.0 / (mypos-situation.playerWithIndex(i).pos()).length() ),
			                                    ( situation.playerWithIndex(i).pos() - mypos).angle(), POLAR, RADIAN );
			                                      
			    
			      }
			      else { // Defensiv, Manndeckung
			        opponentTeamForce  += Vector( min( 50.0, 150.0 / (mypos -
			                                      situation.playerWithIndex(i).pos() - Vector( -4.0, 0.0)).length() ),
			                                     (situation.playerWithIndex(i).pos() - mypos).angle(), POLAR, RADIAN );
			                                      
			      }
			    }
			  }
			//cerr << "OURTEAMFORCE = " << ourTeamForce << endl;
			//cerr << "OPPONENTTEAMFORCE = " << opponentTeamForce << endl;
			  // Aufsummieren der Komponenten
			  Vector target = homeForce + ballForce + ourTeamForce + opponentTeamForce;
			
			  // Gegen einen Schwellwert prfen (Stamina sparen!)
			  if( target.length() < 1.0 ){
			    target = Vector( 0.0, 0.0 );
			  }
			
			  // Absoluten Vektor bilden
			  target += mypos;
			
			  // Abseitslinie vermeiden
			  if( target.x() > situation.offsideLineX() ){
			    target.setX( situation.offsideLineX() - 0.5 );
			  }
			
			  if( target.y() > 30.0 ){
			    target.setY( 30.0 );
			  }
			  if( target.y() < -30.0 ){
			    target.setY( -30.0 );
			  }
			  if( target.x() > 48.0 ){
			    target.setX( 48.0 );
			  }
			  if( target.x() < -48.0 ){
			    target.setX( -48.0 );
			  }
			  
//			  cout <<target.length()<<endl;
  		}
//  		cout <<endl;
  	}
//  	cout<<endl;
//  	cout<<"e\n"<<flush;
  }
*/  			
  
//cerr << "PositionAnalyzer : " << (globalTimer->getAgentTime() - duration) << "  ms\n";
 

  INFO( Log::CONDITIONS,  1,  idString() , globalLog->infoStream
    << "Position Target: " << positionTarget << endl );
  INFO( Log::CONDITIONS,  1,  idString() , globalLog->infoStream
    << "PositionAnalyzer::calculateValues ended at " << globalTimer->getAgentTime() << endl );
}


