#ifndef _STRING_H_
#define _STRING_H_

#include <string>
#include <iostream.h>
#include <stream.h>
#include "Common.h"
#include "Exception.h"

/**
  Die Klasse verwaltet einen Ath98String.


  {\small {\bf Quelltext}

  [\URL[Header]{src/String.h}] | [\URL[Implementierung]{src/String.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Ath98String.h,v $
  *   $Revision: 1.1.1.1 $
  *   $Date: 2001/07/31 22:23:08 $
  *   $Author: rami $
  *
  **************************************************
  \end{verbatim} } */



class Ath98String
{
public:
  /** @name Fehlerklassen */
  //@{
    /** Allgemeine Fehlerklasse */ 
    class Ath98StringException: public Exception {
    public:
      Ath98StringException(const Ath98String& aDescription )
        : Exception( aDescription ) {}
    };

    /** Umwandlungsfehlerklasse.
      Sie wird bei fehlschlagenden Umwandlungen, z.b. in einen int-Wert
      erzeugt. */
    class ConversionException: public Ath98StringException {
    public:
      ConversionException(const Ath98String& aDescription )
        : Ath98StringException( aDescription ) {}
    };

  //@}

  /** @name Ath98Strings anlegen und zerst\"oren */
  //@{
    /** Neuer Ath98String aus einer Zeichenkette.
        @param newValue Der Ath98String wird mit dieser Zeichenkette initialisiert. */
    Ath98String (const char* newValue="");

    /** Neuer Ath98String aus einem int-Wert.
      @param newValue Der Ath98String wird mit einer Zeichenkette, die diesen
      int-Wert darstellt, initialisiert. */
    Ath98String (const int newValue);

    /** Neuer Ath98String aus einem long-Wert.
      @param newValue Der Ath98String wird mit einer Zeichenkette, die diesen
        long-Wert darstellt, initialisiert. */
    Ath98String (const long newValue);

    /** Neuer Ath98String aus einem double-Wert.
      @param newValue Der Ath98String wird mit einer Zeichenkette, die diesen
        double-Wert darstellt, initialisiert. */
    Ath98String (const double newValue);

    /** Neuer Ath98String aus einem bool-Wert.
      @param newValue Der Ath98String wird mit einer Zeichenkette, die diesen
        double-Wert darstellt, initialisiert. */
    Ath98String( const bool newValue );

    /** Der Copy-Konstruktor
        @param s Der Ath98String wird mit den Werten des Ath98Strings s initialisiert. */
    Ath98String (const Ath98String& s);

    /** Legt eine Kopie des Objekts auf dem Heap an.
      @see StorableObject::copy
      @return Zeiger auf die neu allozierte Kopie */
    Ath98String* copy() const;

    /** Destruktor.
      Der Belegte Speicher wird wieder freigegeben. */
    ~Ath98String ();

    //@}


  /** @name Anfragen und Tests  */
  //@{
    /** Test auf Gleichheit */
    friend bool operator== (const Ath98String&, const Ath98String&);

    /** Test auf Ungleichheit */
    friend bool operator!= (const Ath98String&, const Ath98String&);

    /** Test, ob der Ath98String mit dem angegebenen Ath98String anfaengt.
      @param string wird mit dem Anfang des Ath98Strings verglichen.
      @return YES, wenn der Anfang des Ath98Strings mit dem Argument uebereinstimmt,
        NO sonst. */
    bool beginsWith( const Ath98String& string ) const;

    /** Liefert die Laenge des Ath98Strings. Das abschlie\3ende 0-Byte wird nicht
      mitgez\"ahlt.
      @return Anzahl der Zeichen im Ath98String. */
    int length() const;
  //@}


  /** @name Inhalt \"andern */
  //@{
    /** Einen zweiten Ath98String ang\"angen */
    friend Ath98String operator+ (const Ath98String&, const Ath98String&);

    /** Zuweisung eines neuen Wertes. */
    const Ath98String& operator= (const Ath98String& s);

  //@}


  /** @name Teilstrings erfragen */
  //@{
    /** Liefert einen Teilstring, links beginnend.
      @param charCount Laenge des Teilstrings. Ist die Laenge kleiner als 0,
        wird ein leerer Ath98String zurueckgegeben. Ist die Laenge groesser als
        die Ath98Stringlaenge, wird der gesamte Ath98String zurueckgegeben.
      @return Anfangsstueck des Ath98Strings mit der gewaehlten Laenge */
    Ath98String leftString( int charCount ) const;
    /** Liefert einen Teilstring, rechts endend.
      @param charCount Laenge des Teilstrings. Ist die Laenge kleiner als 0,
        wird ein leerer Ath98String zurueckgegeben. Ist die Laenge groesser als 
        die Ath98Stringlaenge, wird der gesamte Ath98String zurueckgegeben.
      @return Endstueck des Ath98Strings mit der gewaehlten Laenge */
    Ath98String rightString( int charCount ) const;
    /** Liefert einen Teilstring.
      @param leftIndex Index bei dem der Teilstring anfangen soll. Das erste
        Zeichen des Ath98Strings hat den Index 0. Ist der Index kleiner als 0, wird
        bei 0 begonnen. Ist der Index groesser als die Ath98Stringlaenge, dann wird
        ein leerer Ath98String zur\"uckgegeben.
      @param charCount Laenge des Teilstrings. Ist die Laenge kleiner als 0,
        wird ein leerer Ath98String zurueckgegeben. Falls die Laenge ab dem Index
        die Laenge des Ath98Strings ueberschreitet, wird sie so gesetzt, das sie bis
        zum Ende des Ath98Strings reicht.
      @return Teilstueck des Ath98Strings in der gewaehlten Laenge, beginnend ab
        dem gewaehlten Index. */
    Ath98String subString( int leftIndex, int charCount = UNDEF_INT ) const;
  //@}

  
  /** @name Konvertierungen */
  //@{
    /** cString (also Char-Ptr zurueckgeben). 
      Der Ath98String darf nicht ge\"andert werden. Der gelieferte Pointer ist nur
      solange g\"ultig, wie das Ath98String-Objekt existiert, da\3 ihn geliefert
      hat. 
      @return Char-Zeiger auf den Ath98String */
    const char *cString() const;

    /** Liefert einen Int, indem es den eigenen Wert als Dezimalzahl
      interpretiert. 
      
      Hier sollte eigentlich eine Ath98String::ConversionException erzeugt werden,
      falls die Konvertierung fehlschl\"agt. Wer es schafft, atoi den
      entsprechenden Fehlerstatus zu entlocken, darf mir gerne die neue Routine
      zukommen lassen. 
      @return der ensprechende int-Wert. Schl\"agt die Konvertierung fehl, wird
        0 geliefert. */
    int asInt() const;

    /** Beschreibung als Ath98String fuer die Ausgabe auf einem Terminal. Ath98String
    selbst liefert nur sich selbst zur\"uck, die allgemeine ( Foo ... )
    Konvention verletzend. 
      @return Der Ath98String selbst */
    Ath98String toString() const;

  /** Ausgabe auf einen String-Stream */
    friend ostream& operator<< (ostream&, const Ath98String& );
  //@}



private:
  /// Char-Pointer, der auf den eigentlichen Inhalt zeigt.
  char *value;

  /** Laenge des Ath98Strings */
  int len;

  /** Konstruktor f\"ur uninitialisierten Ath98String der gegebenen L\"aenge.
    Der Konstruktor wird von \Ref{operator+} und \Ref{subString} verwendet.
      @param length Die gew\"unschte L\"ange
      @param dummyParameter, dessen Wert nicht interpretiert wird. Er ist
      notwendig, um den Konstruktor von Ath98String(int) zu unterscheiden. */
  Ath98String(int length, char dummyParameter);
};


inline const char *Ath98String::cString() const
{
  return value;
}

inline int Ath98String::length() const
{
  return len;
}

inline bool operator== (const Ath98String& s1, const Ath98String& s2)
{ 
  return ( ( s1.value == s2.value ) || strcmp( s1.value, s2.value ) == 0 );
}

inline bool operator!= (const Ath98String& s1, const Ath98String& s2)
{
  return !(s1==s2);
}

inline bool Ath98String::beginsWith( const Ath98String& string ) const 
{
  return (strncasecmp(cString(), string.cString(), string.length()) == 0);
}

inline ostream& operator<< (ostream& os, const Ath98String& s )
{
  os << s.cString()  ;
  return os;
}


#endif










