#ifndef _COMMON_H_
#define _COMMON_H_

/*
 * Aktueller Stand
 *
 *   $RCSfile: Common.h,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 2001/07/31 22:23:08 $
 *   $Author: rami $ 
 * */
#include <limits.h>
#include <float.h>
#include <math.h>
#include <stdlib.h>
#include <strstream>


/**@name Allgemeine gemeinsame Deklarationen */
//@{
  
  /** boolscher Wert true (empfohlen) */
#define YES 1
  /** boolscher Wert false (empfohlen) */
#define NO 0
  
  /** Undefinierter double-Wert */
#define UNDEF_DOUBLE DBL_MAX
  
  /** Undefinierter int-Wert */
#define UNDEF_INT INT_MAX
  
  /** Undefinierter long-Wert */
#define UNDEF_LONG LONG_MAX
  
  /** Der uebliche minimum-Operator */
#define min(a,b) (((a)<(b))?(a):(b))
  
  /** Der uebliche maximum-Operator */
#define max(a,b) (((a)>(b))?(a):(b))
  
  /** Der uebliche Absolutwert-Operator */
    /*#define abs(a) (((a)>=0)?(a):(-a))*/
  
  /** Ungueltiger double-Wert */
#define INVALID_DOUBLE DBL_MAX

 /** fuer einen HACK - Positionierung... **/
 enum RegionOnField { UPPER = 6, UPPER_MIDDLE = 4,
                       LOWER = 7, LOWER_MIDDLE = 5,
                       TO_UPPER = 1, TO_LOWER = 2,
                       REST = 3,
                       UPPER_EDGE = 9, LOWER_EDGE = 10,
                       ENEMY_GOAL = 8 };

  /** Ath98_PI */
  const double Ath98_PI = 3.141592654;

  /** Ath98_PI/2 */
  const double Ath98_PI_2 = Ath98_PI/2;

  /** Ath98_PI/180 */
  const double Ath98_PI_FACTOR = Ath98_PI/180.0;

  /** Ath98_PI/9 */
  const double Ath98_PI_20_DEGREE = Ath98_PI/9;

  /** Ath98_PI/18 */
  const double Ath98_PI_10_DEGREE = Ath98_PI/18;

  /** Ath98_PI/36 */
  const double Ath98_PI_5_DEGREE = Ath98_PI/36;

  /** Ath98_PI/60 */
  const double Ath98_PI_3_DEGREE = Ath98_PI/60;

  /** Ath98_PI/90 */
  const double Ath98_PI_2_DEGREE = Ath98_PI/90;

  /** Ungueltige Zeit */
  const int UNDEF_TIME = UNDEF_LONG;
  
  /** Unendliche Wartezeit */
  const int WAIT_FOREVER = UNDEF_INT;
  
  /** Maximale Groesse einer Nachricht vom SoccerServer */
  const int MAX_MESSAGE = 16384;
  
  /** defaultmaessige Rechengenauigkeit */
  const double DEFAULT_COMPARISON_ACCURACY = 0.001;
  
  /** Rechengenauigkeit */
  extern double globalComparisonAccuracy;

  /** Genauigkeit bei der Bestimmung der eigenen Position */
  const double TRIANGULATION_ACCURACY = 0.2;

  //@}

/**@name Objekt-Ids
  Objekt-Ids identifizieren in der Aussenwelt vorkommende Objekte wie Flaggen,
  Linien, Tore, Spielfeldseiten, Ball, Spieler (und auch
  etwas abstraktere Ideen wie Mannschaft) eindeutig.
  
  Zu jeder Gruppe von Ids, die gleichartige Objekte xxx beschreibt, gibt es
  eine MIN_xxx_ID und eine MAX_EXISTING_xxx_ID. Liegt eine Objekt-Id numerisch
  zwischen diesen beiden Konstanten (einschliesslich, also mit <= / >=
  vergleichen) , ist es eine gueltige Id fuer ein existierendes Objekt.
  Ausserdem gibt es noch allgemeinere Konstanten fuer die Art des Objekts
  (xxx_ID), fuer ein undefiniertes Objekt dieser Art (UNDEX_xxx_ID) und eine
  absolute obere Grenze MAX_xxx_ID. xxx_ID und UNDEX_xxx_ID sind garantiert <=
  MAX_xxx_ID. */
//@{
  ///
  const int MIN_ID = 0;
  ///
  const int UNDEF_ID = 699;
  ///
  const int MAX_ID = 699;
  
  /**@name Flaggen */
  //@{
    // die alten Flaggen;
    /// 
    const int MIN_FLAG_ID = 0;
    ///
    const int UPPER_LEFT_FLAG_ID = 0;
    ///
    const int UPPER_MIDDLE_FLAG_ID = 1;
    ///
    const int UPPER_RIGHT_FLAG_ID = 2;
    ///
    const int LOWER_LEFT_FLAG_ID = 3;
    ///
    const int LOWER_MIDDLE_FLAG_ID = 4;
    ///
    const int LOWER_RIGHT_FLAG_ID = 5;
    ///
    const int UPPER_LEFT_PENALTY_FLAG_ID = 6;
    ///
    const int MIDDLE_LEFT_PENALTY_FLAG_ID = 7;
    ///
    const int LOWER_LEFT_PENALTY_FLAG_ID = 8;
    ///
    const int UPPER_RIGHT_PENALTY_FLAG_ID = 9;
    ///
    const int MIDDLE_RIGHT_PENALTY_FLAG_ID = 10;
    ///
    const int LOWER_RIGHT_PENALTY_FLAG_ID = 11;
    ///
    const int LEFT_GOAL_CENTER_FLAG_ID = 12;
    ///
    const int RIGHT_GOAL_CENTER_FLAG_ID = 13;

    // die neuen Flaggen, innerhalb;
    ///
    const int CENTER_FLAG_ID = 14;
    ///
    const int LEFT_GOAL_TOP_FLAG_ID = 15;
    ///
    const int LEFT_GOAL_BOTTOM_FLAG_ID = 16;
    ///
    const int RIGHT_GOAL_TOP_FLAG_ID = 17;
    ///
    const int RIGHT_GOAL_BOTTOM_FLAG_ID = 18;
    
    // die neuen Flaggen, ausserhalb;
    ///
    const int OUTSIDE_TOP_LEFT_10_FLAG_ID = 19;
    ///
    const int OUTSIDE_TOP_LEFT_20_FLAG_ID = 20;
    ///
    const int OUTSIDE_TOP_LEFT_30_FLAG_ID = 21;
    ///
    const int OUTSIDE_TOP_LEFT_40_FLAG_ID = 22;
    ///
    const int OUTSIDE_TOP_LEFT_50_FLAG_ID = 23;
    ///
    const int OUTSIDE_TOP_RIGHT_10_FLAG_ID = 24;
    ///
    const int OUTSIDE_TOP_RIGHT_20_FLAG_ID = 25;
    ///
    const int OUTSIDE_TOP_RIGHT_30_FLAG_ID = 26;
    ///
    const int OUTSIDE_TOP_RIGHT_40_FLAG_ID = 27;
    ///
    const int OUTSIDE_TOP_RIGHT_50_FLAG_ID = 28;
    ///
    const int OUTSIDE_BOTTOM_LEFT_10_FLAG_ID = 29;
    ///
    const int OUTSIDE_BOTTOM_LEFT_20_FLAG_ID = 30;
    ///
    const int OUTSIDE_BOTTOM_LEFT_30_FLAG_ID = 31;
    ///
    const int OUTSIDE_BOTTOM_LEFT_40_FLAG_ID = 32;
    ///
    const int OUTSIDE_BOTTOM_LEFT_50_FLAG_ID = 33;
    ///
    const int OUTSIDE_BOTTOM_RIGHT_10_FLAG_ID = 34;
    ///
    const int OUTSIDE_BOTTOM_RIGHT_20_FLAG_ID = 35;
    ///
    const int OUTSIDE_BOTTOM_RIGHT_30_FLAG_ID = 36;
    ///
    const int OUTSIDE_BOTTOM_RIGHT_40_FLAG_ID = 37;
    ///
    const int OUTSIDE_BOTTOM_RIGHT_50_FLAG_ID = 38;
    ///
    const int OUTSIDE_LEFT_TOP_10_FLAG_ID = 39;
    ///
    const int OUTSIDE_LEFT_TOP_20_FLAG_ID = 40;
    ///
    const int OUTSIDE_LEFT_TOP_30_FLAG_ID = 41;
    ///
    const int OUTSIDE_LEFT_BOTTOM_10_FLAG_ID = 42;
    ///
    const int OUTSIDE_LEFT_BOTTOM_20_FLAG_ID = 43;
    ///
    const int OUTSIDE_LEFT_BOTTOM_30_FLAG_ID = 44;
    ///
    const int OUTSIDE_RIGHT_TOP_10_FLAG_ID = 45;
    ///
    const int OUTSIDE_RIGHT_TOP_20_FLAG_ID = 46;
    ///
    const int OUTSIDE_RIGHT_TOP_30_FLAG_ID = 47;
    ///
    const int OUTSIDE_RIGHT_BOTTOM_10_FLAG_ID = 48;
    ///
    const int OUTSIDE_RIGHT_BOTTOM_20_FLAG_ID = 49;
    ///
    const int OUTSIDE_RIGHT_BOTTOM_30_FLAG_ID = 50;
    ///
    const int OUTSIDE_TOP_CENTER_FLAG_ID = 51;
    ///
    const int OUTSIDE_BOTTOM_CENTER_FLAG_ID = 52;
    ///
    const int OUTSIDE_LEFT_CENTER_FLAG_ID = 53;
    ///
    const int OUTSIDE_RIGHT_CENTER_FLAG_ID = 54;
    
    ///
    const int MAX_EXISTING_FLAG_ID = 54;
    ///
    const int UNDEF_FLAG_ID = 55;
    ///
    const int FLAG_ID = 55;
    ///
    const int MAX_FLAG_ID = 55;
    ///
    const int FLAG_COUNT = 55;
    //@}

  /** @name Linien */
  //@{
    ///
    const int MIN_LINE_ID = 100;
    ///
    const int UPPER_LINE_ID = 100;
    ///
    const int LOWER_LINE_ID = 101;
    ///
    const int LEFT_LINE_ID = 102;
    ///
    const int RIGHT_LINE_ID = 103;
    ///
    const int MIDDLE_LINE_ID = 104;
    ///
    const int MAX_EXISTING_LINE_ID = 104;
    ///
    const int UNDEF_LINE_ID = 105;
    ///
    const int LINE_ID = 105;
    ///
    const int MAX_LINE_ID = 105;
    ///
    const int LINE_COUNT = 5;
    //@}

  /** @name Tore */
  //@{
    ///
    const int MIN_GOAL_ID = 200;
    ///
    const int LEFT_GOAL_ID = 200;
    ///
    const int RIGHT_GOAL_ID = 201;
    ///
    const int MAX_EXISTING_GOAL_ID = 201;
    ///
    const int UNDEF_GOAL_ID = 202;
    ///
    const int GOAL_ID = 202;
    ///
    const int MAX_GOAL_ID = 202;
    ///
    const int GOALCOUNT = 2;
    //@}
  
  /** @name Seiten */
  //@{ 
    
    // und als Seiten aufgefasst;
    ///
    const int MIN_SIDE_ID = 200;
    ///
    const int LEFT_SIDE_ID = 200;
    ///
    const int RIGHT_SIDE_ID = 201;
    ///
    const int MAX_EXISTING_SIDE_ID = 201;
    ///
    const int NO_SIDE_ID = 202;
    ///
    const int UNDEF_SIDE_ID = 203;
    ///
    const int MAX_SIDE_ID = 203;
    ///
    const int SIDE_COUNT = 2;
    //@}

  /** @name Teams */
  //@{ 
    
    // und als Teams aufgefasst;
    ///
    const int MIN_TEAM_ID = 210;
    ///
    const int OUR_TEAM_ID = 210;
    ///
    const int THEIR_TEAM_ID = 211;
    ///
    const int MAX_EXISTING_TEAM_ID = 211;
    ///
    const int NO_TEAM_ID = 212;
    ///
    const int UNDEF_TEAM_ID = 213;
    ///
    const int BOTH_TEAMS_ID = 213;
    ///
    const int MAX_TEAM = 213;
    //@}
  
  /** @name Der Ball */
  //@{ 
    
    ///
    const int BALL_ID = 300;
    //@}
  
  /** @name Spieler
    Fuer einzelne Spieler wird die ObjectId aus  ( MINOPPONENT_ID +
    Spielernummber - 1 ) bzw. ( MINTEAMMATE_ID + Spielernummer - 1 ) berechnet.

    SELF_ID liegt nicht zwischen MIN_TEAMMATE_ID und MAX_EXISTING_TEAMMATE_ID,
    aber unterhalb von MAX_PLAYER_ID.

    REFEREE_ID liegt ausserhalb [MIN_PLAYER_ID, MAX_PLAYER_ID]. */
  //@{
    ///
    const int MIN_PLAYER_ID = 400;
    ///
    const int MIN_OPPONENT_ID = 400;
    ///
    const int MAX_EXISTING_OPPONENT_ID = 410;
    ///
    const int UNDEF_OPPONENT_ID = 411;
    ///
    const int OPPONENT_ID = 411;
    ///
    const int MAX_OPPONENT_ID = 411;
    
    ///
    const int MIN_TEAMMATE_ID = 420;
    ///
    const int MAX_EXISTING_TEAMMATE_ID = 430;
    ///
    const int UNDEF_TEAMMATE_ID = 431;
    ///
    const int TEAMMATE_ID = 431;
    ///
    const int SELF_ID = 432;
    ///
    const int MAX_TEAMMATE_ID = 432;
    ///
    const int UNDEF_PLAYER_ID = 433;
    ///
    const int PLAYER_ID = 433;
    ///
    const int MAX_PLAYER_ID = 433;
    
    ///
    const int REFEREE_ID = 450;
  //@}  
//@}

/** @name Aktionen
  Identifizierende Konstanten f\"ur elementare Aktionen, die der Agent an den
  SoccerServer schicken kann. */
//@{
  ///
  const int MIN_ACTION = 500;
  ///
  const int MIN_DEFAULT_ACTION = 500;
  ///
  const int DASH_ACTION = 500;
  ///
  const int TURN_ACTION = 501;
  ///
  const int KICK_ACTION = 502;
  ///
  const int MOVE_ACTION = 503;
  ///
  const int CATCH_ACTION = 504;
  ///
  const int TURN_NECK_ACTION = 505;
  ///
  const int SENSE_BODY_ACTION = 506;
  ///
  const int EMPTY_ACTION = 507;
  ///
  const int MAX_DEFAULT_ACTION = 507;
  ///
  const int SAY_ACTION = 508;
  ///
  const int CHANGE_VIEW_ACTION = 509;
  ///
  const int UNDEF_ACTION = 510;
  ///
  const int MAX_ACTION = 510;

  /** @name Parameter von ChangeView */
  //@{
    ///
    const int MIN_PARAM_ACTION = 520;
    ///
    const int VIEW_WIDTH_NARROW = 520;
    ///
    const int VIEW_WIDTH_NORMAL = 521;
    ///
    const int VIEW_WIDTH_WIDE = 522;
    ///
    const int VIEW_QUALITY_LOW = 523;
    ///
    const int VIEW_QUALITY_HIGH = 524;
    ///
    const int VIEW_UNDEF = 525;
    ///
    const int MAX_PARAM_ACTION = 525;
    //@}
  //@}


  /** Sichtwinkel */
  const double VIEW_WIDTH_NARROW_ANGLE = Ath98_PI / 4;
  const double VIEW_WIDTH_NORMAL_ANGLE = Ath98_PI / 2;
  const double VIEW_WIDTH_WIDE_ANGLE   = Ath98_PI;

   /** liefert Sichtwinkel zur Id zurueck */
  inline double viewWidthAngleWithId(const int viewWidthId)
  {
    switch (viewWidthId) {
    case VIEW_WIDTH_NARROW :
      return VIEW_WIDTH_NARROW_ANGLE;
      break;
    case VIEW_WIDTH_NORMAL :
      return VIEW_WIDTH_NORMAL_ANGLE;
      break;
    case VIEW_WIDTH_WIDE :
      return VIEW_WIDTH_WIDE_ANGLE;
      break;
    default:
      cerr << "bad viewWidthId - returning VIEW_WIDTH_NORMAL_ANGLE\n";
      return VIEW_WIDTH_NORMAL_ANGLE;
    }
  }


    
/** @name Nutzen und Kosten von Handlungsoptionen
  Kosten koennen in Nutzen umgerechnet werden durch Invertierung des
  Vorzeichens. */
//@{
  ///
  const int MAX_UTILITY = 1000;
  ///
  const int NEUTRAL_UTILITY = 0;
  ///
  const int MIN_UTILITY = -1000;
  
  /* */
  const int MAX_COST = MAX_UTILITY;
  ///
  const int NEUTRAL_COST = NEUTRAL_UTILITY;
  ///
  const int MIN_COST = MIN_UTILITY;
//@}
  
/** @name Rollen
  Rollen bestimmen das Verhalten der Spieler. Bei der Einstellung in der
  Konfigurationsdatei muessen die internen Werte verwendet werden. Sie sind
  daher in der Dokumentation angegeben in Klammern angegeben. */
//@{
  /// St\"urmer (10)
  const int FORWARD_ROLE = 10;
  /// Online Lernender Stuermer
  const int LEARNING_FORWARD_ROLE = 15;
  /// zurckweichender Strmer
  const int DRAWBACK_FORWARD_ROLE = 17;
  /// zurckweichender lernender Strmer
  const int DRAWBACK_LEARNING_FORWARD_ROLE = 19;
  /// Mittelfeldspieler (20)
  const int MIDFIELD_ROLE = 20;
  /// nachrueckender Mittelfeldspieler (22)
  const int MOVE_FORWARD_MIDFIELD_ROLE = 22;
  /// Online Lernender Mittelfeldspieler (15)
  const int LEARNING_MIDFIELD_ROLE = 25;
  /// zurckweichender Mittelfeldspieler
  const int DRAWBACK_MIDFIELD_ROLE = 26;
  /// vorwrts und rckwrtsbewegender Mittelfeldspieler
  const int MOVING_MIDFIELD_ROLE = 27;
  /// zurckweichender lernender Mittelfeldspieler
  const int DRAWBACK_LEARNING_MIDFIELD_ROLE = 28;
  ///
  const int GENERIC_ROLE = 21;
  /// Verteidiger (30)
  const int DEFENDER_ROLE = 30;
  /// nachrueckender Verteidiger (32)
  const int MOVE_FORWARD_DEFENDER_ROLE = 32;
  /// Online lernender Verteidiger (35)
  const int LEARNING_DEFENDER_ROLE = 35;
  /** Torwart (40). 
    Er kann seine Rolle nicht wechseln. */
  const int GOALIE_ROLE = 40;
  /** Zusatztorwart ohne Torwartvorrechte (41). 
    Er kann wechseln, darf aber kein catch machen. */
  const int ASSISTANT_GOALIE_ROLE = 41;
//@}

    
/** @name Blocktrennzeichen im ConstantLog
 Diese Zeichen werden vom Logbrowser benutzt, um die ansonsten unkommentierten
 Werte zu erkennen */
//@{
  ///
  const char BODY_INFOS_MARK = 216;       // gestrichene Null
  ///
  const char MODELLED_PLAYER_MARK = 169;  // copyright
  ///
  const char WORLD_MODEL_MARK = 167;      //  
  ///
  const char DELIBERATOR_MARK = 64;       // @ 
  ///
  const char EXPECTED_UTILITY_MARK = 174; // registered mark
  ///
  const char QUEUESTART_MARK = 163;       // Zeichen fr Whrung Pfund 
  ///
  const char STEP_MARK = 183;             // .
  ///
  const char SAY_MESSAGE_END_MARK = 184;       // Endezeichen fuer Say-Message
  ///
  const char SENDED_ACTIONS_MARK = 185;          // 
//@}
  

/** @name Verhalten des Nackens
    Diese Konstanten schicken die Optionen an die Klasse Neck 
    um das gewschte Verhalten des Nackens mitzuteilen */
//@{
  ///
  const int NECK_IN_FRONT = 600;
  ///
  const int NECK_TO_BALL = 601;       
  ///
  const int NECK_TO_MIDDLE = 602;  
  ///
  const int NECK_TO_POS = 603;
  ///
  const int NECK_OSCILLATE = 604;
  ///
  const int NECK_TO_OWN_GOAL = 605;  
  ///
  const int NECK_TO_ENEMY_GOAL = 606;  
  ///
  const int NECK_TO_PLAYER_WITH_ID = 609;  
  ///
  const int NECK_TO_PREFERRED_KICK_DIRECTION = 610;
  ///
  const int NECK_TO_BACK_KICK_DIRECTION = 611;
  ///
  const int NECK_TO_LEFT = 612;       
  ///
  const int NECK_TO_RIGHT = 613;  
//@}

/** @name Ausrichtung des Nackens
      Ausrichtung des Nackens bzgl. des zu sehenden Objekts (soll das Objekt
      am rechten Rand, am linken Rand oder in der Mitte des Blickfeldes sein) */
//@{
  ///
  const int ALIGN_LEFT = 630;
  ///
  const int ALIGN_CENTER = 631;
  ///
  const int ALIGN_RIGHT = 632;
  ///
//@}

/** @name Prioritaet der TurnNeck Aktion 
    Definiert wie wichtig die TurnNeck Aktion ist. Der Nacken fuehrt nur das
    Kommando mit der hoechsten Prioritaet aus. */
//@{
  ///
  const int VERY_HIGH_PRIORITY = 640;
  ///
  const int HIGH_PRIORITY = 641;
  ///
  const int MEDIUM_PRIORITY = 642;
  ///
  const int LOW_PRIORITY = 643;
  ///
  const int VERY_LOW_PRIORITY = 644;
  ///
  const int NO_PRIORITY = 645;

  // SoccerServer Wert f\"ur perfekte Sicht
  // XXX k\"onnte sich bei neunen Versionen \"andern XXX
  const double  UNUM_TOO_FAR_LENGTH = 20.0;
//@}


  /** @name M\"ogliche Gr\"unde die einen Abbruch des laufenden Planes
    verursachen k\"onnen. */
  //@{
    ///
    const int CANCEL_REASON_REFEREE_DECISION = 1;
    ///
    const int CANCEL_REASON_NEW_BALL_INFO = 2;
    ///
    const int CANCEL_REASON_CONT_TIME_EXPIRED = 4;
    ///
    const int CANCEL_REASON_PLAN_COMPLETED = 8;
    ///
    const int CANCEL_REASON_FREEKICK = 16;
    ///
    const int ALL_CANCEL_REASONS = 31;
  //@}

/** @name Hilfreiche Funktionen */
//@{
  
  /** Globaler Vergleichsoperator fuer double-Werte. Zwei Werte werden als
    gleich gewertet, wenn sie sich im weniger als die Vergleichsgenauigkeit
    unterscheiden. */
  bool isAlmostEqual(double x1, double x2);
  
  /** Setzt die Vergleichsgenauigkeit, die fuer den Naeherungsvergleich von
    double Werten verwendet wird. Default ist 1e-7; */
  void setComparisonAccuracy( double newComparisonAccuracy );
  
  /** Wandelt Gradzahl ins Bogenmass um. */
  double radFromDegree(double degreeValue);
  
  /** Wandelt Bogenmass in Gradzahl um. */
  double degFromRadiant(double degreeValue);
  
  /** Gibt 1 zurueck wenn value >= 0, sonst -1. */
  int signum( double value );
  
  /** Normalisiert einen Winkel.
    @param angle Ein Winkel im Bogenma\3
    @return Ein Winkel zwischen 0 und +2*Ath98_PI */
  double normalizeAngle(double anAngle) ;

  /** Berechnet den Unterschied zwischen zwei ATH-Winkeln
 	  das Ergebnis ist ein angle, keine Dir !
 	  d.h. -PI < result < PI */
  double deviationBetween(double Dir1, double Dir2 );
  
  double minAngleTo(double angleFrom, double angleTo );
  
  /** Nichtterminierende Funktion.
    Wird f\"ur Debugging-Zwecke verwendet. Sobald das Programm hier angekommen
    ist, kann man sich mit dem Debugger per 'attach' einklinken, mit 'return'
    die Funktion beenden und nun schrittweise den weiteren Ablauf verfolgen. */
  void endlessLoop();

  //@}


// Inline - Definitionen

inline bool isAlmostEqual(double x1, double x2) 
{
  return ( fabs( x1 - x2 ) < globalComparisonAccuracy );
}

inline void setComparisonAccuracy( double newComparisonAccuracy )
{
  globalComparisonAccuracy = newComparisonAccuracy;
}

inline double radFromDegree( double degreeValue ) 
{
  return Ath98_PI_FACTOR * degreeValue;
}

inline double degFromRadiant(double degreeValue) 
{
  return degreeValue / Ath98_PI_FACTOR;
}

inline int signum( double value )
{
  return ((value >= 0) ? 1 : -1);
}

inline double minAngleTo( double angleFrom, double angleTo )
{
  return normalizeAngle ( angleTo - angleFrom );
}

inline double deviationBetween(double Dir1, double Dir2 )
{
  double dirDiff = normalizeAngle(Dir2 - Dir1);
  return (dirDiff < Ath98_PI) ? dirDiff : dirDiff - 2.0 * Ath98_PI;
}

#endif


