/*
 * Aktueller Stand
 *
 *   $RCSfile: Configuration.cc,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 2001/07/31 22:23:08 $
 *   $Author: rami $
 *
 */

#include <ctype.h>
#include <fstream.h>
#include <iostream.h>
#include <stdio.h>
#include <strings.h>

#include "cstdlib"

#include "Configuration.h"
#include "Exception.h"
#include "Log.h"

Configuration *globalConf;

/** Tabelle fuer die Intercept-Berechnung */
vector < vector < vector  <int> > > table;
/** Tabelle fuer die Ballbeschleunigungen */
vector< vector < double > > speedVectors;

Configuration::Configuration() {

  theIdString = "Config";
/* Die ganze Datenstruktur mit default-Werten */
  static options serverOptionsTemp[] = {
    // Server-Parameter
    /** Sollte der Server die server.conf nicht einlesen (bzw. einen Parameter
        auslassen) verwendet er folgende (merkwuerdige) DefaultParameter. ) */
    {"goal_width",          (void *)&goalWidth,           DOUBLE,  7.32},
    {"player_size",         (void *)&playerSize,          DOUBLE,  1.0},
    {"player_decay",        (void *)&playerDecay,         DOUBLE,  0.5},
    {"player_rand",         (void *)&playerRand,          DOUBLE,  0.1},
    {"player_weight",       (void *)&playerWeight,        DOUBLE,  60.0},
    {"player_speed_max",    (void *)&playerSpeedMax,      DOUBLE,  32.0},
    {"player_accel_max",    (void *)&playerAccelMax,      DOUBLE,  0.6},   
    {"stamina_max",         (void *)&playerStaminaMax,    DOUBLE,  2500.0},
    {"stamina_inc_max",     (void *)&playerStaminaInc,    DOUBLE,  50.0},
    {"recover_dec_thr",     (void *)&playerRecoverDecThr, DOUBLE,  0.3},
    {"recover_dec",         (void *)&playerRecoverDec,    DOUBLE,  0.05},
    {"recover_min",         (void *)&playerRecoverMin,    DOUBLE,  0.1},
    {"effort_dec_thr",      (void *)&playerEffortDecThr,  DOUBLE,  0.4},
    {"effort_dec",          (void *)&playerEffortDec,     DOUBLE,  0.05},
    {"effort_inc_thr",      (void *)&playerEffortIncThr,  DOUBLE,  0.9},
    {"effort_inc",          (void *)&playerEffortInc,     DOUBLE,  0.05},
    {"effort_min",          (void *)&playerEffortMin,     DOUBLE,  0.1},
    {"hear_max",            (void *)&hearCapacityMax,     INT,     2}, 
    {"hear_inc",            (void *)&hearCapacityInc,     INT,     1}, 
    {"hear_decay",          (void *)&hearCapacityDecay,   INT,     1}, 
    {"inertia_moment",      (void *)&inertiaMoment,       DOUBLE,  5.0}, 
    {"sense_body_step",     (void *)&senseBodyStep,       DOUBLE,  100}, 
    {"catchable_area_l",    (void *)&catchAreaLength,     DOUBLE,  2.0}, 
    {"catchable_area_w",    (void *)&catchAreaWidth,      DOUBLE,  1.0}, 
    {"catch_probability",   (void *)&catchProbability,    DOUBLE,  1.0}, 
    {"catch_ban_cycle",     (void *)&catchCycle,          INT,     5}, 
    {"goalie_max_moves",    (void *)&goalieMaxMovesAfterCatch,  INT,  2},
    {"ball_size",           (void *)&ballSize,            DOUBLE,  0.15},
    {"ball_decay",          (void *)&ballDecay,           DOUBLE,  0.8},
    {"ball_rand",           (void *)&ballRand,            DOUBLE,  0.2},
    {"ball_weight",         (void *)&ballWeight,          DOUBLE,  0.2},
    {"ball_speed_max",      (void *)&ballSpeedMax,        DOUBLE,  32.0},
    {"ball_accel_max",      (void *)&ballAccelMax,        DOUBLE,  1.6},
    {"wind_dir",            (void *)&windDirection,       DOUBLE,  0.0}, 
    {"wind_force",          (void *)&windForce,           DOUBLE,  10.0}, 
    {"wind_rand",           (void *)&windRand,            DOUBLE,  0.3}, 
    {"kickable_margin",     (void *)&kickMarginServer,    DOUBLE,  1.0}, 
    {"ckick_margin",        (void *)&cornerKickMargin,    DOUBLE,  1.0}, 
    {"dash_power_rate",     (void *)&dashPowerRate,       DOUBLE,  0.1},
    {"kick_power_rate",     (void *)&kickPowerRate,       DOUBLE,  0.1},
    {"kick_rand",           (void *)&kickRand,            DOUBLE,  0.0},
    {"visible_angle",       (void *)&visibleAngleInDegree,DOUBLE,  90.0}, 
    {"audio_cut_dist",      (void *)&audioCutOffDistance, DOUBLE,  50.0}, 
    {"quantize_step",       (void *)&distQuantStep,   DOUBLE,  0.1}, 
    {"quantize_step_l",     (void *)&landmarksQuantStep,  DOUBLE,  0.01}, 
    {"maxpower",            (void *)&maxPower,            DOUBLE,  100.0}, 
    {"minpower",            (void *)&minPower,            DOUBLE,  -30.0}, 
    {"maxmoment",           (void *)&maxMoment,           DOUBLE,  180.0}, 
    {"minmoment",           (void *)&minMoment,           DOUBLE,  -180.0}, 
    {"maxneckmoment",       (void *)&maxNeckMoment,       DOUBLE,  180.0}, 
    {"minneckmoment",       (void *)&minNeckMoment,       DOUBLE,  -180.0},
    {"maxneckang",          (void *)&maxNeckAngle,        DOUBLE,  90.0}, 
    {"minneckang",          (void *)&minNeckAngle,        DOUBLE,  -90.0}, 
    {"port",                (void *)&portNumber,          INT,     6000}, 
    {"coach_port",          (void *)&trainerPortNumber,   INT,     6001}, 
    {"say_coach_cnt_max",   (void *)&sayCoachCntMax,      DOUBLE,  128}, 
    {"say_coach_msg_size",  (void *)&sayCoachMsgSize,     DOUBLE,  128}, 
    {"clang_win_size",      (void *)&clangWinSize,        INT,     100}, 
    {"clang_define_win",    (void *)&clangDefineWin,      INT,     1}, 
    {"clang_meta_win",      (void *)&clangMetaWin,        INT,     1}, 
    {"clang_advice_win",    (void *)&clangAdviceWin,      INT,     1}, 
    {"clang_info_win",      (void *)&clangInfoWin,        INT,     1}, 
    {"clang_mess_delay",    (void *)&clangMessDelay,      INT,     50}, 
    {"clang_mess_per_cycle", (void *)&clangMessPerCycle,  INT,     1}, 
    {"send_vi_step",       (void *)&sendViStep,           DOUBLE,  100}, 
    {"simulator_step",      (void *)&simulatorStep,       INT,     100}, 
    {"send_step",           (void *)&sendStep,            INT,     150}, 
    {"recv_step",           (void *)&receiveStep,         INT,     20}, 
    {"half_time",           (void *)&halfTimeInSecs,      INT,     600}, 
    {"drop_ball_time",      (void *)&dropBallTime,        INT,     0}, 
    {"say_msg_size",        (void *)&sayMessageSize,      INT,     256}, 
    {"use_offside",         (void *)&useOffSide,          BOOL,    1}, 
    {"offside_active_area_size", (void *)&offsideArea,    DOUBLE,  9.15}, 
    {"forbid_kick_off_offside", (void *)&forbidKickOffOffside, BOOL, 0}, 
    {"verbose",             (void *)&verbose,             BOOL,  0}, 
    {"record_version",      (void *)&recordVersion,       INT,     2}, 
    {"record_log",          (void *)&recLog,              BOOL,  0}, 
    {"send_log",            (void *)&sendLog,             BOOL,  1}, 
    {"offside_kick_margin", (void *)&offsideKickMargin,   DOUBLE,  9.15}, 
    {"log_times",           (void *)&logTimes,            BOOL,    0}, 
    {"log_file",            (void *)&logFile,             STRING,  0}, 
    {"record",              (void *)&recFile,             STRING,  0}, 
    {"record_messages",     (void *)&recordMessages,      BOOL,    1}, 
    {"coach_w_referee",     (void *)&coachWithRefereeMode, OPTIONAL,    0}, 
    {"slow_down_factor",    (void *)&slowDownFactor,      INT,     1},
    {"control_radius",      (void *)&controlRadius,       DOUBLE,  2.0}, 
    {"visible_distance",    (void *)&visibleDistance,     DOUBLE,  3.0}, 
    {"quantize_step_dir",   (void *)&dirQuantStep,        DOUBLE,  0.1}, 
    {"wind_none",           (void *)&isWindFactor,        OPTIONAL,    0}, 
    {"wind_random",         (void *)&windRandom,          OPTIONAL,    0}, 
    {"olcoach_port",        (void *)&coachPortNumber,     INT,     6002},
    {"coach",               (void *)&coachMode,           OPTIONAL,    0}, 
    {"old_coach_hear",      (void *)&oldCoachHear,        OPTIONAL,    0}, 
    {"replay",              (void *)&replay,              STRING,  0}, 
    {"start_goal_l",        (void *)&startGoalL,          INT,     0},
    {"start_goal_r",        (void *)&startGoalR,          INT,     0},
    {"fullstate_l",         (void *)&fullstateL,          BOOL,    0},
    {"fullstate_r",         (void *)&fullstateR,          BOOL,    0},
    {"learn_file",          (void*)&learnFile,            STRING,  0}, 
    {"learning",            (void*)&learning,             BOOL,    0}, 
 
    // Agent-Parameter 
    {"situationLog",         (void *)&situationLog,        BOOL,    0},
    {"optimalKickDist",     (void *)&optimalKickDist,    DOUBLE,  0.5}, 
    {"maxKickDist",     (void *)&maxKickDist,    DOUBLE,  1.0}, 
    {"minShotLength",     (void *)&minShotLength, DOUBLE, 6.0}, 
    {"historyCount",        (void *)&historyCount,       INT,     7}, 
    {"maxLengthOfPlan",     (void *)&maxLengthOfPlan,    INT,     7}, 
    {"toleratedStabilityCost", (void *)&toleratedStabilityCost, INT, 0}, 
    {"minConstraintUtility", (void *)&minConstraintUtility, INT,  7}, 
    {"minDesireUtility",    (void *)&minDesireUtility,   INT,     7}, 
    {"enforcedMinDistToBall", (void *)&enforcedMinDistToBall, DOUBLE, 9.15}, 
    {"maxStepsOfPlayerMovement", (void *)&maxStepsOfPlayerMovement, INT, 7}, 
    {"maxStepsOfBallMovement", (void *)&maxStepsOfBallMovement, INT, 25}, 
    {"maxEffectiveBallSpeed", (void *)&maxEffectiveBallSpeed, DOUBLE, 2.4}, 
    {"averageStepsOfBallMovement", (void *)&averageStepsOfBallMovement,INT,15}, 
    {"roles", (void *)&roles, ARRAY_OF_INT, 11}, 
    {"kickoffPositionsOffensive", (void *)&kickoffPositionsOffensive, ARRAY_OF_VECTOR, 11}, 
    {"kickoffPositionsDefensive", (void *)&kickoffPositionsDefensive, ARRAY_OF_VECTOR, 11}, 
    {"homePositionsOffensive", (void *)&homePositionsOffensive, ARRAY_OF_VECTOR, 11}, 
    {"homePositionsDefensive", (void *)&homePositionsDefensive, ARRAY_OF_VECTOR, 11}, 
    {"maxGoalKickUtility",    (void *)&maxGoalKickUtility,   INT, 900}, 
    {"goalKickUtilityWeights", (void *)&goalKickUtilityWeights, ARRAY_OF_INT, 4}, 
    {"goalReduce", (void *)&goalReduce, DOUBLE, 0.5}, 
    {"initialBallSpeed", (void *)&initialBallSpeed, DOUBLE, 22.0}, 
    {"maxForwardPassUtility",    (void *)&maxForwardPassUtility,   INT, 600}, 
    {"minForwardPassUtility",    (void *)&minForwardPassUtility,   INT, 50}, 
    {"maxDirectPassUtility",    (void *)&maxDirectPassUtility,   INT,   700}, 
    {"maxBackwardPassUtility",    (void *)&maxBackwardPassUtility,   INT,   300}, 
    {"maxDoublePassUtility",    (void *)&maxDoublePassUtility,   INT,   600}, 
    {"maxDribbleUtility",    (void *)&maxDribbleUtility,   INT,     600}, 
    {"maxWaitUtility",    (void *)&maxWaitUtility,   INT,     750}, 
    {"maxControlBallUtility",    (void *)&maxControlBallUtility,   INT, 1000}, 
    {"maxAvoidOffsideUtility", (void *)&maxAvoidOffsideUtility, INT, 500}, 
    {"avoidOffsideStaminaLimitBonus", (void *)&avoidOffsideStaminaLimitBonus, DOUBLE, 200}, 
    {"maxConserveStaminaUtility", (void *)&maxConserveStaminaUtility, INT, 1000}, 
    {"conserveStaminaActivation", (void *)&conserveStaminaActivation, INT, 1500}, 
    {"conserveStaminaUtilityIncrease", (void *)&conserveStaminaUtilityIncrease, ATH98VECTOR, UNDEF_INT }, 
    {"dribbleSectorLength",    (void *)&dribbleSectorLength, DOUBLE , 5.0}, 
    {"dribbleSectorWidth",    (void *)&dribbleSectorWidth, DOUBLE, 10.0}, 
    {"minCollectDataUtility", (void *)&minCollectDataUtility, INT, 100},     
    {"normalCollectDataUtility", (void *)&normalCollectDataUtility, INT, 200}, 
    {"maxCollectDataUtility", (void *)&maxCollectDataUtility, INT, 300}, 
    {"maxGoToBaseUtility", (void *)&maxGoToBaseUtility, INT, 200},
    {"normalGoToBaseUtility", (void *)&normalGoToBaseUtility, INT, 150}, 
    {"minGoToBaseUtility", (void *)&minGoToBaseUtility, INT, 50}, 
    {"defenderMoveForwardBallDistActivation", (void *)&defenderMoveForwardBallDistActivation, DOUBLE, 30}, 
    {"defenderMoveForwardStopLine", (void *)&defenderMoveForwardStopLine, DOUBLE, -10}, 
    {"midfieldMoveForwardBallDistActivation", (void *)&midfieldMoveForwardBallDistActivation, DOUBLE, 15}, 
    {"midfieldMoveForwardStopLine", (void *)&midfieldMoveForwardStopLine, DOUBLE, 10}, 
    {"forwardDrawbackStopLine", (void *)&forwardDrawbackStopLine, DOUBLE, 10},
    {"midfieldDrawbackStopLine", (void *)&midfieldDrawbackStopLine, DOUBLE, 10},
    {"gotoBasePositionAccuracy", (void *)&gotoBasePositionAccuracy, DOUBLE, 1.0}, 
    {"getIntoSpaceAccuracy", (void *)&getIntoSpaceAccuracy, DOUBLE, 1.0}, 
    {"standardTargetAccuracy", (void *)&standardTargetAccuracy, DOUBLE, 0.5}, 
    {"interceptBallAccuracy", (void *)&interceptBallAccuracy, DOUBLE, 1.0}, 
    {"normalInterceptBallUtility", (void *)&normalInterceptBallUtility, INT, 800}, 
    {"normalGetIntoSpaceUtility", (void *)&normalGetIntoSpaceUtility, INT, 700}, 
    {"leaveBasePositionRange", (void *)&leaveBasePositionRange, DOUBLE, 12}, 
    {"getIntoSpaceDist", (void *)&getIntoSpaceDist, INT, 20}, 
    {"getIntoSpaceMinDist", (void *)&getIntoSpaceMinDist, DOUBLE, 10}, 
    {"getIntoSpaceSimSteps", (void *)&getIntoSpaceSimSteps, INT, 7}, 
    {"getIntoSpaceAvoidBasePositionDist", 
     (void *)&getIntoSpaceAvoidBasePositionDist, DOUBLE, 5}, 
    {"getIntoSpaceWindowEnemy", (void *)&getIntoSpaceWindowEnemy, DOUBLE, 1}, 
    {"getIntoSpaceWindowSelf", (void *)&getIntoSpaceWindowSelf, DOUBLE, 15}, 
    {"getIntoSpaceMinPosition", (void *)&getIntoSpaceMinPosition, DOUBLE, 25}, 
    {"getIntoSpaceMinParallelPos", (void *)&getIntoSpaceMinParallelPos, DOUBLE, -52.5}, 
    {"avoidCoverActivationDist", (void *)&avoidCoverActivationDist, DOUBLE, 3}, 
    {"avoidCoverBestPositionDist", (void *)&avoidCoverBestPositionDist, DOUBLE, 2}, 
    {"maxForceContinuationTime", (void *)&maxForceContinuationTime, INT, 25}, 
    {"distToBallForEvaluation", (void *)&distToBallForEvaluation,DOUBLE,40.0}, 
    {"maxStepsToCalculate", (void *)&maxStepsToCalculate, INT, 40 }, 
    {"debugModes",              (void *)&debugModes,  ARRAY_OF_INT,     17}, 
    {"tournamentMode",              (void *)&tournamentMode,  BOOL,     0}, 
    {"useCommunication",              (void *)&useCommunication,  BOOL,     0}, 
    {"communicationTimeout", (void *)&communicationTimeout, INT, 10}, 
    {"defaultStaminaLimit", (void *)&defaultStaminaLimit, DOUBLE, 1201},
    {"dribbleStaminaLimit", (void *)&dribbleStaminaLimit, DOUBLE, 1201}, 
    {"interceptBallStaminaLimit", (void *)&interceptBallStaminaLimit, DOUBLE, 1201}, 
    {"goalieInterceptBallStaminaLimit", (void *)&goalieInterceptBallStaminaLimit, DOUBLE, 1201}, 
    {"gotoBaseStaminaLimit", (void *)&gotoBaseStaminaLimit, DOUBLE, 1201}, 
    {"positioningStaminaLimit", (void *)&positioningStaminaLimit, DOUBLE, 1201}, 
    {"getIntoSpaceStaminaLimit", (void *)&getIntoSpaceStaminaLimit, DOUBLE, 1201}, 
    {"interceptionTolerance", (void *)&interceptionTolerance, INT, 2 }, 
    {"goalieDistFromGoal", (void *)&goalieDistFromGoal, DOUBLE, 7.0 }, 
    {"passLeadForTeammates", (void *)&passLeadForTeammates, INT, 5 }, 
    {"passLeadForTeammatesGoalieOffset", 
     (void *)&passLeadForTeammatesGoalieOffset, DOUBLE, 2 }, 
    {"safetyNeckAngleToBall", (void *)&safetyNeckAngleToBall, DOUBLE, 10.0 }, 
    {"shouldNeckBeUsed", (void *)&shouldNeckBeUsed, INT, 1 }, 
    {"emergencyPassDist", (void *)&emergencyPassDist, INT, 5 }, 
    {"reactionTimeSteps", (void *)&reactionTimeSteps, INT, 2 }, 
    {"dribbleInOffsideBonus", (void *)&dribbleInOffsideBonus, INT, 100 }, 
    {"maxGoalKickDist", (void *)&maxGoalKickDist, DOUBLE, 30.0 }, 
    {"waitTimeDuringFreeKick", (void *)&waitTimeDuringFreeKick, INT, 10}, 
    {"minWaitTimeForDefendCoordination", (void *)&minWaitTimeForDefendCoordination, INT, 10}, 
    {"encodeTable", (void *)&encodetable, STRING, 0 },
    {"encodeTableLength", (void *)&encodetablelength, INT, 0}, 
    {"neckTurnPrioBall", (void *)&neckTurnPrioBall, INT, 4},
    {"neckTurnPrioRegion", (void *)&neckTurnPrioRegion, INT, 1},
    {"neckTurnPrioPlayer", (void *)&neckTurnPrioPlayer, INT, 2},
    {"raiseNeckPriorityBoundary", (void*)&raiseNeckPriorityBoundary, INT, 80},

  //Online Learning Parameter     
    { "xResolution",               (void *)&xResolution,             INT,  5 }, 
    { "yResolution",               (void *)&yResolution,             INT,  4 }, 
    { "incPlayer",                 (void *)&incPlayer,               INT, 10 }, 
    { "incBall",                   (void *)&incBall,                 INT, 20 }, 
    { "decPlayer",                 (void *)&decPlayer,               INT,  5 }, 
    { "decBall",                   (void *)&decBall,                 INT,  0 }, 
    { "threshold",                 (void *)&threshold,               INT, 30 }, 
    { "minEdgeLength",             (void *)&minEdgeLength,           INT,  6 }, 
    { "maxEdgeLength",             (void *)&maxEdgeLength,           INT, 35 }, 
    { "manGuardingBorder",         (void *)&manGuardingBorder,       INT, 55 }, 
    { "distFromOpponent",      (void *)&distFromOpponent,    INT,  3 }, 
    { "weightDistFromHome",(void *)&weightDistFromHome, DOUBLE, 9.0 }, 
    { "weightDistFromCurrent", (void *)&weightDistFromCurrent,DOUBLE,2.0 }, 
    { "weightBallAppearance",     (void *)&weightBallAppearance,DOUBLE, 0.5 }, 
    { "weightEdgeColour",         (void *)&weightEdgeColour,   DOUBLE, 1.0  }, 
    { "maxUtilityDist",       (void *)&maxUtilityDist, INT, 40 }, 
    { "minUtilityDist",       (void *)&minUtilityDist, INT, 12 }, 
    { "ballLearnDist",        (void *)&ballLearnDist,  INT, 12 }, 
    { "maxDistFromHome",      (void *)&maxDistFromHome, INT, 20 }, 
    { "learnNoGoalieDist",    (void *)&learnNoGoalieDist, INT, 3 },  
    { "defenderFreeKickDist", (void *)&defenderFreeKickDist, DOUBLE, 13 },  
    { "defenderFreeKickMoveUp", (void *)&defenderFreeKickMoveUp, DOUBLE, 5 },  
    { "dangerousRangeFromGoal", (void *)&dangerousRangeFromGoal, DOUBLE, 40 }, 
    { "outerKickCircleTolerance", (void *)&outerKickCircleTolerance, DOUBLE, 0.15 }, 
    { "innerKickCircleTolerance", (void *)&innerKickCircleTolerance, DOUBLE, 0.2 }, 
    { "intermediateKickTolerance", (void *)&intermediateKickTolerance, DOUBLE, 0.2 }, 
    { "ourMessageIdentifier", (void *)&ourMessageIdentifier, INT, 666 }, 
    { "escapeFieldOfViewDist", (void *)&escapeFieldOfViewDist, DOUBLE, 5 }, 
    { "maximalMergeDist", (void *)&maximalMergeDist, DOUBLE, 10 }, 
    { "distToPlayerForSay", (void *)&distToPlayerForSay, DOUBLE, 30 }, 
    { "sayAngleDifference", (void *)&sayAngleDifference, DOUBLE, 20 }, 
    { "calculatePositionInterval", (void *)&calculatePositionInterval, INT, 3 },
    // Ende 
    {"\0",                          NULL,                0,       0} 
  };  
 
  serverOptions = serverOptionsTemp;  
 
  int i;  
  /* Einstellen der default-Werte */ 
  for(i=0;  serverOptions[i].pointer != NULL;  i++) { 
    switch( serverOptions[i].type ) { 
      case DOUBLE: 
        *(double *)serverOptions[i].pointer = serverOptions[i].value;  
        break;  
      case INT: 
        *(int *)serverOptions[i].pointer = (int)serverOptions[i].value;  
        break;  
      case BOOL: 
      case OPTIONAL: 
        *(bool *)serverOptions[i].pointer = (bool)serverOptions[i].value;  
        break;  
      case STRING: 
        *(char *)serverOptions[i].pointer = '\0';  
        break;  
      case ATH98VECTOR: 
        *(Vector *)serverOptions[i].pointer = Vector();  
        break;  
      case ARRAY_OF_VECTOR: 
        *(vector<Vector> *)serverOptions[i].pointer =
            vector<Vector>( (int)serverOptions[i].value );
        break;  
      case ARRAY_OF_INT: 
        *(vector<int> *)serverOptions[i].pointer =
            vector<int>( (int)serverOptions[i].value );
        break;  
      default: 
        throwException( Exception( "Unknown Parametertype" ) );  
    } 
  } // Ende for-Schleife 
 
  calculateDependentParameters();  
 
  // Tabelle einlesen 
  readTable();  

  coachWithRecordFile = NO;
  coachUseInteractive = NO;
} 
 
char *Configuration::readLine( FILE *filePointer )  
{ 
  static char buffer[MAX_MESSAGE];  
  bool isDataAvailable = NO;  
  int length = 0;  
  do { 
    if ( fgets( buffer + length, MAX_MESSAGE - length, filePointer ) == NULL) { 
      if ( length > 0 ) { 
        break;  
      }  
      else { 
        return NULL;  
      } 
    } 
    /* Kommentare und Leerzeilen ignorieren */ 
    if (buffer[length] == '#' || buffer[length] == '\n') { 
      continue;  
    } 
     
    length = strlen( buffer );  
    if ( length >= 2 && strcmp( buffer+length-2, "\\\n") == 0 ) { 
      length -= 2;  
      continue;  
    } 
     
    if ( length > 0 ) { 
      isDataAvailable = YES;  
    } 
  } while ( isDataAvailable == NO );  
   
  // Ersten ':' in ' ' umwandeln 
  char *t = buffer;  
  while(*t != '\0') { 
    if (*t == ':') { 
      *t = ' ' ;  
      break;  
    } 
    t++;  
  }  
    
  return buffer;  
} 
 
 
int Configuration::scanArrayOfVector( char *buffer, vector<Vector> *ArrayPtr )
{ 
  int count;  
  char parameterName[256];  
  char parameterBuffer[256];  
  int n;  
  n = sscanf(buffer, "%s %d %[^\n]", parameterName, &count, parameterBuffer);  
  if ( n == 3 && count >=0 ) { 
    ArrayPtr->clear();
    int index = 0;  
    double x, y;  
    char *parameterPtr = parameterBuffer;  
    while ( isspace( parameterPtr[0] ) ) { 
      parameterPtr++;  
    } 
    while ( ( n = sscanf( parameterPtr, "(%lf,%lf)%[^\n]",  
        &x, &y, parameterPtr ) ) >= 2 ) { 
      if ( n == 2 ) { 
        // Das letzte Element gelesen. ParameterPtr von Hand setzen 
        parameterPtr[0] = '\0';  
      } 
      if ( index >= count ) { 
        WARNING( Log::CONFIGURATION, "Too many elements given for array " + 
                 Ath98String(parameterName) );  
        break;  
      } 
      ArrayPtr->push_back( Vector( x, y) );
      index++;  
 
      while ( isspace( parameterPtr[0] ) ) { 
        parameterPtr++;  
      } 
    }               
    if ( n == 1 ) { 
      WARNING( Log::CONFIGURATION, "Incomplete element given for array " + 
               Ath98String(parameterName) );  
      return n;  
    } 
    if ( index < count ) { 
      WARNING( Log::CONFIGURATION, "Not enough elements given for array " +  
               Ath98String(parameterName) );  
    }  
    else { 
      n = 2;  
    } 
  }  
  else { 
    n = 1;  
  } 
  return n;  
} 
 
int Configuration::scanArrayOfInt( char *buffer, vector<int> *ArrayPtr )
{ 
 
  int count;  
  char parameterName[256];  
  char parameterBuffer[256];  
  int n;  
  n = sscanf(buffer, "%s %d %[^\n]", parameterName, &count, parameterBuffer);  
  if ( n == 3 && count >=0 ) { 
    ArrayPtr->clear();
    int index = 0;
    double value;  
    char *parameterPtr = parameterBuffer;  
    while ( isspace( parameterPtr[0] ) ) { 
      parameterPtr++;  
    } 
    while ( ( n = sscanf( parameterPtr, "%lf %[^\n]",  
        &value, parameterPtr ) ) >= 1 ) { 
      if ( n == 1 ) { 
        // Das letzte Element gelesen. ParameterPtr von Hand setzen 
        parameterPtr[0] = '\0';  
      } 
      if ( index >= count ) { 
        WARNING( Log::CONFIGURATION, "Too many elements given for array " + 
                 Ath98String(parameterName) );  
        break;  
      } 
      ArrayPtr->push_back( value );
      index++;  
 
      while ( isspace( parameterPtr[0] ) ) { 
        parameterPtr++;  
      } 
    }               
    if ( index < count ) { 
      WARNING( Log::CONFIGURATION, "Not enough elements given for array " +  
               Ath98String(parameterName) );  
    }  
    else { 
      n = 2;  
    } 
  }  
  else { 
    n = 1;  
  } 
  return n;  
} 
 
void Configuration::readConfigurationFile(Ath98String configurationFile) { 
  int i;  
  FILE *filePointer;  
  char *buffer;  
   
  /* Oeffnen des Files */ 
  if ((filePointer = fopen(configurationFile.cString(),"r")) == NULL) { 
    // WARNING( Log::CONFIGURATION, "Can't open config file " + 
    //         configurationFile ); 
    cerr << "WARNING: Can't open config file " << configurationFile << endl;
    return;  
  } 
 
  /* Einlesen der Parameter */ 
 
  while( ( buffer = readLine( filePointer ) ) != NULL) { 
    int n;  
    char parameterName[256];  
    char onOff[16] ;  
 
    n = sscanf(buffer,"%s", parameterName) ;  
    if (n < 1) { 
      WARNING( Log::CONFIGURATION, "Illegal line: (" + Ath98String(buffer) + 
               ")\n" );  
      continue ;  
    } 
 
    for(i = 0;  serverOptions[i].pointer != NULL;  i++) { 
      if( strcmp(parameterName, serverOptions[i].optionName) ) { 
        continue;  
      } 
 
      // Parameter gefunden 
      switch( serverOptions[i].type ) { 
        case DOUBLE: 
          n = sscanf(buffer, "%s %lf", parameterName,  
              (double *)serverOptions[i].pointer) ;  
          break ;  
        case INT: 
          n = sscanf(buffer, "%s %d", parameterName, 
              (int *)serverOptions[i].pointer) ;  
          break ;  
        case OPTIONAL: 
          // Wenn es da steht, dann ist es yes 
          // sonst der Default-Wert. 
          n = 2;  
          *((int *)serverOptions[i].pointer) = YES;  
          break;  
        case BOOL: 
          n = sscanf(buffer, "%s %s", parameterName, onOff);  
          if ( ( strcasecmp( onOff, "yes" ) == 0 ) ||  
               ( strcasecmp( onOff, "on" ) == 0 ) ) { 
            *((bool *)serverOptions[i].pointer) = YES;  
          } 
          else { 
            *((bool *)serverOptions[i].pointer) = NO;  
          }   
          break ;  
        case STRING: 
          n = sscanf(buffer, "%s %s", parameterName, 
              (char *)serverOptions[i].pointer) ;  
          break ;  
        case ATH98VECTOR: 
          double x, y;  
          n = sscanf(buffer, "%s (%lf, %lf)", parameterName, &x, &y);  
          if ( n == 3 ) { 
            *((Vector *)serverOptions[i].pointer) = Vector( x, y);  
          }  
          else { 
            n = 1;  
          } 
          break ;  
        case ARRAY_OF_VECTOR: 
          n = scanArrayOfVector( buffer,  
              (vector<Vector> *)serverOptions[i].pointer );
          break;  
        case ARRAY_OF_INT: 
          n = scanArrayOfInt( buffer,  
              (vector<int> *)serverOptions[i].pointer );
          break;  
      } 
 
      if (n < 2) { 
        WARNING( Log::CONFIGURATION, "Illegal line (" +  
                 Ath98String(parameterName) + ")\n" );  
      } 
      break ;  
    } 
     
    if (serverOptions[i].pointer == NULL) { 
      WARNING( Log::CONFIGURATION, "Illegal line (" + 
               Ath98String(parameterName) + ")\n" );  
    } 
  } 
 
  fclose(filePointer);  
 
  calculateDependentParameters();  
 
  return;  
} 
 
void Configuration::calculateDependentParameters()  
{ 
  kickMargin = kickMarginServer * maxKickDist;  
 
  halfTime = halfTimeInSecs * 1000 / simulatorStep;  
   
  visibleAngle = radFromDegree(visibleAngleInDegree);  
   
  sendStepNorm = sendStep/4;  
   
  kickRange = kickMargin + playerSize + ballSize;  
   
  maxShotLength = ballSpeedMax *  
      ( ( pow( ballDecay, maxStepsOfBallMovement ) - 1 ) / ( ballDecay - 1 ) );  
   
  averageShotLength = maxEffectiveBallSpeed *  
      ( ( pow( ballDecay, averageStepsOfBallMovement ) - 1 ) / 
        ( ballDecay - 1 ) );  
 
  distPerStaminaPoint = dashPowerRate * 
      ( ( pow( playerDecay, maxStepsOfPlayerMovement ) - 1 ) /  
        ( playerDecay - 1 ) );  
   
  sumOfGoalKickUtilityWeights = 0;  
  for ( unsigned int i = 0;  i < goalKickUtilityWeights.size();  i++ ) {
    sumOfGoalKickUtilityWeights += goalKickUtilityWeights[i];  
  } 
  conserveStaminaGradient = ( conserveStaminaActivation -  
                              conserveStaminaUtilityIncrease.x() ) /  
    conserveStaminaUtilityIncrease.y();
  defaultStaminaLimit =
    max( playerEffortDecThr, playerRecoverDecThr ) * playerStaminaMax;

  bzero(decodetable, 256 * sizeof(unsigned char));
  for( int i =0; i < encodetablelength; ++i ) {
    decodetable[encodetable[i]]=i;
  }
} 
 
void Configuration::logCurrentParameterValues()  
{ 
    if ( IS_ACTIVE( Log::CONFIGURATION, 6 ) ) { 
    for(int i = 0;  serverOptions[i].pointer != NULL;  i++) { 
      INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
	    serverOptions[i].optionName << "\t Value: " );  
      switch(serverOptions[i].type) { 
      case DOUBLE: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream <<
              (*((double *)serverOptions[i].pointer)) << "\n");  
        break;  
      case INT: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
              (*((int *)serverOptions[i].pointer)) << "\n");  
        break;  
      case BOOL:
      case OPTIONAL:
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
              (*((int *)serverOptions[i].pointer)) << "\n");  
        break;  
      case STRING: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
              (char *)serverOptions[i].pointer << "\n");  
        break;  
      case ATH98VECTOR: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
              (*((Vector *)serverOptions[i].pointer)) << "\n");  
        break;  
      case ARRAY_OF_VECTOR: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream <<
              /*(*((vector<Vector> *)*/serverOptions[i].pointer/*))*/);          // ACHTUNG HACK !!!! (MG)
        break;  
      case ARRAY_OF_INT: 
        INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream <<
              /*(*((vector<int> *)*/serverOptions[i].pointer/*))*/);             // ACHTUNG HACK !!!! (MG)
        break;  
      } 
    } 
     
    // Die automatischen Parameter muessen hier extra angegeben werden. 
    INFO( Log::CONFIGURATION , 6 , idString() , globalLog->infoStream << 
	  "\nDep. parameter kickMargin: " <<kickMargin <<  
	  "\nDep. parameter halfTime : " << halfTime <<  
	  "\nDep. parameter visibleAngle : " << visibleAngle << 
	  "\nDep. parameter kickRange : " << kickRange << 
	  "\nDep. parameter maxShotLength : " << maxShotLength << 
	  "\nDep. parameter averageShotLength : " <<  averageShotLength << 
	  "\nDep. parameter distPerStaminaPoint : " << distPerStaminaPoint <<   
	  "\nDep. parameter halfTime : " << halfTime <<  
	  "\nDep. parameter sumOfGoalKickUtilityWeights : " << 
	  sumOfGoalKickUtilityWeights <<  
	  "\nDep. parameter conserveStaminaGradient: " << 
	  conserveStaminaGradient << 
          "\n" );  
  } 
} 
 
void Configuration::readTable()
{
  std::ifstream input("tabelle.dat");
 
  if( !input ){ 
    cerr << "Kein input-file .tabelle.dat. gefunden -> exit !!" << endl;
    exit( -1 );  
  } 
   
  table.resize(51);  
  for(int i=0;  i< 51;  ++i){ 
    table[i].resize(72);  
    for(int j=0;  j< 72;  ++j){ 
      table[i][j].resize(28);  
    } 
  }// C++ - Buch 899 
   
  int erg = 0;  
  for( int ballSpeed = 0;  ballSpeed < 27;  ++ballSpeed ){ 
    // alle Ballbeschleunigungen mit Intervall 0.1 -> 27Stueck 
    for( int row = 0;  row < 51;  ++row ){ 
      for( int column = -20;  column < 51;  ++column ){ 
	//fuer jede Gegnerposition 
	input >> erg;  
	table[row][column+20][ballSpeed] = erg;  
      } 
    } 
  } 
 
} 
 
 
;  
