#ifndef _CONFIGURATION_H_
#define _CONFIGURATION_H_

#include <stdio.h>
#include <string>
#include <vector>

#include "Log.h"
#include "Common.h"
#include "Vector.h"
#include "Ath98String.h"


/**
  Konfiguration des Servers und des Agenten.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Configuration.h} | [\URL[Implementierung]{src/Configuration.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Configuration.h,v $
  *   $Revision: 1.1.1.1 $
  *   $Date: 2001/07/31 22:23:08 $
  *   $Author: rami $ 
  *
  **************************************************
  \end{verbatim} }*/
class Configuration
{
public:
  
  /** Konstruktor */
  Configuration();
  
  /** Liest eine Konfigurationsdatei ein. agent.conf und server.conf werden
  gleich behandelt. */
  void readConfigurationFile(Ath98String configurationFile);
  
  /** Schreibt aktuelle Parameter in das Log.
    @see Log */
  void logCurrentParameterValues();
  
#define MAX_FILE_LEN 256

  /** @name Parameter des SoccerServers */
  //@{
    /** zunaechst die Parameter, die in der offizillen server.conf erscheinen, in der
    Reihenfolge ihres Auftretens */
    /// goal width    
    double goalWidth;
                   
    /// player size    
    double playerSize;              
    /// player decay
    double playerDecay;             
    /// player rand
    double playerRand;              
    /// player weight
    double playerWeight;            
    /// player speed max
    double playerSpeedMax;
    /// player acceleration max
    double playerAccelMax;  
            
    /// player stamina max
    double playerStaminaMax;        
    /// player stamina inc
    double playerStaminaInc;
            
    /// player recovery decriment threshold
    double playerRecoverDecThr;     
    /// player recovery decriment
    double playerRecoverDec;        
    /// player recovery min
    double playerRecoverMin;  
          
    /// player dash effort decriment threshold
    double playerEffortDecThr;      
    /// player dash effort decriment
    double playerEffortDec;         
    /// player dash effort incriment threshold
    double playerEffortIncThr;      
    /// player dash effort incriment
    double playerEffortInc; 
    /// player dash effort min
    double playerEffortMin; 
            
    /// player hear_capacity_max
    int hearCapacityMax;            
    /// player hear_capacity_inc
    int hearCapacityInc;            
    /// player hear_capacity_decay
    int hearCapacityDecay;          
                
    /// intertia moment for turn
    double inertiaMoment;

    /// interval of sense_body
    double senseBodyStep;

    /// goalie catchable area length
    double catchAreaLength;         
    /// goalie catchable area width
    double catchAreaWidth;     
         
    /// goalie catchable probability
    double catchProbability;         
     
    /// goalie catch ban cycle
    int catchCycle;                 
    
    /// goalie max moves after catch
    int goalieMaxMovesAfterCatch;
    
    /// ball size
    double ballSize;                
    /// ball decay
    double ballDecay;               
    /// ball rand
    double ballRand;                
    /// ball weight
    double ballWeight;              
    /// ball speed max
    double ballSpeedMax;
    /// ball acceleration max
    double ballAccelMax;     
          
    /// wind force
    double windForce;               
    /// wind direction
    double windDirection;           
    /// wind random
    double windRand;                

    /// kickable margin
    double kickMarginServer;

    /// corner kick margin
    double cornerKickMargin;        

    /// dash power rate
    double dashPowerRate;           
    /// kick power rate
    double kickPowerRate;
    
    /// kick randomness
    double kickRand;
               
    /// visible angle
    double visibleAngleInDegree;
            
    /// audio cut off Dist
    double audioCutOffDistance;
         
    /// quantize step of Dist
    double distQuantStep;       
    /// quantize step of Dist for landmark
    double landmarksQuantStep;      

    /// max power
    double maxPower;                
    /// min power
    double minPower;                
    /// max moment
    double maxMoment;               
    /// min moment
    double minMoment;               
    /// max neck moment
    double maxNeckMoment;               
    /// min neck moment
    double minNeckMoment; 
    /// max neck angle
    double maxNeckAngle;               
    /// min neck angle
    double minNeckAngle;               

    /// port number
    int portNumber;                 

    /// port number des Trainers der alles darf
    int trainerPortNumber;            

    /// Default upper limit of length of online coach's messages
    double sayCoachCntMax;
    
    /// Default interval of online coach's look
    double sayCoachMsgSize;


    /// Coach language message time window size
    int clangWinSize;
    /// Coach language define messages per window
    int clangDefineWin;
    /// Coach language meta  messages per window
    int clangMetaWin;
    /// Coach language advice  messages per window
    int clangAdviceWin;
    /// Coach language info messages per window
    int clangInfoWin;
    /// Coach language message delay
    int clangMessDelay;
    /// Coach language messages per cycle
    int clangMessPerCycle;

    /// Default upper limit of the number of online coach's message
    double sendViStep;
        
    /// simulator step interval msec
    int simulatorStep;              
    /// udp send step interval msec
    int sendStep;                   
    /// udp recv step intercal msec
    int receiveStep;                

    /// half time
    int halfTimeInSecs;             

    /// cycles to wait until dropping the ball a utomatically
    int dropBallTime;
    
    /// string size of say message
    int sayMessageSize; 
                
    /// flag for using off side rule
    bool useOffSide;                

    /// offside active area size
    double offsideArea;

    /// flag for forbid kick off offside
    bool forbidKickOffOffside;  

    /// flag for verbose mode
    bool verbose;
    
    /// flag for record log
    int recordVersion;

    /// flag for client command log recording
    bool recLog;
 
    /// flag for send client command log
    bool sendLog;

     /// offside kick margin
    double offsideKickMargin;    

    /// flag for writing ms between cycles
    bool logTimes;
    
    /// server log file name
    char logFile[MAX_FILE_LEN];     

    /// log file name for recording
    char recFile[MAX_FILE_LEN];     
    /// Flag for recording messages
    bool recordMessages;

    /// coach with referee mode
    bool coachWithRefereeMode;      

    /** es folgen weitere Parameter, deren Wert z.Z. nicht in der server.conf
    gesetzt wird. */
    /// control radius
    double controlRadius;           
    /// visible Dist
    double visibleDistance;         
    /// slow down factor
    int slowDownFactor;
    /// quantize step of Dir
    double dirQuantStep;    
    /// wind factor is none
    bool isWindFactor;              
    /// wind factor is random
    bool windRandom;                
    /// port number des Coaches der im Turnier erlaubt ist
    int coachPortNumber;            
    /// coach mode
    bool coachMode;                 
    /// old format for hear command (coach)
    bool oldCoachHear;              
    /// log file name for replay
    char replay[MAX_FILE_LEN];          
    /// start goal left
    int startGoalL;
    /// start goal right
    int startGoalR;
    /// fullstate information left
    bool fullstateL;
    /// fullstate information right
    bool fullstateR;
    
    /** Die folgenden Parameter wurden mal fuers Lernen benoetigt und werden vom
    offiziellen SoccerServer nicht verwendet */
    /// learn file name for learning
    char learnFile[MAX_FILE_LEN];
    /// flag for learning
    bool learning;
  //@}

  // double playerRecoverInit;       /* player recovery init */
  // double playerEffortInit;        /* player dash effort init */
  // double controlRadiusWidth;      /* (control radius) - (player size) */

    
  /** @name Parameter des Agent-Files */
  //@{
    /// Wenn auf 1 gesetzt legt der Spieler ein Logfile an, da pro Simstep
    /// die von ihm geglaubte Welt abspeichert (also seine currentSituation)
    bool situationLog;
    /// 0 meint direkt 1 weit weg vom Koerper
    double optimalKickDist;
    /// maximale Kick-Entfernung zum Ball
    double maxKickDist;
    /// minimale Schu\3entfernung (beim Passen)
    double minShotLength;
    /// Zahl der gespeicherten Situationen
    int historyCount;            
    /// die maximale Laenge eines Plans
    int maxLengthOfPlan;         
    /// zu tolerierende Kosten zur Stabilitaet der Wuensche
    int toleratedStabilityCost;  
    /// Nutzen, ab dem ein Constraint gefordert wird
    int minConstraintUtility;    
    /// Nutzen, ab dem ein Desire eventuell umgesetzt wird
    int minDesireUtility;
    /// Minimale Distanz, die ein Spieler ohne Initiative vom Ball haben mu\3.
    double enforcedMinDistToBall;
    /// Zahl von Takten nach einem Dash, die fuer die Spielersimulation noch
    /// beruecksichtigt werden.
    int maxStepsOfPlayerMovement;
    /// Zahl von Takten nach einem Kick, die fuer die Ballsimulation noch
    /// beruecksichtigt werden.
    int maxStepsOfBallMovement;
    /// Effektive Ballgeschwindigkeit, die ein Spieler mit einem kickBall erzielen
    /// kann
    double maxEffectiveBallSpeed;
    /// Zahl von Takten, die ein sich freilaufender Spieler ber\"ucksichtigt
    /// um festzustellen ob er im Einflu\3bereich des kickenden Spielers steht.
    int averageStepsOfBallMovement;
    /// Rollen der Spieler
    vector<int> roles;
    /// Aufstellung, wenn wir Anstoss haben
    vector<Vector> kickoffPositionsOffensive;
    /// Aufstellung, wenn der Gegner Anstoss hat
    vector<Vector> kickoffPositionsDefensive;
    /// Aufstellung waehrend des Spiels offensiv
    vector<Vector> homePositionsOffensive;
    /// Aufstellung waehrend des Spiels defensiv
    vector<Vector> homePositionsDefensive;
    /// maximale baseUtility fuer GoalKickOption
    int maxGoalKickUtility;
    /// Gewichte fuer GoalKickUtilityBerechnung
    vector<int> goalKickUtilityWeights;
    /// Meter, die der St\"urmer vom Pfosten gesehen zur Mitte hin h\"lt
    double goalReduce;
    /// Minimale angenommene Ballgeschwindigkeit in Dezimetern
    double initialBallSpeed;
    /// maximale baseUtility fuer ForwardPassOption
    int maxForwardPassUtility;
    /// minimale baseUtility fuer ForwardPassOption
    int minForwardPassUtility;
    /// maximale baseUtility fuer DirectPassOption
    int maxDirectPassUtility;
    /// maximale baseUtility fuer BackwardPassOption
    int maxBackwardPassUtility;
    /// maximale baseUtility fuer DoublePassOption
    int maxDoublePassUtility;
    /// maximale baseUtility fuer DribbleOption
    int maxDribbleUtility;
    /// maximale baseUtility fuer WaitOption
    int maxWaitUtility;
    /// Laenge des beim Dribbeln betrachteten Sektors (in m)
    double dribbleSectorLength;    
    /// Breite des beim Dribbeln betrachteten Sektors (in Grad)
    double dribbleSectorWidth;
    /// maximale baseUtility fuer ControlBallOption
    int maxControlBallUtility;
    /// minimaler Nutzen fuer CollectDataOption
    int minCollectDataUtility;
    /// normaler Nutzen fuer CollectDataOption
    int normalCollectDataUtility;
    /// maximaler Nutzen fuer CollectDataOption
    int maxCollectDataUtility;
    /// minimaler Nutzen fuer GoToBaseOption
    int minGoToBaseUtility;
    /// normaler Nutzen fuer GoToBaseOption
    int normalGoToBaseUtility;
    /// maximaler Nutzen fuer GoToBaseOption
    int maxGoToBaseUtility;
    /// Abstand vom Ball in der X-Koordinate, den ein Verteidiger bei
    /// Nachruecken zum Ball versucht einzuhalten
    double defenderMoveForwardBallDistActivation;
    /// maximale X-Position die ein nachrueckender Verteidiger einnimmt.
    double defenderMoveForwardStopLine;
    /// Abstand vom Ball in der X-Koordinate, den ein Mittelfeldspieler bei
    /// Nachruecken zum Ball versucht einzuhalten
    double midfieldMoveForwardBallDistActivation;
    /// maximale X-Position die ein nachrueckender Mittelfeldspieler einnimmt.
    double midfieldMoveForwardStopLine;
    /// minimale X-Position die ein zurckweichender Mittelfeldspieler einnimmt
    double midfieldDrawbackStopLine;
    /// minimale X-Position die ein zurckweichender Strmer einnimmt
    double forwardDrawbackStopLine;
    /// Genauigkeit mit der ein Spieler auf seine BasePosition laufen soll
    double gotoBasePositionAccuracy;
    /// Genauigkeit mit der ein Spieler auf ein InterceptPosition laufen soll
    double getIntoSpaceAccuracy;
    /// Genauigkeit mit der ein Spieler zum Ball laufen soll
    double interceptBallAccuracy;
    /// StandardGenauigkeit, falls TargetId unklar
    double standardTargetAccuracy;
    /// normaler Nutzen fuer die InterceptBallOption
    int normalInterceptBallUtility;
    /// normaler Nutzen fuer die GetIntoSpaceOption
    int normalGetIntoSpaceUtility;
    /// Distanz von der Baseposition bis zu der sich der Spieler freilaeuft.
    double leaveBasePositionRange;
    /// Distanz vom Abfangpunkt des Balles ab der sich ein Spieler freilaeuft
    int getIntoSpaceDist;
    /// minimale Distanz vom Abfangpunkt bei der freigelaufen werden soll.
    double getIntoSpaceMinDist;
    /// Simulationsschritte bis zum Abfangpunkt ab der sich ein Spieler
    /// freilaeuft
    int getIntoSpaceSimSteps;
    /// Entfernung des Spielers von seiner BasePostion, bis zu der er sich bei
    /// Aktivierung der GetIntoSpaceOption zum Ball dreht und diesen hinterher
    /// schaut
    double getIntoSpaceAvoidBasePositionDist;
    /// Fenstergr\"o\3e vom Ball in Richtung des gegnerischem Tors, in dem
    /// ein Spieler versucht parallel mitzulaufen
    double getIntoSpaceWindowEnemy;
    /// Fenstergr\"o\3e vom Ball in Richtung des eigenen Tors, in dem
    /// ein Spieler versucht parallel mitzulaufen
    double getIntoSpaceWindowSelf;
    /// Minimale X-Position von der ab immmer GetIntoSpace
    /// gemacht werden soll.
    double getIntoSpaceMinPosition;
    /// Minimale X-Position, ab der ein Spieler parallel mitlaufen
    /// soll. (-52.5 == immer)
    double getIntoSpaceMinParallelPos;
    /// Distanz von einem gegenerischen Spieler, ab der sich der Spieler
    /// anfaengt aktiv freizulaufen.
    double avoidCoverActivationDist;
    /// Entfernung die der Spieler vom Gegner haben will, wenn er sich beim
    /// freilaufen vor diesen stellt.
    double avoidCoverBestPositionDist;
    /// maximale Zeit die ein Spieler einen Plan verfolgen soll
    int maxForceContinuationTime;
    /// Entfernung zum Ball, ab dem er die Situation genauer betrachtet
    double distToBallForEvaluation;
    /// Schrittezahl ab der, die Berechnung von calculatedStepsToBall abbricht
    int maxStepsToCalculate;
    /// Aktivierung der einzelnen Modi f\"ur die Debug-Ausgabe
    vector<int> debugModes;
    /// Aktivierung der einzelnen Modi f\"ur die LearnLog-Ausgabe
    vector<int> logModes;
    /** Stamina-Wert unterhalb dessen das ConserveStaminaConstraint
      aktiv wird. */ 
    int conserveStaminaActivation;
    /** Vektor, der die Steilheit (Steigung) des linearen Anstiegs
      des Nutzens des ConserveStaminaConstraint festlegt.  x: stamina, y:
      utility. y mu\3 kleiner als conserveStaminaActivation sein. */
    Vector conserveStaminaUtilityIncrease;
    /// maximaler Nutzen des ConserveStaminaConstraint
    int maxConserveStaminaUtility;
    /// maximaler Nutzen des AvoidOffsideConstraint
    int maxAvoidOffsideUtility;
    /// Bonus f\"ur das Staminalimit, wenn das AvoidOffsideConstraint aktiviert
    /// wird
    double avoidOffsideStaminaLimitBonus;
    /// Spielt der Agent ein echtes Spiel?
    bool tournamentMode;
    /// Variable die aussagt ob Kommunikation genutzt werden soll.
    bool useCommunication;
    /// Timeout fuer Kommunikation mit dem Server in Sekunden
    int communicationTimeout;
    /// DefaultStaminaLimit ist das harte vom Server vorgegebene Limit
    double defaultStaminaLimit;
    /// Staminalimit fuers Dribbeln
    double dribbleStaminaLimit;
    /// Staminalimit fuers Ball abfangen
    double interceptBallStaminaLimit;
    /// Staminalimit fuers Ball abfangen
    double goalieInterceptBallStaminaLimit;
    /// Staminalimit fuer die Rueckkehr zur Basisposition
    double gotoBaseStaminaLimit;
    /// Staminalimit fuers Freilaufen
    double positioningStaminaLimit;
    /// Staminalimit fuers Positionieren
    double getIntoSpaceStaminaLimit;
    /// Toleranz bei wieviel Schritten Differenz zum schnellsten Spieler,
    /// dennoch zum Ball gerannt werden soll
    int interceptionTolerance;
    /// Distanz vom Tor, in der sich der Torwart normalerweise aufhaelt
    double goalieDistFromGoal;
    /// Distanz, ab der der Torwart den Ball oder andere Spieler als relevent
    /// ansieht
    double goalieDangerZone;
    /// Wenn ein nicht freundlicher Spieler naeher als diese Distanz kommt,
    /// wird nicht mehr gedribbelt (sondern hoffentlich gepasst).
    double emergencyPassDist;
    /// Voraussichtliche Reaktionszeit des gegnerischen Torwarts in Takten 
    int reactionTimeSteps;
    /// Bonus fuer das Dribbeln, wenn der Agent sich in Abseitsverdaechtiger
    /// Position befindet
    int dribbleInOffsideBonus;
    /// Wartezeit bei Freistoss
    int waitTimeDuringFreeKick;
    /// Wartezeit zwischen einzelnen Rufen des Torwarts zur 
    /// Verteidigungskoordination
    int minWaitTimeForDefendCoordination;
    /// Alphabet einer "SAY Message" zum verschluesseln  
    unsigned char encodetable[256];
    /// Alphabet einer "SAY Message" zum entschluesseln
    unsigned char decodetable[256];
    /// Laenge des Alphabets einer "SAY Message"
    int encodetablelength;
    /// Neck Priorit\"at f\"ur den Ball
    int neckTurnPrioBall;
    /// Neck Priorit\"at f\"ur die Spielfeldabdeckung
    int neckTurnPrioRegion;
    /// Neck Priorit\"at f\"ur die Spieler 
    int neckTurnPrioPlayer;
    /// Schwellwert, ab dem die Prioritaet fuer den Wunsch Nackendrehung
    /// des Weltmodells von LOW auf AVG gesetzt wird
    int raiseNeckPriorityBoundary;
    /// Ob der Coach ein aufgezeichnetes Spiel auswertet
    bool coachWithRecordFile;
    /// Ob der Coach interaktiv genutzt wird (Eingabe per stdin)
    bool coachUseInteractive;

  //@}

  /** @name Parameter fr das Online-Wissensakquirieren */
  //@{
    /// Aufloesung des gerasterten Spielfeldes, zum Lernen
    /// X-Koordinate (die laengere Seite des Spielfeldes)
    int xResolution;
    /// Y-Koordinate
    int yResolution;
    /// Staerke der Faerbung des Spielers
    int incPlayer;
    /// Staere der Faerbung des Balls
    int incBall;
    /// Staerke des Verblassens des Spielers
    int decPlayer;
    /// Staere des Verblassens des Balls
    int decBall;
    /// Schwellwert, ab dem ein Rasterkaestchen als Schwerpunkt gilt
    int threshold;
    /// minimale Laenge einer Schwerpunkt-Kante
    int minEdgeLength;
    /// maximale Laenge einer Schwerpunkt-Kante
    int maxEdgeLength;
    /// Abstand vom eigenen Tor-Aus, innerhalb dem Manndeckung erfolgt
    int manGuardingBorder;
    /// Abstand des Agenten von dem Schwerpunkt, der naeher dem eigenen Tor ist
    int distFromOpponent;
    /// Gewichte der Kriterien fuer die Bewertung der ermittelten Position:
    /// fuer den Abstand der home-Position davon
    double weightDistFromHome;
    /// fuer den Abstand von der eigenen aktuellen Position davon
    double weightDistFromCurrent;
    /// fuer die Haeufigkeit mit der der Ball auf der Schwerpunkt-Kante langlief
    double weightBallAppearance; 
    /// fuer die Farbe der Schwerpunkt-Kante auf der sich diese Position 
    /// befindet 
    double weightEdgeColour;   
    /// Abstand Agent - Ball, ab dem "GoToBaseOption" MAX_UTILITY zurueckgibt
    int maxUtilityDist;
    /// Abstand Agent - Ball, unter dem "GoToBaseOption" MIN_UTILITY zurueckgibt
    int minUtilityDist;
    /// Abstand Agent-Ball, unter dem der Ball gelernt wird,
    int ballLearnDist;
    /// maximaler Abstand des Agenten von der Home-Position zur Akzeptanz 
    /// einer ermittelten basePosition
    int maxDistFromHome;
    /// Abstand vom gegnerischen Tor-Aus, innerhalb dem der Gegner nicht
    /// gelernt
    /// wird, um nicht den gegnerischen Torwart zu lernen
    int learnNoGoalieDist;
    /// Abstand des Verteidigers vom Ball bei Einwuerfen ec
    double defenderFreeKickDist;
    /// Betrag um den verteidigende Spieler bei Einwuerfen an den Ball 
    /// nachruecken.
    double defenderFreeKickMoveUp;
    /// Abstand des Balles vom eigenen Tor, ab dem eine zu verteidigende 
    /// Situation vorliegt.
    double dangerousRangeFromGoal;
    /// Maximale Abweichung/Toleranz f\"ur Kickroutine am Aussenradius.
    double outerKickCircleTolerance;
    /// Maximale Abweichung/Toleranz f\"ur Kickroutine am Innenradius.
    double innerKickCircleTolerance;
    /// Maximale Abweichung/Toleranz f\"ur Kickroutine fuer Zwischenziele.
    double intermediateKickTolerance;
    /// Bezeichner f\"ur unsere Nachrichtenzeichenketten beim Weltmodelruf.
    int ourMessageIdentifier;
    /// Vorsprung in Takten, den unsere Spieler bei einem Pass haben
    /// muessen, damit er als erfolgreich bewertet wird
    int passLeadForTeammates;
    /// Offest, den der Goalie auf den passLeadForTeammates drauf addiert,
    /// damit nur sichere Paesse gespielt werden.
    int passLeadForTeammatesGoalieOffset;
    /// Winkel in Grad fuer die Sicherheit bei der Ballueberwachung durch den Nacken
    double safetyNeckAngleToBall;
    /// Soll Nacken benutzt werden ? ( 1= Ja, 0 = Nein )
    int shouldNeckBeUsed;
    /// Entfernung innerhalb der ein nicht zuordenbarer Spieler an den Rand
    /// des Sichtbereiches des Spielers verschoben wird.
    double escapeFieldOfViewDist;
    /// Maximale Dist bei der noch Spieler aufeinander abgebildet werden.
    double maximalMergeDist;
    /// Distanz bis zu der ein Spieler einen anderen Spieler was zuruft.
    double distToPlayerForSay;
    /// Winkeldifferenz zwischen eigenen Blickwinkel und des Partners 
    /// Vorzugsrichtung die fuer einen Ruf akzeptiert wird (im Gradmass).
    double sayAngleDifference;
    
    double maxGoalKickDist;
  //@}

  /** @name fuer den Analyzer */
  //@{
    /// Intervall in dem die Positionierung aktualisiert wird
    int calculatePositionInterval;
    /// Formation mit der gespielt werden soll.
    //int formation;
  //@}
    
  /** @name Abhaengige, automatisch berechnete Parameter */
  //@{
    /// korigiertes kickMargin (kickMarginServer * maxKickDist)
    double kickMargin;
    /// (halfTimeInSecs * 1000 / simulatorStep)
    int halfTime;                   
    /// (radFromDegree(visibleAngleInDegree))
    double visibleAngle;            
    /// (sendStep /= 4)
    int sendStepNorm;               
    /// (kickMargin+playerSize+ballSize)
    double kickRange;
    ///  maxEffectiveBallSpeed * ( ( ballDecay^maxStepsOfBallMovement - 1 ) / 
    ///                   (ballDecay - 1 ) )
    double maxShotLength;
    ///  maxEffectiveBallSpeed * ( ( ballDecay^averageStepsOfBallMovement - 1 ) / 
    ///                   (ballDecay - 1 ) )
    double averageShotLength;
    ///  dashPowerRate * ( ( playerDecay^maxStepsOfPlayerMovement - 1 ) /
    ///                    ( playerDecay - 1 ) )
    double distPerStaminaPoint;
    /// Summe der goalKickUtility-Gewichte
    int sumOfGoalKickUtilityWeights;
    /// linearer Anstieg des ConserveStaminaConstraints
    double conserveStaminaGradient;
  //@}
    
private:
  ///
  void calculateDependentParameters();

  /** Liest eine Zeile aus einer geoeffneten Datei.
    Dabei werden Zeilen, die mit einem Backslash (\) gefolgt von einem
    newline (\n) enden, aneinandergehaengt. Leerzeilen und Zeilen, die mit einem
    (#) beginnen, werden ignoriert. Die Maximallaenge einer Zeile ist 
    \Ref{MAX_MESSAGE}.
    @param filePointer Zeiger auf Dateistruktur, so wie sie z.B. von fopen
      geliefert wird.
    @return Zeiger auf einen Buffer, der die naechste Zeile enthaelt, oder NULL,
      wenn das Dateiende erreicht worden ist. */
  char *readLine( FILE *filePointer );
  
  /** Scant ein Array von Vektoren ein. 
    @param buffer Der Speicherbereich, in dem der Ath98String steht.
    @param ArrayPtr Zeiger auf das Array, das initialisiert werden soll.
  */
  int scanArrayOfVector( char *buffer, vector<Vector> *ArrayPtr );

  /** Scant ein Array von Integerwerten ein. 
    @param buffer Der Speicherbereich, in dem der Ath98String steht.
    @param ArrayPtr Zeiger auf das Array, das initialisiert werden soll.
  */
  int scanArrayOfInt( char *buffer, vector<int> *ArrayPtr );
  
  /** Liest die Tabelle mit den Intercept-Takten ein.
   */
  void readTable();

  /** Liefert einen String mit Klassenbezeichnung zurueck. */
  string idString() const;

  ///
  typedef struct {
    char optionName[64];
    void *pointer;
    int type;
    double value;
  } options;
  
  ///
  options *serverOptions; 

#define INT               1
#define DOUBLE            2
#define BOOL              3
#define STRING            4
#define OPTIONAL          5
#define ATH98VECTOR       6
#define ARRAY_OF_VECTOR   7
#define ARRAY_OF_INT      8
#define NONE              99

  /** String der die Kurzkennung der Klasse enthlt */
  string theIdString;



};

inline string Configuration::idString() const
{
  return theIdString;
}  
  
/** Global verfuegbares Configuration-Object */
extern Configuration *globalConf;

/** Global verf"ugbare Tabelle zur Intercept-Berechnung */
extern vector < vector < vector  <int> > > table;
/** Global verf"ugbare Tabelle f"ur die Ballbeschleunigungen */
extern vector< vector < double > > speedVectors;

#endif




