#ifndef _STRAIGHT_LINE_H_
#define _STRAIGHT_LINE_H_

#include "Common.h"
#include "Vector.h"
#include <string>

/* Moeglichkeiten, wie eine Gerade initialisiert werden kann:
   Moegliche Werte sind {POINT, DIRECTION}. */
enum GeradeInitEnum {POINT, DIRECTION};

/**
  Diese Klasse beschreibt eine 2D-Gerade.
  Die Gerade wird intern durch einen Punkt und
  einen Richtungsvektor repraesentiert.
  Der Richtungsvektor wird stets normalisiert, d.h. er hat immer die
  Laenge 1.

  {\small {\bf Quelltext}

  [\URL[Header]{src/StraightLine.h} | [\URL[Implementierung]{src/StraightLine.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: StraightLine.h,v $
  *   $Revision: 1.1.1.1 $
  *   $Date: 2001/07/31 22:23:08 $
  *   $Author: rami $ 
  *
  **************************************************
  \end{verbatim} }*/

class StraightLine{
 public:
    /** @name Gerade anlegen */
    //@{
    /** neue leere Gerade anlegen */
    StraightLine();



    /** Neue Gerade.
      @param aPoint Ein Punkt auf der Geraden.
      @param aPointOrDirection Ein Zweiter Punkt auf der Geraden bzw.
      der Richtungsvektor der Geraden.
      @param withDirection. Initialisierung mit zwei Punkten
      (DEFAULT == POINT)
      oder mit einem Punkt und dem Richtungsvektor (DIRECTION). */
    StraightLine( const Vector& aPoint,
                  const Vector& aPointOrDirection,
                  bool withDirection = POINT );
    //@}

    /** @name Berechnungen*/
    //@{
        
    /** Das Lot auf die Gerade berechnen   */ 
    Vector rightAngledVector() const;

    /** Einen Punkt auf der Geraden berechnen.
        @param amount Abstand des Punktes von Geradenpunkt Point. */
    Vector calcPoint( const double amount ) const;

    /** Abstand eines Punktes von der Geraden Bestimmen */
    double distTo(const Vector& aVector) const ;

    //@}
    /** @name Werte erfragen */
    /** Der Punkt. */
    Vector point() ;
    /** Der normalisierte Richtungsvektor. */
    Vector direction() ;
        
    /** Beschreibung als Ath98String fuer die Ausgabe. */
    Ath98String toString() const ;

 private:
    ///
    Vector thePoint;
    ///
    Vector theDirection;
}; /* Ende der Klassendefinition StraightLine */
    
#endif











