/* 
 * Aktueller Stand 
 * 
 *   $RCSfile: Vector.cc,v $ 
 *   $Revision: 1.1.1.1 $ 
 *   $Date: 2001/07/31 22:23:08 $ 
 *   $Author: rami $  
 * 
 */ 

#include <math.h>

#include "Vector.h"
#include "Common.h"
#include "Ath98String.h"

Vector::Vector(double value1, double value2,  
               CoordinateSystemEnum aCoordinateSystem = CARTESIAN,  
               AngleSystemEnum anAngleSystem = RADIAN )
{ 
  if ( value1 == UNDEF_DOUBLE || value2 == UNDEF_DOUBLE) { 
    theX = UNDEF_DOUBLE;  
    theY = UNDEF_DOUBLE;  
  } 
  else { 
    if ( aCoordinateSystem == CARTESIAN ) { 
      theX = value1;  
      theY = value2;  
    } 
    else if ( aCoordinateSystem == POLAR ) { 
      if( value1 < 0 ) { 
        value1 = fabs( value1 );
        value2 += Ath98_PI;
      } 
      if ( anAngleSystem == RADIAN ) { 
	      theX = value1 * cos( value2 );   
	      theY = value1 * sin( value2 );  
      } 
      else if ( anAngleSystem == DEGREE ) { 
	      theX = value1 * cos( radFromDegree( value2 ) );  
	      theY = value1 * sin( radFromDegree( value2 ) );  
      } 
      else{ 
        throwException( VectorException( "unknown AngleSystem" ) );  
      } 
    } 
    else{ 
      throwException( VectorException( "unknown CoordinateSystem" ) );  
    } 
  }  
} 
 
 
Vector::Vector() : 
  theX( UNDEF_DOUBLE ), 
  theY( UNDEF_DOUBLE )
{ 
} 
 
 
double Vector::angle( AngleSystemEnum anAngleSystem = RADIAN ) const 
{ 
  double myAngle;  
   
  if( !isValid() ){
    ERROR( "Angle from UNDEF Vector!" );
    return UNDEF_DOUBLE;
  }
  if ( theX == 0 ) {
    myAngle = M_PI_2;
    if ( theY < 0 ) { 
      myAngle += M_PI;
    } 
    else if( theY == 0 ){ 
      myAngle = 0;  
    }
  }
  else { 
    myAngle = atan( theY / theX );
    if( theY < 0 && theX > 0 ){
      myAngle += (M_PI*2);
    }
    else if( theX < 0 ){
      myAngle += M_PI;
    }
  }
  if ( anAngleSystem == RADIAN ) { 
    return myAngle;  
  } 
  else if ( anAngleSystem == DEGREE ) { 
    return ( degFromRadiant( myAngle ) );  
  } 
  else { 
    throwException( VectorException( "unknown AngleSystem" ) );  
  } 
} 
 
 
void Vector::setPolar( double aLength, double anAngle, 
                       AngleSystemEnum anAngleSystem = RADIAN ) 
{ 
 
  if ( aLength == UNDEF_DOUBLE || anAngle == UNDEF_DOUBLE ) { 
    theX = UNDEF_DOUBLE;  
    theY = UNDEF_DOUBLE;  
  } 
  else { 
    if ( anAngleSystem == RADIAN ){ 
    } 
    else if ( anAngleSystem == DEGREE ) { 
      anAngle = radFromDegree( anAngle );  
    } 
    else { 
      throwException( VectorException( "unknown AngleSystem" ) );  
    } 
    theX = cos( anAngle ) * aLength;  
    theY = sin( anAngle ) * aLength;  
  } 
} 
 
 
void Vector::set( double value1, double value2,  
                  CoordinateSystemEnum aCoordinateSystem = CARTESIAN, 
                  AngleSystemEnum anAngleSystem = RADIAN  ) 
{ 
  if ( aCoordinateSystem == CARTESIAN ) { 
    setCartesian( value1, value2 );  
  } 
  else if ( aCoordinateSystem == POLAR ) { 
    setPolar( value1, value2, anAngleSystem );  
  } 
  else { 
    throwException( VectorException( "unknown CoordinateSystem" ) );  
  } 
} 
 
Vector Vector::orthogonal(){
  Vector result;
  result.setPolar( length(), angle() + Ath98_PI_2 );
  return result;
}

void Vector::normalize() { 
  double myLength = length();  
  if ( myLength != 0.0 ) { 
    theX /= myLength;  
    theY /= myLength;  
  } 
  else { 
    theX = UNDEF_DOUBLE;  
    theY = UNDEF_DOUBLE;  
  } 
} 
 

 
Ath98String Vector::toString() const  
{ 
  Ath98String output;  
  if ( !isValid() ) { 
    output = "( ungueltiger Vektor )";  
  } 
  else { 
    output = Ath98String( "( kart: ( " ) + Ath98String( theX ) + Ath98String(", ") + 
      Ath98String( theY ) + Ath98String( "), polar: ( " ) + Ath98String( length() ) + 
      Ath98String( ", " ) + Ath98String( angle(DEGREE) ) + Ath98String( ") )" );  
  } 
  return output;  
} 
 
Vector Vector::operator * (double aScalar) const 
{ 
  return Vector( theX * aScalar, theY * aScalar ); 
} 

Vector Vector::operator / (double aScalar) const 
{ 
  return Vector( theX / aScalar, theY / aScalar ); 
} 

