#ifndef _COACH_H_
#define _COACH_H_

#include <vector>
#include <string>
#include "../../basic/Exception.h"
#include "../../basic/Ath98String.h"
class Communicator;
class Sensors;
class Effectors;
class Deliberator;

/**
  Die zentrale Hauptklasse des Coach's.
  Hier werden alle Komponenten angelegt und initialisiert.
 
  {\small {\bf Quelltext}

  [\URL[Header]{src/Coach.h} | [\URL[Implementierung]{src/Coach.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Coach.h,v $
  *   $Revision: 1.4 $
  *   $Date: 2001/06/18 16:07:53 $
  *   $Author: wendler $ 
  *
  **************************************************
  \end{verbatim} }*/
class Coach
{
public:

  /** Fehler, der bei nicht gueltiger oder fehlgeschlagener
      Initialisierung auftritt. */
  class InitFailedException: public Exception 
    {
    public:
      InitFailedException(const Ath98String& aDescription )
      	: Exception( aDescription ) {}
    };
  
  /**Der einzige Konstruktor.
     Er startet alle notwendigen Komponenten.
     @param argv Paare -<Option> <Wert> ; -h liefert Hilfe. */
  Coach( int argc, char* argv [] );
  
  /**Der Destruktor.
     Er deinstalliert alle Komponenten und raeumt auf. Logdateien werden
     geschlossen. */
  ~Coach ();

  /**Startet den Ueberlegungsprozess. 
     Das Ausfuehren der Handlungen und das Aufnehmen neuer Sensorinformationen
     geschieht asynchron ueber den Scheduler. */
  void run ();

private:

  /** Der UDP/IP-level Kommunikationsverwalter */
  Communicator* theCommunicator;
  
  /** Verwaltet die Sensorik */
  Sensors* theSensors;
  
  /** Die agierende Komponente */
  Effectors* theEffectors;
  
  /** Die Ueberlegungskomponente. Frueher Brain und Entscheidung. */
  Deliberator* theDeliberator;

  Ath98String teamName;
  Ath98String serverName;
  int serverPort;
  Ath98String serverConfigurationFile;
  Ath98String agentConfigurationFile;
  Ath98String coachConfigurationFile;
  Ath98String recFile;
  vector<Ath98String> extendedConfigurationFiles;
  bool isHelpWanted;
  bool useInteractive;

  /** Die Parameter werden eingelesen. */
  void readParameter( int argCount, char* argv [] );  

  /** Die Effektoren werden initialisiert und gestartet. */
  void startComponents();

  /** Initialisiert den Zufallszahlengenerator. */
  void initRandomNumberGenerator();
  
  /** Schreibt Hilfeinformationen nach stdout.
    @param programName Name des aufgerufenen Programmes. */
  void showHelp(Ath98String programName);

  /** Liefert einen String mit Klassenbezeichnung zurueck. */
  string idString() const;

};

#endif
