
/*
 * Aktueller Stand
 *
 *   $RCSfile: Deliberator.cc,v $
 *   $Revision: 1.10 $
 *   $Date: 2001/08/02 13:30:44 $
 *   $Author: wendler $ 
 *
 */
 
#include "./Deliberator.h"
#include "./Timer.h"
#include "./CommonCoach.h"
#include "../../basic/Common.h"
#include "../../basic/Configuration.h"
#include "../../basic/Log.h"
#include "../../worldmodel/Situation.h"
#include "../../worldmodel/WorldModelInterface.h"
#include "../serverInterface/Sensors.h"
#include "../serverInterface/Effectors.h"
#include <strings.h>
#include "stdlib.h"

Deliberator::Deliberator( Sensors *sensors, Effectors *effectors ) :
  theSensors( sensors ), theEffectors( effectors ),
  playerTypes( vector<int>(0) )
{
  theIdString = "Deliberator";
  lastMessageSentTime = globalConf->firstSayByCoachTime - 300;
  playerIndexToExchangeNext = 0;
  skippedGoalie = 0;
  alreadyAllPlayersExchanged = NO;

  /*
  for( int i = 0; i < 7; i ++ ) {
    for( int j = 0; j < 7; j++ ) {
      criticalPairs[i][j] = 0;
    }
  }
  */
}

Deliberator::~Deliberator()
{
  /*
  cout << "Falsch erkannte Paare\n";
  for( int i = 0; i < 7; i ++ ) {
    for( int j = 0; j < 7; j++ ) {
      if( criticalPairs[i][j] != 0 ) {
        cout << "Kritisches Paar: (" << j << "," << i << "): " <<
          criticalPairs[i][j] << endl;
      }
    }
  }
  */
}

void Deliberator::initialize(){
  // zum initialisieren von Daten, die erst nach Betreten des Spielfeldes
  // zur Verfuegung stehen
  globalTeamInfo->Initialize();
  evaluatePlayerTypes();
}


void Deliberator::deliberate()
{
  while( YES ) {
    globalTimer->controller();

    // Funktionalitaet des laengeren Nachdenkens

    globalTimer->agentSleep();
  }
}

void Deliberator::execute()
{
  // Funktionalitaet des kuerzeren Nachdenkens
  globalTeamInfo->Calculate();

  changePlayerTypes();
  //analyseIdentify();
  sendMessageToPlayers();
}


string Deliberator::idString() const 
{ 
  return theIdString;  
} 

void Deliberator::changePlayerTypes()
{
  if( alreadyAllPlayersExchanged ) {
    return;
  }
  if( !theSensors->isNewOkInfo() && playerIndexToExchangeNext != 0 ) {
    // nochmal senden
    playerIndexToExchangeNext--;
  }
  theSensors->initOkInfo();
  int playerId = playerIndexToExchangeNext + MIN_TEAMMATE_ID;
  try {
    if( globalWorldModel->situation().playerWithId( playerId ).isGoalie() ) {
      skippedGoalie = 1;
    } 
  }
  catch( Exception e ) {
    return;
  }

  if( playerIndexToExchangeNext + skippedGoalie < 
      globalConf->substitutePlayer.size() ) {
    int i = playerIndexToExchangeNext;
    int j = skippedGoalie;
    theEffectors->changePlayerType( globalConf->substitutePlayer[i+j], 
                                    playerTypes[i] );
    playerIndexToExchangeNext++;
  }
  else {
    alreadyAllPlayersExchanged = YES;
  }
}

void Deliberator::sendMessageToPlayers() 
{
  long simStep = globalWorldModel->simStep();
  if( simStep > lastMessageSentTime + globalConf->clangWinSize + 1 ) {
    //send new message to players
    globalTeamInfo->CalcPlayerTypes();
    int length = 8191;
    char message[length + 1];
    bzero( message, (length + 1)* sizeof(unsigned char));
    char* pointer = message;

    int count = 0;
    const Situation& situation = globalWorldModel->situation();
    for (int i=0;i<situation.playerCount();i++) {
      int playerNumber = playerNumberById(situation.playerWithIndex(i).id());
      if( situation.playerWithIndex(i).team() == THEIR_TEAM_ID && 
          !situation.playerWithIndex(i).isGoalie() &&
          globalTeamInfo->changedOpponentPlayerTypeByNumber[playerNumber] ) {
        int playerType = globalTeamInfo->typeOfPlayerWithIndex( i );
        if( playerType != -1 ) {
          pointer += sprintf( pointer, "(do opp {%d} (htype %d)) ", 
                              playerNumber, playerType );
          count++;
        }
      }
    }

    if( count > 0 ) {
      theEffectors->sayMessage( "(info (6000 (true) " + 
                                Ath98String( message ) + "))" ); 
      lastMessageSentTime = simStep;
      //globalTeamInfo->Output();
    }
    /*
    else {
      theEffectors->sayMessage( Ath98String("(info (6000 (true) ") + 
                                "(do opp {1 4} (htype 2)) " +
                                "(do opp {2} (oline (null))) " + 
                                "(do opp {3} (htype 5)) " + "))" );
      lastMessageSentTime = simStep;
    }
    */
  }
}

void Deliberator::analyseIdentify()
{
  const Situation& situation = globalWorldModel->situation();
  long simStep = globalWorldModel->simStep();
  if( simStep % 10 == 0 && simStep >= 50 ) {
    globalTeamInfo->CalcPlayerTypes();
    int goodCount = 0; int wrongCount = 0;
    for (int i=0;i<globalWorldModel->situation().playerCount();i++) {
      int playerNumber = playerNumberById(situation.playerWithIndex(i).id());
      if( situation.playerWithIndex(i).team() != THEIR_TEAM_ID ||
          situation.playerWithIndex(i).isGoalie() ||
          !globalTeamInfo->changedOpponentPlayerTypeByNumber[playerNumber] ) {
        continue;
      }
      int id = globalWorldModel->situation().playerWithIndex(i).id();
      int correct = globalPlayerTypes->typeOfPlayerWithId( id );
      int analyzed = globalTeamInfo->typeOfPlayerWithIndex( i );
      if( analyzed != -1 && correct != analyzed ) {
        wrongCount++;
        //int firstIndex = (correct>analyzed?correct:analyzed);
        //int secondIndex = (correct>analyzed?analyzed:correct);
        //criticalPairs[firstIndex][secondIndex] = criticalPairs[firstIndex][secondIndex]+1;
        cout << "Wrong: id: " << id << " correct: " << correct <<
          "\tanalyzed: " << analyzed << endl;
      }
      else if( correct == analyzed ) {
        goodCount++;
      }
      else {
      }
    }
    cout << "Analysis: time: " << simStep << "\tgood: " << 
      goodCount << "\twrong: " << wrongCount << endl;
    //globalTeamInfo->Output();
  }
}



void Deliberator::evaluatePlayerTypes() {
  // hack-values of hetroplayer-parameter - to be read in by paramreader
  double standardValue[] = {1.0, 0.4, 0.006, 0.7, 0.0};
  double maxValueDelta[] = {0.2, 0.2, 0.002, 0.2, 100.0};
  int nrParameters  = 5;
  int nrPlayerTypes = 7;
  double hetroPlayers[nrPlayerTypes][nrParameters];
  bool isInRanking[nrPlayerTypes];

  char* path = globalConf->hetro_player_ranking_file;
  int maxSteps, k, level;
  // Faktor, der den aktuellen Parameterwert auf die Test-Skala abbildet
  double normalizingFactor[nrParameters];
  // die Bewertung der Spieler
  double sumOfBenefits[nrPlayerTypes];
  // enthaelt die geordneten Spielertypen - ..[0] ist der beste
  int hetroPlayerRanking[nrPlayerTypes];
  // naechsten Parameterwerte fuer alle Spieler
  int neighborValue[nrPlayerTypes][nrParameters][2];
  // aktueller Testtype
  char paramString[nrParameters];
  // das Quadrat des Abstandes
  double squareDistance;
  // Gewicht, das mit der Rankingposition multipliziert wird
  double weight;
  // Summe der Gewichte
  double sumOfWeights;
  double maxValue;
  int posOfMaximum;
	 
  INFO( Log::DELIBERATOR, 2, idString(), globalLog->infoStream <<
        "Start evaluating playertypes ..." << endl );
	
  // Initialisiere hetroPlayers
  for(int i=0; i < nrPlayerTypes; i++) {
    hetroPlayers[i][0] = globalPlayerTypes->playerMaxSpeed(i);
    hetroPlayers[i][1] = globalPlayerTypes->playerDecay(i);
    hetroPlayers[i][2] = globalPlayerTypes->dashPowerRate(i);
    hetroPlayers[i][3] = globalPlayerTypes->kickableMargin(i);
    hetroPlayers[i][4] = globalPlayerTypes->extraStamina(i);
    
    isInRanking[i] = false;
        
    INFO( Log::DELIBERATOR, 5, idString(), globalLog->infoStream <<
          "Parameters of PlayerType " << i << ":\n" );

    for ( int j = 0; j < nrParameters; j++ ) {
      INFO( Log::DELIBERATOR, 5, idString(), globalLog->infoStream <<
            hetroPlayers[i][j] << "; " );
    }
    INFO( Log::DELIBERATOR, 5, idString(), globalLog->infoStream << "\n" );
  }
	
  // Oeffne Rankingfile
  FILE* fp;	
  if ( (fp = fopen ( path, "r" ) ) == NULL ) {
    throwException( Exception( "Can't open ranking file " + 
                               Ath98String( path ) + "\n" ) ); 
  }
	
  // lese Aufloesung der Testtypen ein
  maxSteps = getc(fp)-48;  
  rewind(fp);

  // ranking-File einlesen
  int lines = (int)pow(3, nrParameters);
  int idString[lines];
  float evaluation[lines];
  float goalBonus[lines];
  
  char buffer[100];
  int i = 0;
  int id, games, win, tie, loose, goalsOur, goalsTheir;
  float eval;
  while( fgets(buffer, 100, fp) ) {
    if ( strncmp("2", buffer, 1) == 0 && i < lines ) {
      sscanf( buffer, "%d %d %f %d %d %d %d %d", &id, &games, &eval,
              &win, &tie, &loose, &goalsOur, &goalsTheir );
      idString[i] = id;
      evaluation[i] = eval;
      goalBonus[i] = ((float)(goalsOur-goalsTheir))/((float)games)*10.0;
      //cout << "line " << i << " string: " << id << " Eval: " << eval <<
      //  " GoalDiff: " << goalBonus[i] << endl;
      i++;
    }
  }
	
  // berechne den Normalisierungsfaktor fuer jeden Parameter 
  // -> Abbildung von (Parameterwert-Standardwert) auf [0-maxSteps]
  for ( int i = 0; i < nrParameters; i++ ) {
    normalizingFactor[i] = maxSteps/maxValueDelta[i];
  }
	
  // berechne zu jedem Spielertypen und jedem Parameter den naechsten
  // oberen und unteren Nachbarn
  for ( int i = 0; i < nrPlayerTypes; i++ ) {
    for ( int j = 0; j < nrParameters; j++ ) {
      neighborValue[i][j][1] = 
        (int)ceil((hetroPlayers[i][j]-standardValue[j])*normalizingFactor[j]);
      neighborValue[i][j][0] = 
        (int)floor((hetroPlayers[i][j]-standardValue[j])*normalizingFactor[j]);
    }
  }
	
  // berechne gewichteten Benefit fuer jeden Spielertypen:
  for ( int i = 0; i < nrPlayerTypes; i++ ) {
    sumOfWeights = 0;
    sumOfBenefits[i] = 0;
		
    // berechene Benefit fuer alle Nachbarn (|N| = 2^nrParameters)
    for ( int j = 0; j < pow(2, nrParameters); j++ ) {
      k = 1;
      squareDistance = 0;
      paramString[0] = maxSteps + 48;

      // erstelle den paramString fuer den Nachbarn und berechnet die
      // Summer der Abstandsquadrate zum Spielertypen
      for ( int l = 0; l < nrParameters; l++ ) {
        // oberer oder unterer Parameter
        if( j & k ){ level = 1; }
        else{ level = 0; }
        
        // setze Parameter fuer Testtyp
        paramString[l+1] = neighborValue[i][l][level] + 48;
        
        // addiere Quadrat der Koordinatenabweichung (normalisiert)
        squareDistance += pow(fabs(neighborValue[i][l][level]-
                                   (hetroPlayers[i][l]-standardValue[l])*
                                   normalizingFactor[l]), 2);
        // naechster Parameter
        k = k << 1;
      }
      
      paramString[nrParameters+1] = '\0';

      double benefit;
      sscanf( paramString, "%d", &id );
      //cout << "Searching for id: " << id << endl;
      for( int z = 0; z < lines; z++ ) {
        if( id == idString[z] ) {
          //cout << "i:" << i << " j:" << j << " Using line " << z << 
          //  "with goalBonus " << goalBonus[z] << endl;
          benefit = goalBonus[z];
          //benefit = evaluation[z];
          //benefit = z;
        }
      }

      if ( squareDistance < 0.001 ) {
        // falls exates Matching, uebernehme Rankingposition als
        // Bewertung und teste naechsten Spielertypen
        sumOfBenefits[i] = benefit;
        sumOfWeights = 1.0;
        break;
      }	
      else {
        //cout << "Square dist " << squareDistance << endl;
        // Wichtungsfunktion: (1/delta^2)^nrParameters - beliebig aenderbar
        weight = pow(1/(squareDistance+1), 2);
        
        // addiere die Position des Testtypen in der Rankingliste
        // gewichtet nach dem Abstand zum Benefit hinzu
        sumOfBenefits[i] += weight*benefit;
        sumOfWeights += weight;
      }
    }

    // Bestimme gewichtete Rankingposition, d.h. teile durch die Summe
    // der Wichtungen 
    if ( sumOfWeights != 0 ) {
      sumOfBenefits[i] /= sumOfWeights;
    }
  }

  // sortiere nach Guete
  for ( int i = 0; i < nrPlayerTypes; i++ ) {
    maxValue = -10000.0;
    
    for ( int j = 0; j < nrPlayerTypes; j++ ) {
      if ( !isInRanking[j] && sumOfBenefits[j] > maxValue ) {
        maxValue = sumOfBenefits[j];
        posOfMaximum = j;
      }
    }
    
    isInRanking[posOfMaximum] = true;
    hetroPlayerRanking[i] = posOfMaximum;
  }

  for ( int i = 0; i < nrPlayerTypes; i++ ) {
    INFO( Log::DELIBERATOR, 2, idString(), globalLog->infoStream <<
          "Ranking position " << i << ": " << hetroPlayerRanking[i] <<
          " (Virtual Ranking: " << sumOfBenefits[hetroPlayerRanking[i]] <<
          ")\n" );
  }
  
  // Generieren der Absende-Befehle
  int subs = 0;
  int maxSubs = globalConf->substitutePlayer.size()-1;
  double overallBenefit = 0.0;
  for ( int i = 0; i < nrPlayerTypes; i++ ) {
    if( sumOfBenefits[hetroPlayerRanking[i]] > sumOfBenefits[0] 
        + globalConf->playerSubstituteBonus && subs < maxSubs &&
        hetroPlayerRanking[i] != 0 ) {
      int subsPerType = 0;
      while( subs < maxSubs && subsPerType < 3 ) {        
        playerTypes.push_back( hetroPlayerRanking[i] );
        INFO( Log::DELIBERATOR, 2, idString(), globalLog->infoStream <<
              "Preparing for change: " << globalConf->substitutePlayer[subs] <<
              " " << hetroPlayerRanking[i] << endl );
        subs++;
        subsPerType++;
        overallBenefit += sumOfBenefits[hetroPlayerRanking[i]] - 
          sumOfBenefits[0];
      }
    }
  }
  // Auffuellen mit dummy-Auswechselungen
  while( subs < maxSubs ) {
    playerTypes.push_back( 0 );
    INFO( Log::DELIBERATOR, 2, idString(), globalLog->infoStream <<
          "Preparing for change: " << globalConf->substitutePlayer[subs] <<
          " 0\n" );
    subs++;
  }
  overallBenefit /= 10.0;
  INFO( Log::DELIBERATOR, 2, idString(), globalLog->infoStream <<
        "Expected overall Benefit of subsitutions in 10 games: " <<
        overallBenefit << endl );

  fclose(fp);
}



