#ifndef _TIMER_H_
#define _TIMER_H_

#include <string>
class Sensors;
class Effectors;
class Deliberator;

/**
  Diese Klasse koordiniert das Timing zwischen den Komponenten
  Effectors, Sensors und Deliberator.
 
  {\small {\bf Quelltext}

  [\URL[Header]{src/Timer.h} | [\URL[Implementierung]{src/Timer.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Timer.h,v $
  *   $Revision: 1.5 $
  *   $Date: 2001/06/18 16:07:53 $
  *   $Author: wendler $ 
  *
  **************************************************
  \end{verbatim} }*/
class Timer
{
public:

  /** @name Timer erzeugen und entfernen */
  //@{
  /** Erzeugt einen neuen Timer. 
      Dieser ist noch nicht aktiv. Er wird erst aktiv, wenn er mit run() 
      gestartet wird.
      @param sensors Zeiger auf das Sensorik-Objekt.
      @param effectors Zeiger auf das Effector-Objekt.
      @param deliberator Zeiger auf das Deliberator-Objekt. */
  Timer( Sensors *sensors, Effectors *effectors, Deliberator *deliberator );
  
  ~Timer();
  //@}
  
  
  /** Testet auf neue Sensorinformationen und generiert gegebenenfalls
      Aktionen indem die execute-Methode des Deliberators aufgerufen wird.
      @return YES oder NO
      YES : falls neue Informationen da sind
      NO : sonst. */
  bool controller();
  
  /** Versetzt den Agenten-Prozess in den sleep-modus,
      bis die controller()-Methode YES zurueckgeben wuerde.
      ACHTUNG: dieser Schlaf kann recht lange dauern (20ms). */
  void agentSleep();
  
  /// @return Zeit seit Start des Agenten in Millisekunden
  int getAgentTime() const;

  /** Liest eine Nachricht von stdin und sendet sie direkt zum Server. */
  void readAndSend();  

  /// @return String mit Klassenbezeichnung
  string idString() const;

private:
  Sensors* theSensors;
  Effectors* theEffectors;
  Deliberator* theDeliberator;

  /// Setzt die absolute Zeit in s seit 1970 bis zum Start des Agenten
  void setConstructionTime();
  /** absolute Zeit in s seit 1970 bis zum Start des Agenten
      alle anderne Zeiten sind relativ zu dieser Zeit in ms angegeben */
  long theTimeOfConstruction;

  /** Zeit zu der der Timer das letzte Mal gerufen wurde.*/
  long theTimeOfLastCall;

  /** @return String der die Kurzkennung der Klasse enth\"alt. */
  string theIdString;

};

/// Globaler Zeiger auf das aktive Timer-Objekt.
extern Timer *globalTimer;

#endif
