
#include "./MessageCoach.h" 
#include "../../basic/Exception.h"
#include "../../basic/Configuration.h"
 
MessageCoach::MessageCoach( const Ath98String& information )  
{ 
  theTime = UNDEF_TIME;
  setByHearInfo(information);
} 
 
MessageCoach::MessageCoach( const Ath98String& information, long simStep )  
{ 
  theTime = UNDEF_TIME;
  setBySayInfo( information, simStep );
} 

MessageCoach::MessageCoach()  
{ 
  theTime = UNDEF_TIME;  
}

MessageCoach::~MessageCoach()
{
}

void MessageCoach::parseHearMessage(const Ath98String& information)
{
  if( !(information.beginsWith( "(hear") ) ) { 
    throwException( ( Exception("Message was incorrect used") ) );  
  } 

  char message[MAX_MESSAGE];
  char sender[256];

  sscanf( information.cString(), "(hear %ld %[^\n\r\f]", &theTime, message );
  if( message[0] != '(' ) {
    sscanf( message, "%s %[^)]", sender, message );
  }
  else {
    sscanf( message, "(%[^)]) \"%[^\")])", sender, message );
  }

  if ( strcmp(sender, "referee") == 0 ) { 
    theSenderInfo = REFEREE_ID;  
  }
  else if( sender[0] == 'p' ) {
    sscanf( sender, "p \"%[^\"]\" %i", sender, &thePlayerNum );
    if( theTeamname == Ath98String( sender ) ) {
      theSenderInfo = OUR_TEAM_ID;
    }
    else {
      theSenderInfo = THEIR_TEAM_ID;
    }
  }
  else {
    theSenderInfo = UNDEF_TEAM_ID;  
    INFO( Log::SENSORS, 1, idString(), globalLog->infoStream <<
          "WARNING: Unknown sender info: " << sender << "\n" );
  }

  theMessage = Ath98String(message);  
 
  INFO( Log::SENSORS, 6, idString(), globalLog->infoStream <<
        "new message: " << *this );
}

ostream& operator<< (ostream& os, const MessageCoach& message ) {
  os << ((Message)(message));
  return os;
} 

int MessageCoach::playerNum() const {
  return thePlayerNum;
}
 
;  
