#ifndef _MESSAGE_COACH_H_
#define _MESSAGE_COACH_H_

#include "../../serverInterface/Message.h"

/**
  Kapselung einer Nachricht die vom Coach gehoert werden kann, oder die
  vom Coach gerufen werden kann.
  Eine gehoerte Nachricht besteht aus der eigentlichen Mitteilung
  (einem Ath98String), einer Zeit fuer die sie gilt und einem Absender. 
  Der Absender ist entweder eine Spieler- oder die Schiedsrichter-ID.
  Eine gerufene Nachricht besteht aus der eigentlichen Mitteilung
  (einem Ath98String) und einer Zeit fuer die sie gilt.
 

  {\small {\bf Quelltext}

  [\URL[Header]{src/MessageCoach.h} | [\URL[Implementierung]{src/MessageCoach.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: MessageCoach.h,v $
  *   $Revision: 1.3 $
  *   $Date: 2001/06/21 12:17:52 $
  *   $Author: wendler $
  *
  **************************************************
  \end{verbatim} }*/
class MessageCoach : public Message
{
public:

  /** Aufbau einer gehoerten Nachricht die vom SoccerServer gekommen ist.
      Initialisierung mit einem Ath98String. */
  MessageCoach( const Ath98String& information );
  
  /** Generierung einer Nachricht, die an den SoccerServer
      gesendet werden soll. */
  MessageCoach( const Ath98String& information, long simStep );
  
  /** Konstrukter fuer Template-Klassen */
  MessageCoach();

  ~MessageCoach();

  /** Falls der Sender ein Spieler ist, wird mit dieser Methode dessen
      Nummer zurueckgegeben, andernfalls 0. */
  int playerNum() const;

  /** Ausgabe auf einen String-Stream */
  friend ostream& operator<< (ostream&, const MessageCoach& );

private:
  
  /** Setzt alle Daten, die man dem Ath98String entnehmen kann. */
  void parseHearMessage(const Ath98String& information);

  /** Falls der Sender ein Spieler ist, wird hier dessen Nummer gespeichert */
  int thePlayerNum;

};

#endif
