
/*
 * Aktueller Stand
 *
 *   $RCSfile: SensorsServer.cc,v $
 *   $Revision: 1.5 $
 *   $Date: 2001/08/02 11:52:29 $
 *   $Author: wendler $ 
 *
 */
 
#include "./SensorsServer.h"
#include "../basic/Timer.h"
#include "../../basic/Common.h"
#include "../../worldmodel/ModelledBall.h"
#include "../../worldmodel/ModelledPlayer.h"
#include "../../worldmodel/WorldModelInterface.h"

SensorsServer::SensorsServer( Communicator* aCommunicator )
  : Sensors(), theCommunicator( aCommunicator ) {
  theIdString = "SensorsServer";
  INFO( Log::ANY, 2, idString(), globalLog->infoStream <<
	"SensorsServer created. \n" );
}

SensorsServer::~SensorsServer()
{
}

void SensorsServer::sense() {
  Ath98String message;
  static int timeOfLastMessage = globalTimer->getAgentTime();

  try {
    while(1) {
      message = theCommunicator->getNextMessage(0);
      timeOfLastMessage = globalTimer->getAgentTime();
      theIsNewInfo = YES;
      parseSensoryData(message);
    }
  }
  catch( Communicator::NothingReceived ) {
    int timeSinceLastMessage = globalTimer->getAgentTime() - timeOfLastMessage;
    if ( timeSinceLastMessage > globalConf->communicationTimeout * 1000 ) {
      throwException( SensorsException( "Server has probably terminated." ) );
    }
  }
}

void SensorsServer::parseSensoryData(const Ath98String& message) {
  if( message.beginsWith( "(see_global" ) ) {
    char seeMessage[MAX_MESSAGE];
    sscanf( message.cString(), "(see_global %ld%[^\n]", &theSimStep, 
            seeMessage);
    theSituation = Situation( theSimStep );
    parseSights( seeMessage );
    theIsNewSeeInfo = YES;
  }
  else if( message.beginsWith( "(hear" ) ) {
    MessageCoach heardMessage = MessageCoach( message );
    theHeardMessages.push_back( heardMessage );
    if( heardMessage.senderInfo() == REFEREE_ID ) {
      theIsNewRefereeInfo = YES;
    }
    else if( heardMessage.senderInfo() == OUR_TEAM_ID ){
      theIsNewHearInfo = YES;
    }
  }
  else if( message.beginsWith( "(change_player_type" ) ) {
    globalPlayerTypes->parseSubstituteMessage( message );
    globalTeamInfo->parseSubstituteMessage( message );
  }
  else if( message.beginsWith( "(ok change_player_type" ) ) {
    theIsNewOkInfo = YES;
  }
  else if( message.beginsWith( "(ok eye on)" ) ) {
    theIsNewOkInfo = YES;
  }
  else if( message.beginsWith( "(ok say)" ) ) {
    theIsNewOkInfo = YES;
  }
  else if( message.beginsWith( "(error" ) ) {
    INFO( Log::SENSORS, 1, idString(), globalLog->infoStream <<
          "WARNING: Server error: " << message << "\n" );
  }
  else {
    INFO( Log::SENSORS, 0, idString(), globalLog->infoStream <<
          "ERROR: Unknown server message: " << message << "\n" );
    //throwException( SensorsException( "Unknown server message: " + message ) );
  }
}

void SensorsServer::parseSights(char* seeMessage) {
  double x, y, xChange, yChange, bodyDir, faceDir;
  char objectArray[256];
  char* object;
  char information[256];

  while( seeMessage[0] != ')' ) {
    object = objectArray;
    sscanf( seeMessage, " ((%[^)]) %[^)]) %[^\n]", object,
            information, seeMessage);
    strcat( object, ")" );
    strcat( information, ")" );

    INFO( Log::SENSORS, 12, idString(), globalLog->infoStream <<
          "Analyzing information: " << information << "\n" );
    globalLog->flushInfoStream();
    // parse information string
    x = UNDEF_DOUBLE;
    y = UNDEF_DOUBLE;
    xChange = UNDEF_DOUBLE;
    yChange = UNDEF_DOUBLE;
    bodyDir = UNDEF_DOUBLE;
    faceDir = UNDEF_DOUBLE;
    sscanf( information, "%lf %lf %[^\n]", &x, &y, information );
    y = -y;
    if( globalWorldModel->side() == RIGHT_SIDE_ID ) {
      x = -x; y = -y;
    }
    if( information[0] != ')' ) {
      sscanf( information, "%lf %lf %[^\n]", &xChange, &yChange, information );
      yChange = -yChange;
      if( globalWorldModel->side() == RIGHT_SIDE_ID ) {
        xChange = -xChange; yChange = -yChange;
      }
      if( information[0] != ')' ) {
        sscanf( information, "%lf %lf %[^\n]", &bodyDir, &faceDir, 
                information );
        bodyDir = radFromDegree( -bodyDir );
        faceDir = radFromDegree( -faceDir ) + bodyDir;
        if( globalWorldModel->side() == RIGHT_SIDE_ID ) {
          bodyDir = normalizeAngle( bodyDir - Ath98_PI );
          faceDir = normalizeAngle( faceDir - Ath98_PI );
        }
      }
    }


    INFO( Log::SENSORS, 12, idString(), globalLog->infoStream <<
          "Analyzing object: " << object << "\n" );
    globalLog->flushInfoStream();
    // parse object string    
    if( strncasecmp (object, "b", 1) == 0 ) { // ball oder Ball
      theSituation.setBall( ModelledBall( BALL_ID, Vector( x, y ),
                                          Vector( xChange, yChange ) ) );
    }
    else if( strncasecmp(object, "p", 1) == 0 ) { // player oder Player
      char team[256];
      int playerNumber;
      int playerId = UNDEF_PLAYER_ID;
      bool isGoalie = NO;

      if( strncmp(object, "p", 1) == 0 ) { // player
        sscanf( object, "p%[^\n]", object);
      }
      else { // Player
        sscanf( object, "P%[^\n]", object);
      }

      if( object[0] != ')' ) {
        sscanf( object, " \"%[^)\" \n\t\r\f]\"%[^\n]", team, object);
        if( globalWorldModel->teamName() == Ath98String(team) ) {
          playerId = MIN_TEAMMATE_ID;
        }
        else {
          playerId = MIN_OPPONENT_ID;
        }
        if( object[0] != ')' ) {
          sscanf( object, " %i%[^\n]", &playerNumber, object);
          playerId += playerNumber - 1;
        }
        else {
          if( playerId == MIN_TEAMMATE_ID ) {
            playerId = UNDEF_TEAMMATE_ID;
          }
          if( playerId == MIN_OPPONENT_ID ) {
            playerId = UNDEF_OPPONENT_ID;
          }
        }
        if( object[0] != ')' ) {
          if( object[1] == 'g' ) { //goalie gefunden
            isGoalie = YES;
          }
        }
      }
      theSituation.addPlayer( ModelledPlayer( playerId, Vector( x, y ),
          Vector( xChange, yChange ), faceDir, bodyDir, VIEW_WIDTH_WIDE, 
          VIEW_QUALITY_HIGH, UNDEF_DOUBLE, UNDEF_DOUBLE, UNDEF_DOUBLE,
          isGoalie ) );
    }
    else if( strncasecmp(object, "g", 1) == 0 ) { // goal oder Goal
      if( strncmp(object, "g r", 3) == 0) {
        //StableObject( RIGHT_GOAL_CENTER_FLAG_ID, Vector( x, y ) );
      }
      else if( strncmp(object, "g l", 3) == 0) {
        //StableObject( LEFT_GOAL_CENTER_FLAG_ID, Vector( x, y ) );
      }
      else {
        // anonymes Tor, ignorieren.
        continue;
      }
    }
    else {
      INFO( Log::SENSORS, 0, idString(), globalLog->infoStream <<
            "ERROR: Unknown observed object: " << object << "\n" );
      //throwException( SensorsException( "Unknown observed object: " + object ) );
    } 
  }
}

int SensorsServer::checkEnterFieldForSide() 
{
  Ath98String answer;
  char internalSide;

  try { 
    answer = theCommunicator->getNextMessage(  
        globalConf->communicationTimeout );
    if( answer.beginsWith( "(error ") ) {
      throwException( SensorsException( "Server error: " + answer ) );
    }

    sscanf (answer.cString(), "(init %c", &internalSide); 
    
    return ( (internalSide == 'l') ? LEFT_SIDE_ID : RIGHT_SIDE_ID );
  } 
  catch( Communicator::NothingReceived ) { 
    throwException( SensorsException( "Server didn't respond init message" ) );
  } 
}

void SensorsServer::checkServerParams() {
  Ath98String message;
  try {
    while( YES ) {
      message = theCommunicator->getNextMessage(1); // eine Sekunde Wartezeit
      if( message.beginsWith( "(server_param" ) ) {
        parseServerParams( message );
      }
      else if( message.beginsWith( "(player_param" ) ) {
        parsePlayerParams( message );
      }
      else if( message.beginsWith( "(player_type" ) ) {
        globalPlayerTypes->parsePlayerTypeMessage(message);
      }
      else if( message.beginsWith( "(change_player_type" ) ) {
        globalPlayerTypes->parseSubstituteMessage( message );
        globalTeamInfo->parseSubstituteMessage( message );
      }
      else if( message.beginsWith( "(error ") ) {
        INFO( Log::SENSORS, 1, idString(), globalLog->infoStream <<
              "WARNING: Server error: " << message << "\n" );
      }
      else {
        INFO( Log::SENSORS, 0, idString(), globalLog->infoStream <<
              "ERROR: Unknown server message: " << message << "\n" );
        //throwException( SensorsException( "Unknown server message: " +
        //                                  message ) ); 
      }
    }
  }
  catch( Communicator::NothingReceived ) {} // alle Nachrichten auswerten
}

void SensorsServer::parseServerParams( const Ath98String& message )
{

}

void SensorsServer::parsePlayerParams( const Ath98String& message )
{

}

bool SensorsServer::isNewOkInfo() 
{
  return theIsNewOkInfo;
}
