#ifndef _SENSORS_SERVER_H_
#define _SENSORS_SERVER_H_

#include "./Sensors.h"
#include "../../serverInterface/Communicator.h"

/**
  Diese Klasse ist f\"ur die Verarbeitung und Aufbereitung aller vom
  SoccerServer ankommenden Daten zust\"andig.
 
  {\small {\bf Quelltext}

  [\URL[Header]{src/SensorsServer.h} | [\URL[Implementierung]{src/SensorsServer.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: SensorsServer.h,v $
  *   $Revision: 1.2 $
  *   $Date: 2001/08/02 11:52:29 $
  *   $Author: wendler $ 
  *
  **************************************************
  \end{verbatim} }*/
class SensorsServer : public Sensors
{
public:

  /** Erzeugt ein neues Sensorobjekt.
    @param aCommunicator Das Communicator-Objekt, von dem die 
    Sensor-Informationen kommen.  */
  SensorsServer( Communicator* aCommunicator );

  ~SensorsServer();

 /** Empfaengt Sensorstrings und verarbeitet sie zu Objekten.
     Versucht vom eigenen Communicator Ath98Strings zu lesen. Diese werden zu
     Objekten verarbeitet und spaeter vom Weltmodell ausgelesen.

     Nach einem Aufruf von Sense liegen keine Ath98Strings mehr im Communicator
     bereit. 
     @exception SensorsException wenn der Server l\"anger als
     \Ref{Configuration::communicationTimeout}() Sekunden keine
     Sichtinformationen geschickt hat. Man kann dann davon ausgehen, da\3 der
     Server sich terminiert hat. */
  void sense();

  /** Ein Ath98String vom SoccerServer wird geparst. Die zu diesem Ath98String
      gehoerenden Objekte werden aufgebaut. */
  void parseSensoryData(const Ath98String& message);

  /** Wertet die Information nach Betreten des Spielfeldes aus.
      Im Weltmodell wird die Seite auf der das Team des Coches spielt
      eingetragen.
      Das Weltmodell muss also vor Aufruf dieser Methode bereits initialisiert
      sein. */
  int checkEnterFieldForSide();

  /** Wertet die Paramter aus, die vom SoccerServer gesendet werden. */
  void checkServerParams();

  bool isNewOkInfo();

private:
  /** Interne Referenz auf Communicator Objekt */
  Communicator *theCommunicator;

  /** Parsen einer Sichtinformation. */
  void parseSights(char* seeMessage);

  /** Parsen der Server-Parameter */
  void parseServerParams( const Ath98String& message );

  /** Parsen der Player-Parameter */
  void parsePlayerParams( const Ath98String& message );

};

#endif
