/**
 * Aktueller Stand
 *
 *   $RCSfile: WorldModelInterface.cc,v $
 *   $Revision: 1.7 $
 *   $Date: 2001/07/19 16:09:07 $
 *   $Author: wendler $
 *
 */

#include "./WorldModelCore.h"
#include "../../worldmodel/WorldModelInterface.h"

// Konstruktor/Destruktor
WorldModelInterface *globalWorldModel;  
 
WorldModelInterface::WorldModelInterface( Sensors * theSensors)
{ 
  theWorldModelCore = new WorldModelCore( theSensors);
}

WorldModelInterface::~WorldModelInterface()
{
  delete theWorldModelCore;
}


// Anfragen und Setzen von Coach-Werten
int WorldModelInterface::playerNumber() const 
{
  return theWorldModelCore->playerNumber();
}

const Ath98String& WorldModelInterface::teamName() const
{
  return theWorldModelCore->teamName();
}

int WorldModelInterface::side() const 
{
  return theWorldModelCore->side();
} 
 
int WorldModelInterface::id() const
{
  return theWorldModelCore->id();
}

void WorldModelInterface::setPlayerNumber( int newPlayerNumber, bool isGoalie )
{ 
  theWorldModelCore->setPlayerNumber( newPlayerNumber, isGoalie );
}

void WorldModelInterface::setTeamName( const Ath98String& newTeamName )
{
   theWorldModelCore->setTeamName(newTeamName);
}

void WorldModelInterface::setSide( int newSide ) 
{
  theWorldModelCore->setSide(newSide);
}

// Anfragen zur aktuellen oder aelteren Situationen
const Situation& WorldModelInterface::situation( int timeStepOffset=0 ) const 
{ 
  return theWorldModelCore->situation( timeStepOffset  );
} 

const Situation& WorldModelInterface::correctSituation( ) const 
{ 
  return theWorldModelCore->situation();
} 

const ModelledPlayer& WorldModelInterface::player( int playerId ) const 
{ 
  return theWorldModelCore->player( playerId );
} 
 
const ModelledBall& WorldModelInterface::ball() const 
{ 
  return theWorldModelCore->ball( ) ;  
} 

int WorldModelInterface::simStep() const 
{
  return theWorldModelCore->simStep(); 
}

int WorldModelInterface::gameState() const
{
  return theWorldModelCore->gameState(); 
}

int WorldModelInterface::initiative() const
{
  return theWorldModelCore->initiative(); 
}

bool WorldModelInterface::hasInitiative( int aTeamId ) const 
{ 
  return theWorldModelCore->hasInitiative( aTeamId );
} 

bool WorldModelInterface::isTimeElapsingGameState()
{
  return theWorldModelCore->isTimeElapsingGameState(); 
}


void WorldModelInterface::setSimStep( int newValue )
{
  theWorldModelCore->setSimStep(newValue );
}

void WorldModelInterface::setGameState( int newGameState ) 
{ 
  theWorldModelCore->setGameState( newGameState );
} 

void WorldModelInterface::setInitiative( int newInitiative )  
{ 
  theWorldModelCore->setInitiative( newInitiative );
}


// Allgemeine Anfragen und Setzmoeglichkeiten

int WorldModelInterface::score( int aTeamId ) const
{ 
  return theWorldModelCore->score( aTeamId );
} 

void WorldModelInterface::setScore(int newScore, int aTeamId )
{ 
  theWorldModelCore->setScore( newScore, aTeamId );
} 


const Field& WorldModelInterface::field() const
{
  return theWorldModelCore->field(); 
}


// Vorzugsrichtungen
Vector WorldModelInterface::preferredKickDirection( const Vector& pos ) const
{ 
  return theWorldModelCore->preferredKickDirection( pos );
} 

Vector WorldModelInterface::backKickDirection( const Vector& pos ) const 
{ 
  return theWorldModelCore->backKickDirection( pos );
}


//Einarbeitung und Kommunikation
void WorldModelInterface::integrateSensorInfo() 
{ 
  theWorldModelCore->integrateSensorInfo(); 
}

Ath98String WorldModelInterface::saySituation() {
  return theWorldModelCore->saySituation();
}


// Debugging
string WorldModelInterface::idString() const {
  return theWorldModelCore->idString();
}

ostream& operator<< (ostream& os, const WorldModelInterface& worldmodel) {
  os << *(worldmodel.theWorldModelCore); 
  return os;
}




