#include <algorithm>

#include "Intention.h"

#include "skills/Skills.h"
#include "../worldmodel/Neck.h"

#include "IntentionDictionary.h"


Intention::Intention( IntentionId aId ) :
  theId( aId ), theTarget( Target() ) {

  current = theIntentions.begin();
  executerCycle = 0;
}

Intention::Intention( IntentionId aId, const Target& aTarget) :
  theId( aId ), theTarget( aTarget ) {
  current = theIntentions.begin();
  executerCycle = 0;
}

Intention::Intention( const Intention& anIntention ) :
  theId( anIntention.id() ), theTarget( anIntention.target() ) {
  copy( anIntention.thePreConditions.begin(),
	anIntention.thePreConditions.end(),
	thePreConditions.begin() );
  copy( anIntention.thePostConditions.begin(),
	anIntention.thePostConditions.end(),
	thePostConditions.begin() );
  copy( anIntention.theIntentions.begin(),
	anIntention.theIntentions.end(),
	theIntentions.begin() );
  executerCycle = 0;
}


Action Intention::execute() {
  INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << idString() << "::execute \n");
  cout << "  Intention::execute() \n";

  if( current == theIntentions.end() ){
  cout << "Intention::execute()   current == Intentions.end()\n";
    calculateIntentionOrder();
  }

	if( globalWorldModel->simStep() > 0 ) {
    cout << globalWorldModel->simStep() << " trying  " << (*current)->idString() << endl;
    cout << " PRE   : " << ((*current)->checkPreCond() ? "YES" : "NO") << endl;
    cout << " BREAK : " << ((*current)->checkBreakCond() ? "YES" : "NO") << endl;
    cout << " POST  : " << ((*current)->checkPostCond() ? "YES" : "NO") << endl << endl;
	}

  if( ( (*current)->checkBreakCond() == NO) &&
      ( (*current)->checkPostCond() == NO ) &&
      ( (*current)->checkPreCond() == YES ) ) {
    if( (*current)->id() != lastIntentionId() ) {
      // die Intention wird das erste mal gerufen --> initialisieren
      (*current)->initialize();
      setLastIntentionId( (*current)->id() );
    }
    // ausfhren der Intention
	  executerCycle = 0;
    return (*current)->execute();
  }
  else {
    ++current;
    if( current == theIntentions.end() ) {
      // alle Intentions wurden abgearbeitet --> neue Liste anlegen
	    ++executerCycle;
      calculateIntentionOrder();
		  if( executerCycle > 2 ) {
			    return Action( EMPTY_ACTION, globalWorldModel->simStep() );
		  }
    }
    // die aktuelle Intention kann nicht verwendet werden -->
    cout << "Intention::execute()! - Dummy ? " << endl;
    return Action( EMPTY_ACTION, globalWorldModel->simStep() );
  }
}


Intention::~Intention() {
  eraseAllPreConditions();
  eraseAllPostConditions();
  eraseAllIntentions();
}

bool Intention::checkPreCond() const {
    INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream << 
	  idString() <<	"::checkPreCond with size(): " << thePreConditions.size() << 
	  "\n" );
    for( list<Condition*>::const_iterator i = thePreConditions.begin();
	 i != thePreConditions.end(); ++i ) {
      if( (*i)->check() == false ) return false;
    }
    INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream << 
	  idString() << "::checkPreCond return: true \n" );
    return true;
}

bool Intention::checkPostCond() const {

  INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream 
    << idString() << "::checkPostCond with size(): " << thePostConditions.size() << endl );  
  for( list<Condition*>::const_iterator i = thePostConditions.begin();
       i != thePostConditions.end(); ++i ) {
    if( (*i)->check() == false ) return false;
  }    
  INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream << 
	idString() << "::checkPostCond return: true \n" );
  return true;
}

bool Intention::checkBreakCond() const{
  INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream << 
	idString() << "::checkBreakCond with size(): " << thePostConditions.size() << 
	"\n" );  
  for( list<Condition*>::const_iterator i = theBreakConditions.begin();
       i != theBreakConditions.end(); ++i ) {
    if( (*i)->check() == true ) return true;
  }    
  INFO( Log::INTENTIONS,  16,  idString() , globalLog->infoStream << 
	idString() << "::checkBreakCond return: false \n" );
  return false;
}  

void Intention::calculateIntentionOrder() {}
