#include "ParallelWalkIntention.h"

#include "GetBallIntention.h"
#include "EmergencyIntention.h"

#include "../basicIntentions/DummyIntention.h"
#include "../basicIntentions/GoToIntention.h"
#include "../basicIntentions/WaitIntention.h"

#include "../conditions/KickConditions.h"
#include "../conditions/InterceptCondition.h"
#include "../conditions/WorldModelStatesConditions.h"

#include "../../basic/Timer.h"

ParallelWalkIntention::ParallelWalkIntention() :
  ComplexIntention( theId ){
  addPreCondition( new KickImpossibleCondition() );
  
  addPreCondition( new InterceptCondition(  InterceptCondition::SMART ) );

  addPreCondition( new GameStateCondition(~(WorldModelInterface::FREE_KICK |
					    WorldModelInterface::CATCH_BALL_AND_FREE_KICK) )  );

  addPostCondition( new KickPossibleCondition() );
  addPostCondition( new InterceptCondition( InterceptCondition::ONE ) );

  calculateIntentionOrder();
}

void ParallelWalkIntention::calculateIntentionOrder() {
  initialize();
  theIntentions.push_back( new GoToIntention( target() ) );
  current = theIntentions.begin();
}


 
void ParallelWalkIntention::initialize(){
  if( globalWorldModel->player().pos().y() >
      globalWorldModel->ball().pos().y() ){
    if( globalWorldModel->ball().pos().y() < 20 ){
    // y-Koord. sollte groesser als ball koord sein
    setTarget( Vector( globalWorldModel->ball().pos().x() -0.1,
		       globalWorldModel->ball().pos().y() +10 ) );
    }
    else{
      setTarget( Vector( globalWorldModel->ball().pos().x() - 2,
			 max( globalWorldModel->ball().pos().y() + 7.5, 32 ) ) );
    }
  }
  else if( globalWorldModel->player().pos().y() < globalWorldModel->ball().pos().y() ){
    if( globalWorldModel->ball().pos().y() > -20 ){
      setTarget( Vector( globalWorldModel->ball().pos().x() -0.1,
			 globalWorldModel->ball().pos().y() - 10 ) );
    }
    else{
       setTarget( Vector( globalWorldModel->ball().pos().x() - 2,
			 max( globalWorldModel->ball().pos().y() - 7.5, -32 ) ) );
    }
  }
  else{
    setTarget( Vector( globalWorldModel->ball().pos().x() - 5,
		       globalWorldModel->ball().pos().y() ) );
  }
}
