#ifndef _SCAN_FIELD_INTENTION_H_
#define _SCAN_FIELD_INTENTION_H_

#include "../Intention.h"

#include "../basicIntentions/WaitIntention.h"

#include "../../basic/Timer.h"

/** Aufgabe der ScanFieldIntention ist es bei Bedarf das Feld zu scannen,
 um Infomationen zu sammeln. Sie soll dazu dienen, beim Verlust der Orientierung
 bzw. Verlust der Ball-Position, Daten zu sammeln, um das Weltbild wieder
audf einen Stand zu bringen, der es erlaubt, Entscheidungen zu treffen. */

/** Die ScanFieldIntention soll das Spielfeld scannen, wenn die Ballposition unbekannt
    ist (Reliability >= RANDOM). Dazu wird zun\"achst der Nacken zum Spielfeldmittelpunkt
    gedreht und der Sichtwinkel auf 180 Grad gesetzt. Der Simstep wird festgehalten und
    es wird gewartet, bis eine neue See-Info kommt. */

class ScanFieldIntention : public Intention {
 public:
  static const IntentionId theId = 31;

  /** Konstruktoren. */
  ScanFieldIntention();
  ScanFieldIntention( const double reliability );
   
  Action execute();
  void initialize();

 private: 
  void calculateIntentionOrder();
  /** Flag, das anzeigt, ob der erste Durchlauf ansteht. */
  int theExecuteStep;
  int theSimStepOfEnterScanField;
  int agentTimeOfLastStateChange;
  int theSimStepOfLastStateChange;
};

inline void ScanFieldIntention::calculateIntentionOrder(){
  initialize();
}

inline void ScanFieldIntention::initialize(){
  theExecuteStep = 1; // in den State 1 kann er nicht wirklich gelangen, wird sofort erhht
  theSimStepOfEnterScanField = globalWorldModel->simStep();
  agentTimeOfLastStateChange = globalTimer->getAgentTime();
  theSimStepOfLastStateChange = -1;
}



#endif
