#include "TrivialPassIntention.h"

#include "GetBallIntention.h"
#include "InterceptBallIntention.h"
#include "EmergencyIntention.h"

#include "../basicIntentions/KickIntention.h"
#include "../basicIntentions/DummyIntention.h"

#include "../conditions/KickConditions.h"
#include "../conditions/ConfidenceCondition.h"

#include "../../basic/Timer.h"


TrivialPassIntention::TrivialPassIntention() :
  Intention( theId) {
  addPreCondition( new KickPossibleCondition() );
  addPreCondition( new ConfidenceCondition( BALL_ID, 0.9 ) );
  addPostCondition( new KickImpossibleCondition() );
  // was jetzt folgt, dient nur zu Testzwecken
  if( globalWorldModel->situation().ball().pos().x() > 15 ){
    setTarget( Target( globalWorldModel->field().ourGoal() ) );
  }
  else{
    setTarget( Target( globalWorldModel->field().enemyGoal() ) );
  }
  calculateIntentionOrder();
}


TrivialPassIntention::TrivialPassIntention( const Target& aTarget ) :
  Intention( theId, aTarget ) {
  addPreCondition( new KickPossibleCondition() );
  addPreCondition( new ConfidenceCondition( BALL_ID, 0.9 ) );
  addPostCondition( new KickImpossibleCondition() );
  calculateIntentionOrder();
}


Action TrivialPassIntention::execute() {
  INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << idString() << "::execute\n" );
  if( current != theIntentions.end() ) {
    if( (*current)->checkPostCond() ) {
      INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << idString() << "::checkPost==TRUE\n" );
      ++current;
      theIntentions.pop_front();
      return execute();
    }

    if( (*current)->checkPreCond() ) {
      INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream
        << idString() << "::checkPre==TRUE\n");
      INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream
        << idString() << "::intentions.size(): " << theIntentions.size()
	    <<  '\n' );

      return (*current)->execute();
    }

    // simple Reparatur
    INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << idString() << " repair\n" );
    if( globalTimer->isEmergency() ){
      WARNING( Log::INTENTIONS, "Emergency in DirectPass\n" );
      return globalEmergencyIntention->execute();
    } 
    theIntentions.push_front( new InterceptBallIntention() );
    current = theIntentions.begin();
    return (*current)->execute();
  }
  else {
    // keine Intentions mehr vorhanden (Abbruchbedingung f. Rekursion
    // hier muesste jetzt reaktives Verhalten erfolgen
    WARNING( Log::INTENTIONS, "No valid Action found\n" );
    return Action( EMPTY_ACTION, globalWorldModel->simStep() );
  }  
}


void TrivialPassIntention::calculateIntentionOrder(){
  INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << 
	idString() <<	"::calculateIntentionOrder \n");
  theIntentions.push_back( new KickIntention( target() ) );
  current = theIntentions.begin();
}


int TrivialPassIntention::searchTeamMate(){
  Situation scenario( globalWorldModel->situation() );
  const ModelledPlayer& myself = globalWorldModel->player();
  // remove Players
  int count = scenario.playerCount();
  while( count > 0 ){
    if( scenario.playerWithIndex( count ).id() == myself.id() ){
      // Spieler bleibt selbst drin
      ++count;
      continue;
    }
    // Gegner-TW auch
    if( scenario.playerWithIndex( count ).isGoalie() &&
	scenario.playerWithIndex( count ).team() == OUR_TEAM_ID ){
	scenario.removePlayerWithIndex( count );
	++count;	
	continue;	
    }
    if( ( myself.pos() -
	  scenario.playerWithIndex( count ).pos() ).length_sqr() > 35*35 ){
      scenario.removePlayerWithIndex( count );
      ++count;
    }
  }
  // ab jetzt nur noch relevante Spieler enthalten
  // Beginn Simulation
  Vector prefDir = globalWorldModel->preferredKickDirection( myself.pos() );
  // kick in 10 Grad schritten
  double prefDirAngle = prefDir.angle();
  double prefDirLength = 2.2;
  double kickAngle = 0.0;
  Vector newSpeed( prefDirLength, prefDirAngle, POLAR, RADIAN );
  ModelledBall simBall( scenario.ball() );
  simBall.setSpeed( newSpeed );
  scenario.setBall( simBall );
  int stepsToBall = scenario.stepsOfNearestPlayer();
  int ourStepsToBall = scenario.stepsOfNearestPlayer( OUR_TEAM_ID );
  int playerIdToPassTo = globalWorldModel->id();
  if( stepsToBall - 2 > ourStepsToBall ){
    // Pass OK, 2 steps besser
    playerIdToPassTo = scenario.nearestPlayer( OUR_TEAM_ID ).id();
  }
  else{
    // alle anderen Richtungen
    for( int i = 0; i< 17; ++i ){
      kickAngle += Ath98_PI_10_DEGREE;
      newSpeed.setPolar( prefDirLength, prefDirAngle + kickAngle,
			 RADIAN );
      simBall.setSpeed( newSpeed );
      scenario.setBall( simBall );
      stepsToBall = scenario.stepsOfNearestPlayer();
      ourStepsToBall = scenario.stepsOfNearestPlayer( OUR_TEAM_ID );
      if( stepsToBall - 2 > ourStepsToBall ){
	// Pass OK, 2 steps besser
	playerIdToPassTo = scenario.nearestPlayer( OUR_TEAM_ID ).id();
      }
      double negKickAngle = kickAngle * (-1);
      newSpeed.setPolar( prefDirLength, prefDirAngle + negKickAngle,
			 RADIAN );
      simBall.setSpeed( newSpeed );
      scenario.setBall( simBall );
      stepsToBall = scenario.stepsOfNearestPlayer();
      ourStepsToBall = scenario.stepsOfNearestPlayer( OUR_TEAM_ID );
      if( stepsToBall - 2 > ourStepsToBall ){
	// Pass OK, 2 steps besser
	playerIdToPassTo = scenario.nearestPlayer( OUR_TEAM_ID ).id();
      }
    }
  }
  INFO( Log::INTENTIONS, 10, idString(), globalLog->infoStream << idString()
    << "Player to Pass to: " << globalWorldModel->situation().playerWithId( playerIdToPassTo ) << endl );
  return playerIdToPassTo;
}
   

void TrivialPassIntention::initialize(){
  eraseAllPreConditions();
  int playerIdToPassTo = searchTeamMate();
  if( playerIdToPassTo != globalWorldModel->id() ){
    setTarget( Target( Target::PLAYER, playerIdToPassTo,
		       globalWorldModel->simStep() + 1 ) );
  addPreCondition( new KickPossibleCondition() );
  addPreCondition( new ConfidenceCondition( BALL_ID, 0.9 ) );    
  }
  else{
    addPreCondition( new FalseCondition() );
  }
}
