#ifndef _CONDITION_H_
#define _CONDITION_H_

#include "../../basic/Log.h"

/** Die Klasse Condition ist die Basisklasse f"ur vor- und Nach-
    bedingungen, die von den Intentions benutzt werden. Jede
    abgeleitete Klasse mu\3 die Konstruktoren und mindestens
    die Methode \Ref{check} implementieren. */

class Condition{
 public:
  Condition();
  Condition( bool falseOrTrue );
  
  virtual ~Condition();
  virtual bool check() const;

 private:
  bool theFlag;
};

/** Die Klasse OrCondition verkn\"upft zwei Conditions mit einen logischen
    "ODER". */
class OrCondition : public Condition{
 public:
  OrCondition( Condition* firstCond,
	       Condition* secondCond );

  ~OrCondition();
  virtual bool check() const;

 private:
  Condition* firstCondition;
  Condition* secondCondition;
};

inline bool OrCondition::check() const{
  INFO( Log::ANY  ,  6  ,  "" , globalLog->infoStream <<
	"OrCondition::check\n" );
  return ( firstCondition->check() ||
    secondCondition->check() );
}

/** Die Klasse AndCondition verkn\"upft zwei Conditions mit einen logischen
    "UND". */
class AndCondition : public Condition{
 public:
  AndCondition( Condition* firstCond,
	       Condition* secondCond );

  ~AndCondition();
  virtual bool check() const;

 private:
  Condition* firstCondition;
  Condition* secondCondition;
};

inline bool AndCondition::check() const{
  INFO( Log::ANY  ,  6  ,  "" , globalLog->infoStream <<
	"AndCondition::check\n" );
  return ( firstCondition->check() &&
    secondCondition->check() );
}

/** Die Klasse NotCondition liefert die Negation einer Condition. */
class NotCondition : public Condition{
 public:
  /** Konstruktor, der mit Zeiger auf die Condition aufgerufen wird. */
  NotCondition( Condition* firstCond );
  /** Expliziter Destruktor, der daf\"ur sorgt, da\3 die \"ubergebene
      Condition gel\"oscht wird. */
  ~NotCondition();

  virtual bool check() const;

 private:
  /**@name Die Condition, die negiert werden soll. */
  //@{
  ///
  Condition* firstCondition;

  //@}
};

inline bool NotCondition::check() const{
  INFO( Log::CONDITIONS  ,  6  ,  "" , globalLog->infoStream <<
	"NotCondition::check a: " << firstCondition->check()
	<< '\n' );

  return ( !firstCondition->check() );
}

/** Die TrueCondition liefert immer true zur\"uck. */
class TrueCondition : public Condition{
 public:
  TrueCondition();
  bool check() const;
};

inline bool TrueCondition::check() const{
  return true;
}

/** Die FalseCondition liefert immer false zur\"uck. */
class FalseCondition : public Condition{
 public:
  FalseCondition();
  bool check() const;
};

inline bool FalseCondition::check() const{
  return false;
}

#endif
