#include "GoalieCatchIntention.h"

#include "GoalieConditions.h"
#include "../conditions/WorldModelStatesConditions.h"

#include "../../worldmodel/WorldModelInterface.h"
#include "../../worldmodel/Neck.h"
#include "../skills/Skills.h"


GoalieCatchIntention::GoalieCatchIntention() : Intention( theId ) 
{
  addPreCondition( new NotCondition( new AfterGoalKickCondition() ) );
  addPreCondition( new CatchCondition() );
  addPostCondition( new GameStateCondition( WorldModelInterface::CATCH_BALL_AND_FREE_KICK ) );
}


Action GoalieCatchIntention::execute()
{
  INFO( Log::INTENTIONS,  10,  idString() , globalLog->infoStream << idString() << "::execute with target :\n" );

  globalSkills->resetSituation( globalWorldModel->situation() );
  cerr << globalWorldModel->simStep() << "Catch\n";

  Vector playerToBall = globalWorldModel->ball().pos() - globalWorldModel->player().pos();
  Vector ortho = playerToBall.orthogonal(); // linksgedreht
  double ballDir = ortho * globalWorldModel->ball().speed();
  double catchAngle = globalWorldModel->player().angleOfBodyTo( globalWorldModel->ball().pos() );
  const double catchAngleGain = normalizeAngle( atan( 0.5 * globalConf->catchAreaWidth / globalConf->catchAreaLength) );
/*
  if( ballDir > 0 ) {  // Ball bewegt sich von uns aus nach links
    catchAngle = normalizeAngle( catchAngle + catchAngleGain - radFromDegree(4) ); //(4 = safety)
  }
  else {
    catchAngle = normalizeAngle( catchAngle - catchAngleGain + radFromDegree(4) ); //(4 = safety)
  }
*/
  globalNeck->setNeckBehaviour( NECK_TO_BALL );
  globalNeck->setViewBehaviour( VIEW_WIDTH_NARROW, VIEW_QUALITY_HIGH,  VERY_HIGH_PRIORITY);

  return globalSkills->simpleCatch( catchAngle );
}
