#include "GoalieConditions.h"
#include "../../analyzer/Analyzer.h"


GoalieInterceptCondition::GoalieInterceptCondition( GoalieInterceptBehaviour aBehaviour ) :
  Condition(), theBehaviour(aBehaviour)
{
  needNewInterceptDecision = YES;
  lastInterceptDecision = NO;
}


bool GoalieInterceptCondition::check() const {

//cout << globalWorldModel->simStep() << "enter GoalieInterceptCondidion::check()\n";

  const ModelledBall& aktBall = globalWorldModel->ball();
  ModelledBall simBall = aktBall;

//cout << " BALL : " << aktBall << endl;

  int goalieSteps = globalAnalyzer->getOurGoalieStepsToBall();
  int ourSteps = globalAnalyzer->getNearestTeammateStepsToBall();
  int enemySteps = globalAnalyzer->getNearestEnemyStepsToBall();
  const ModelledPlayer& nearestEnemy = globalWorldModel->situation().nearestPlayer( THEIR_TEAM_ID );
/*
cout << "  goalieSteps = " << goalieSteps << endl;
cout << "  ourSteps = " << ourSteps << endl;
cout << "  enemySteps = " << enemySteps << endl;
*/
  simBall.simulateSteps(goalieSteps);

  double ballIntercectY = aktBall.pos().y()
  											  + ( globalWorldModel->field().ourGoal().x() - aktBall.pos().x() )
  											    * aktBall.speed().y() / aktBall.speed().x();

	const double  enemyDist = (globalWorldModel->field().ourGoal() - nearestEnemy.pos()).length();
	const int addGoalieGain = (enemyDist < 8.0 ? 2 : (enemyDist < 15.0 ? 1 : 0) );

// Entscheidung stabilisieren
	if( lastInterceptDecision == YES ) {
		if( goalieSteps - addGoalieGain  <= min( floor(enemySteps * 1.18 ), enemySteps + 1 ) ) {
//			cout << "  return last = YES\n";	
    	return lastInterceptDecision;
		}
		else {
	    setLastInterceptDecision(NO);
//			cout << "  return NO\n";	
    	return NO;
  	}
	}
	else { //lastInterceptDecision == NO
		if( ( goalieSteps - addGoalieGain <= enemySteps )
						&& ( simBall.pos().y() > -0.5 * globalWorldModel->field().penaltyAreaWidth() )
						&& ( simBall.pos().y() <  0.5 * globalWorldModel->field().penaltyAreaWidth() ) 
						&& ( simBall.pos().x() < globalWorldModel->field().ourGoal().x() + globalWorldModel->field().penaltyAreaLength() ) ) {
	    setLastInterceptDecision(YES);
	//		cout << "  return YES\n";	
    	return YES;
		}
		else {
//			cout << "  return last = NO\n";	
    	return lastInterceptDecision;
		}
  }



}


CatchCondition::CatchCondition() :
  Condition() {}

bool CatchCondition::check() const{
//  double maxCatchLength = sqrt(globalConf->catchAreaLength * globalConf->catchAreaLength + globalConf->catchAreaWidth * globalConf->catchAreaWidth);
  double maxCatchLength = globalConf->catchAreaLength;

  ModelledBall ball = globalWorldModel->ball();
  ModelledPlayer mySelf = globalWorldModel->player();
  double dist = mySelf.distTo( ball.pos() );
  ball.simulateSteps(1);
//  mySelf.simulateSteps(1);
//  double nextDist = mySelf.distTo( ball.pos() );
  double neededSteps = globalSkills->gotoRegion(ball.pos(), maxCatchLength - 0.2,
                                                globalConf->interceptBallStaminaLimit,
                                                globalAnalyzer->getGoalieInterceptRunBehaviour() ).size();
  int enemySteps = globalAnalyzer->getNearestEnemyStepsToBall();

  if( isBallInCatchableArea() == NO ) {
    return NO;
  }
  /* wird der Catch relativ riskant, kein Gegner bekommt den Ball im nchsten Takt,
     es wird kein Tor fallen und ich habe im nchsten Takt eine bessere CatchPosition -> warte noch */
  if( ( dist > maxCatchLength - 0.60 ) && ( enemySteps > 1 ) 
		    && ( ball.pos().x() > globalWorldModel->field().ourGoal().x() )
		    && ( neededSteps < 2 ) ) {
    return NO;
  }
  // ist Catchen erlaubt ?
  if( ( globalWorldModel->simStep() - globalWorldModel->simStepOfLastCatch() >= globalConf->catchCycle )
       && ( globalWorldModel->simStep() - globalWorldModel->simStepOfLastFreekick() > 4 )
	   && ( globalWorldModel->simStep() - globalWorldModel->simStepWhenCatchedBallIsReleased() > 4 ) ) {
    return YES;
  }
  return NO;
}


bool CatchCondition::isBallInCatchableArea() const{
  return ( 
    globalWorldModel->field().isPositionInPenaltyArea( globalWorldModel->ball().pos() )
    && globalWorldModel->player().distTo( globalWorldModel->ball().pos() ) < globalConf->catchAreaLength );
/*        sqrt(globalConf->catchAreaLength * globalConf->catchAreaLength + globalConf->catchAreaWidth * globalConf->catchAreaWidth)
        - 0.20); // safety
*/
}



AfterGoalKickCondition::AfterGoalKickCondition() :
  Condition() {}

bool AfterGoalKickCondition::check() const{
  return ( ( globalWorldModel->simStep() - globalWorldModel->simStepWhenCatchedBallIsReleased() ) < 3 
			  && globalWorldModel->simStepWhenCatchedBallIsReleased() != 0 );
}

GoalKickAfterCatchCondition::GoalKickAfterCatchCondition() :
  Condition() {}

bool GoalKickAfterCatchCondition::check() const{
  return globalAnalyzer->isGoalKickAfterCatchPossible();
}

GoaliePositionTurnCondition::GoaliePositionTurnCondition() :
  Condition() {}


bool GoaliePositionTurnCondition::check() const{
  return ( fabs( deviationBetween( globalWorldModel->player().bodyDir(),
  						 ( globalAnalyzer->getGoaliePositionTurnVector() - globalWorldModel->player().pos() ).angle() ) ) < radFromDegree( 5.0 ) );
}


