#include "GoalieGoalKickIntention.h"

#include "GoalieMoveIntention.h"

#include "GoalieConditions.h"
#include "../conditions/WorldModelStatesConditions.h"
#include "../conditions/KickConditions.h"

#include "../basicIntentions/DummyIntention.h"
#include "../basicIntentions/KickIntention.h"
#include "../basicIntentions/TurnIntention.h"
#include "../basicIntentions/WaitIntention.h"
#include "../complexIntentions/ScanFieldIntention.h"

#include "../complexIntentions/PassIntention.h"

#include "../../worldmodel/WorldModelInterface.h"
#include "../../worldmodel/Neck.h"


GoalieGoalKickIntention::GoalieGoalKickIntention():
  Intention( theId ){
  addPreCondition( new GameStateCondition( WorldModelInterface::CATCH_BALL_AND_FREE_KICK ) );
  // wenn die GoalKickAfterCatchCondition()==NO ist, liegt trotzdem ein Emergency-Kick vor
  // ist sie ==YES, wird auch einen Spieler angepasst 	
  //addPreCondition( new GoalKickAfterCatchCondition() );
    
  addPostCondition( new GameStateCondition( WorldModelInterface::PLAY_ON ) );
  addPostCondition( new KickImpossibleCondition() );
  calculateIntentionOrder();
}


void GoalieGoalKickIntention::calculateIntentionOrder(){
//cout << "GoalieGoalKickIntention: bin jetzt drin." << endl;

  // Die Wartezeiten sollten dem Gegnerverhalten angepasst werden
  // erstmal umsehen...
  addIntention( new TurnIntention( Target( Vector ( 0, globalWorldModel->player().pos().y() ) ) ) );
  globalNeck->setNeckBehaviour( NECK_TO_MIDDLE, ALIGN_CENTER, VERY_HIGH_PRIORITY );
  globalNeck->setViewBehaviour( VIEW_WIDTH_WIDE, VIEW_QUALITY_HIGH, VERY_HIGH_PRIORITY );
  addIntention( new WaitIntention(50));

  // beamen
  addIntention( new GoalieMoveIntention( Target::GOALIE_MOVE )); /*Target(Vector(-37.0, 10.0)) */

  // es muss jetzt nur noch gekickt werden

  // ACHTUNG ! PARALLEL_WALK_DOWN ist umgebogen zu dem Passtarget nach dem beamen...
  addIntention( new KickIntention( Target(Target::GOAL_KICK_AFTER_CATCH_PASS_TARGET ), 0.6 , 3 ));
  // hartgekodeter Notfallabstoss	
  // addIntention( new KickIntention( Target::GOALIE_KICK_TO_THE_OTHER_SIDE, 2.7, 3 ) );

  current = theIntentions.begin();
}

void GoalieGoalKickIntention::initialize() {
  INFO( Log::INTENTIONS  ,  6  ,  idString() , globalLog->infoStream <<	" ENTER " << idString() << "::initialize \n" );
  
  eraseAllIntentions();
  calculateIntentionOrder();

  INFO( Log::INTENTIONS  ,  6  ,  idString() , globalLog->infoStream <<	" LEAVE " << idString() << "::initialize \n" );
}
