#include "GoaliePositionIntention.h"

#include "GoalieConditions.h"

#include "../basicIntentions/TurnIntention.h"
#include "../basicIntentions/GoToIntention.h"
#include "../basicIntentions/DummyIntention.h"

#include "../conditions/ConfidenceCondition.h"
#include "../conditions/PositionConditions.h"
#include "../conditions/PlayerDependentConditions.h"
#include "../conditions/KickConditions.h"
#include "../conditions/WorldModelStatesConditions.h"

#include "../targets/Target.h"

#include "../skills/Skills.h"

#include "../../worldmodel/Neck.h"
#include "../../worldmodel/WorldModelInterface.h"

GoaliePositionIntention::GoaliePositionIntention() :
  Intention( theId ){
  // positionieren kann ich mich immer
  addPreCondition( new TrueCondition() );
  addPreCondition( new ConfidenceCondition( BALL_ID, 0.5 ) );

  // positionieren ist sehr dynamisch und nie wirlich fertig
  //addPostCondition( new FalseCondition() );
  addPostCondition( new PositionCondition( Target( Target::GOALIE_STANDARD_POSITION ), 0.20 ) );
  addPostCondition( new GoaliePositionTurnCondition() );

  // ist eine dieser Bedingungen erfllt, dann breche ich die Intention sofort ab
  addBreakCondition( new CatchCondition() );
  addBreakCondition( new KickPossibleCondition() );
  addBreakCondition( new AndCondition ( new GoalieInterceptCondition() ,
                                        new NotCondition ( new GameStateCondition ( WorldModelInterface::FREE_KICK ) ) ) );
}


void GoaliePositionIntention::initialize() {
  INFO( Log::INTENTIONS  ,  6  ,  idString() , globalLog->infoStream
    << " ENTER " << idString() << "::initialize \n" );

  calculateIntentionOrder();
}



Action GoaliePositionIntention::execute(){
/*
cout << endl << "GoaliePositionIntention::execute()\n";
cout << "BALL : " << globalWorldModel->ball() << endl;;
cout << "-> Goalie Position Target : " << Target(Target::GOALIE_STANDARD_POSITION).pos() << endl;
*/

  globalNeck->setViewBehaviour( VIEW_WIDTH_NARROW, VIEW_QUALITY_HIGH, VERY_HIGH_PRIORITY );
  globalNeck->setNeckBehaviour( NECK_TO_BALL, ALIGN_CENTER, VERY_HIGH_PRIORITY );

  return Intention::execute();
}



void GoaliePositionIntention::calculateIntentionOrder() {
  INFO( Log::INTENTIONS  ,  6  ,  idString() , globalLog->infoStream 
    << " ENTER " << idString() << "::calculate \n" );
// cerr << "GoaliePositionIntention::calculateIntentionOrder()\n";
  eraseAllIntentions();

	addIntention( new GoToIntention (  Target( Target::GOALIE_STANDARD_POSITION ),
				     100, globalConf->interceptBallStaminaLimit ) );

  // HACK TurnIntention ist verbogen
  addIntention( new TurnIntention() );
  current = theIntentions.begin();

}
