#include "Role.h"

#include "../../worldmodel/Neck.h"

Role::Role(RoleId aId, string aName) : theId(aId), theName(aName)
{
  initializeIntentionDictionary();
  executorCycle = 0;
}

Role::~Role()
{
}

void Role::setNeckBehaviour(){
  INFO( Log::ROLE,  10,  name() , globalLog->infoStream
	<< "ENTER " << name() <<	"::setNeckBehaviour\n");
  const ModelledPlayer& player = globalWorldModel->player();
  ModelledBall simBall = globalWorldModel->ball();
  simBall.simulateSteps();



  if( (simBall.posConfidence().reliability() > 0.4) // Scanfield darf nicht gestoert werden
      && (globalWorldModel->gameState() != WorldModelInterface::CATCH_BALL_AND_FREE_KICK) ) { 
	  if( player.distTo( simBall.pos() ) < 15 ){
	    globalNeck->setNeckBehaviour( NECK_TO_BALL, ALIGN_CENTER, HIGH_PRIORITY, simBall.pos() );
	    if( player.isGoalie() == YES ){
	      globalNeck->setViewBehaviour( VIEW_WIDTH_NARROW, VIEW_QUALITY_HIGH, HIGH_PRIORITY );
	    }
	    else{
	      globalNeck->setViewBehaviour( VIEW_WIDTH_NORMAL/*VIEW_WIDTH_NARROW*/, VIEW_QUALITY_HIGH, HIGH_PRIORITY );
	    }
	  }
	  else{
	    if(player.distTo( simBall.pos() ) > 25) {
	      globalNeck->setNeckBehaviour( NECK_TO_BALL, ALIGN_CENTER, HIGH_PRIORITY, simBall.pos() );
	      globalNeck->setViewBehaviour( VIEW_WIDTH_WIDE,VIEW_QUALITY_HIGH, MEDIUM_PRIORITY );
	    }
	    else{
	      globalNeck->setNeckBehaviour( NECK_TO_BALL, ALIGN_CENTER, HIGH_PRIORITY, simBall.pos() );
	      globalNeck->setViewBehaviour( VIEW_WIDTH_NORMAL, VIEW_QUALITY_HIGH, LOW_PRIORITY );
	    }
	  }
  }
  INFO( Log::ROLE,  10,  name() , globalLog->infoStream	<< "LEAVE " << name() <<	"::setNeckBehaviour\n");
}


Action Role::execute() 
{
if( globalWorldModel->player().stamina() < 1201 ) 
  cerr << globalWorldModel->player().id() << " - Stamina zu niedrig = " << globalWorldModel->player().stamina() << endl;
if( globalWorldModel->player().stamina() > 4000 ) 
  cerr << globalWorldModel->player().id() << " - Stamina zu hoch = " << globalWorldModel->player().stamina() << endl;
//if( globalWorldModel->player().isGoalie() == YES ) {
//  cout << " -->" << globalWorldModel->simStep() << " Ball :\n";
//  cout << globalWorldModel->ball() << endl;
//}


  if( current == theIntentions.end() ){
    // dieser Fall kann eintreten, wenn eine neue Liste erzeugt werden sollte,
    // im deliberate, der analyzer gerufen wurde, der wiederum den controller ruft,
    // welcher feststellt dass eine Aktion schnell abgeschickt werden muss, und
    // damit execute ruft ohne das eine neue Liste erzeugt wurde bzw. current auf
    // begin gesetzt wurde
    // HACK --> in diesem Takt liefere ich eine EMPTY_ACTION
    needNewIntentionList = YES;
// cerr << "FALLBACK WEGEN CONTROLLER -----  EMPTY ACTION -----\n";
    return Action( EMPTY_ACTION, globalWorldModel->simStep() );
  }
/*
if( (globalWorldModel->simStep() > 0)  ) {
  cout << globalWorldModel->simStep() << " trying  " << (*current)->idString() << endl;
  cout << " PRE   : " << ((*current)->checkPreCond() ? "YES" : "NO") << endl;
  cout << " BREAK : " << ((*current)->checkBreakCond() ? "YES" : "NO") << endl;
  cout << " POST  : " << ((*current)->checkPostCond() ? "YES" : "NO") << endl << endl;
  cout << "Unser Ball = " << globalWorldModel->ball() << endl;
  cout << "Myself = " << globalWorldModel->player() << endl;
}
*/

/*
if( (globalWorldModel->ball().pos() - globalWorldModel->correctSituation().ball().pos()).length() > 0.5
    || ( (globalWorldModel->ball().speed() - globalWorldModel->correctSituation().ball().speed()).length() > 0.2 ) ){
// der Ball ist mist, zum vergleich muss fullstate angeschaltet sein
  cout << "-----------------------------------------\n";
  cout << "Unser Ball = " << globalWorldModel->ball() << endl;
  cout << "Echter Ball = " << globalWorldModel->correctSituation().ball() << endl;
  cout << "-----------------------------------------\n";
}*/

  if( ( (*current)->checkBreakCond() == NO) &&
      ( (*current)->checkPostCond() == NO ) &&
      ( (*current)->checkPreCond() == YES ) ) {
    // BreakCondition nicht erfllt, PreCondition erfllt, PostCondition nicht erfllt --> die Intention wird abgearbeitet
    if( (*current)->id() != lastIntentionId() /*|| sitClassChange == YES*/ ) {
      // die Intention wird das erste mal gerufen --> initialisieren
      (*current)->initialize();
      setLastIntentionId( (*current)->id() );
    }
    // ausfhren der Intention
	executorCycle = 0;
    return (*current)->execute();
  }
  else {
    ++current;
    if( current == theIntentions.end() ) {
      // alle Intentions wurden abgearbeitet --> neue Liste anlegen
      needNewIntentionList = YES;
	  ++executorCycle;
      deliberate();
	  if( executorCycle > 2 ) {
  	  const ModelledPlayer& mySelf = globalWorldModel->player();
  	  if( (mySelf.pos() - globalWorldModel->ball().pos()).length() < mySelf.kickRange() ) {
  	    return Action( KICK_ACTION, globalWorldModel->simStep(), 100.0, normalizeAngle( (globalWorldModel->field().enemyGoal() - mySelf.pos()).angle() - mySelf.bodyDir() ) );
  	  }
//cerr << globalWorldModel->simStep() << " - " << mySelf.id() << "  keine passende Intention gefunden !\n";
  		return Action( EMPTY_ACTION, globalWorldModel->simStep() );
	  }
      // HACK -> RE_HACK
//      if( lastIntentionId() == DummyIntention::theId ){
//cerr << "FALLBACK in STANDARD_ROLE -----  EMPTY ACTION -----\n";
//        return Action( EMPTY_ACTION, globalWorldModel->simStep() );
//      }
    }
    // die aktuelle Intention kann nicht verwendet werden -->
    return execute();
  }
}// end of execute() 