#ifndef _ROLE_INTENTION_H_
#define _ROLE_INTENTION_H_

#include <list>
#include <string>

#include "../../basic/Action.h"
#include "../../basic/Vector.h"
#include "../Intention.h"


typedef unsigned short RoleId;

class Role {
 public:

  typedef unsigned short RoleId;

  /** Konstruktor. */
  Role(RoleId aId, string aName);

  /** Destruktor. */
  virtual ~Role();

  /** Der virtuelle Deliberator -- bestimmt die Intentions */
  // vormals PlaySoccerIntention::calculateIntentionOrder()
  virtual void deliberate()=0;

  /** Der Executer */
  Action execute();

  /** set Methoden */
  void setHomePosition( Vector homePosition ) ;
  void setUpperOffSet( double upperOffSet ) ;
  void setLowerOffSet( double lowerOffSet ) ;
  void setLeftOffSet( double leftOffSet ) ;
  void setRightOffSet( double rightOffSet ) ;
  void setBeamPositionOurKickOff( Vector beamPositionOurKickOff ) ;
  void setBeamPositionEnemyKickOff( Vector beamPositionEnemyKickOff ) ;


  /** Zugriffsmethoden */
  RoleId id() const;
  string name() const;  
  Vector homePosition() const;
  double upperOffSet() const;
  double lowerOffSet() const;
  double leftOffSet() const;
  double rightOffSet() const;
  Vector beamPositionOurKickOff() const;
  Vector beamPositionEnemyKickOff() const;


 protected:

  int executorCycle;

  long theGameState;

  bool needNewIntentionList;

  /** Liste der Intentions. */
  list<Intention*> theIntentions;

  /** Iterator, der auf die aktuelle Intention zeigt. */
  list<Intention*>::iterator current;

  /** Setzen des Default-Verhaltens des Nackens. */
  void setNeckBehaviour();

  /** Id der zuletzt gerufenen Sub-Intention */
  IntentionId theLastIntentionId;
  
  IntentionId lastIntentionId() const;

  /** Setzen der Id, der zuletzt gerufenen Sub-Intention. */
  void setLastIntentionId( IntentionId anId );

  /** Anf"ugen einer Intention. */
  void addIntention( Intention* aIntention );

  /** L"oschen der Intentionliste. */
  void eraseAllIntentions();

  /** Checken der Vorbedingungen. */
  bool checkPreCond() const;
  /** Checken der Nachbedingungen. */
  bool checkPostCond() const;

 private:
  /** Die RollenId */
  const RoleId theId;

  /** Der Name */
  const string theName;

  /** Die HomePosition */
  Vector theHomePosition;

  /** Die zur HomePosition gehrenden Offsets um ein Rechteck zu beschreiben
      in dem sich der Agent positionieren soll. Dabei ist die Homeposition
      am attraktivsten.
      Die Homeposition mu nicht Mittelpunkt des Rechtecks sein*/
  double theUpperOffSet;
  double theLowerOffSet;
  double theLeftOffSet;
  double theRightOffSet;

  /** Die Beamposition wenn wir Ansto haben */
  Vector theBeamPositionOurKickOff;

  /** Die Beamposition wenn der Gegner Ansto hat */
  Vector theBeamPositionEnemyKickOff;

};

// Ende der Klassendefinition



/** set Methoden */

inline void Role::setHomePosition( Vector homePosition ) {
  theHomePosition = homePosition;
}

inline void Role::setUpperOffSet( double upperOffSet ) {
  theUpperOffSet = upperOffSet;
}

inline void Role::setLowerOffSet( double lowerOffSet ) {
  theLowerOffSet = lowerOffSet;
}

inline void Role::setLeftOffSet( double leftOffSet ) {
  theLeftOffSet = leftOffSet;
}

inline void Role::setRightOffSet( double rightOffSet ) {
  theRightOffSet = rightOffSet;
}

inline void Role::setBeamPositionOurKickOff( Vector beamPositionOurKickOff ) {
  theBeamPositionOurKickOff = beamPositionOurKickOff;
}

inline void Role::setBeamPositionEnemyKickOff( Vector beamPositionEnemyKickOff ) {
  theBeamPositionEnemyKickOff = beamPositionEnemyKickOff;
}

inline void Role::setLastIntentionId( IntentionId anId ){
  theLastIntentionId = anId;
}



/** Zugriffsmethoden */

inline RoleId Role::id() const{
  return theId;
}

inline string Role::name() const{
  return theName;
}

inline Vector Role::homePosition() const{
  return theHomePosition;
}

inline double Role::upperOffSet() const{
  return theUpperOffSet;
}

inline double Role::lowerOffSet() const{
  return theLowerOffSet;
}

inline double Role::leftOffSet() const{
  return theLeftOffSet;
}
inline double Role::rightOffSet() const{
  return theRightOffSet;
}

inline Vector Role::beamPositionOurKickOff() const{
  return theBeamPositionOurKickOff;
}

inline Vector Role::beamPositionEnemyKickOff() const{
  return theBeamPositionEnemyKickOff;
}

inline IntentionId Role::lastIntentionId() const{
  return theLastIntentionId;
}

inline void Role::addIntention( Intention* aIntention ){
  theIntentions.push_back( aIntention );
}

inline void Role::eraseAllIntentions(){
  for_each( theIntentions.begin(), theIntentions.end(), eraseIntention );
  theIntentions.clear();
}

inline bool Role::checkPreCond() const{
  return true;
}

inline bool Role::checkPostCond() const{
  return false;
}



#endif
