#include "StandardGoalieRole.h"
#include "Role.h"

#include "../goalieIntentions/GoalieConditions.h"
#include "../goalieIntentions/GoalieCatchIntention.h"
#include "../goalieIntentions/GoalieGoalKickIntention.h"
#include "../goalieIntentions/GoalieMoveIntention.h"
#include "../goalieIntentions/GoalieInterceptIntention.h"
#include "../goalieIntentions/GoaliePositionIntention.h"

#include "../basicIntentions/DummyIntention.h"
#include "../basicIntentions/KickIntention.h"
#include "../basicIntentions/MoveIntention.h"
#include "../basicIntentions/PMoveIntention.h"
#include "../basicIntentions/TurnIntention.h"
#include "../basicIntentions/WaitIntention.h"


#include "../complexIntentions/ScanFieldIntention.h"
#include "../basicIntentions/InitialScanIntention.h"
#include "../complexIntentions/GetBallIntention.h"
#include "../complexIntentions/PassIntention.h"

#include "../conditions/WorldModelStatesConditions.h"

#include "../../analyzer/Analyzer.h"

#include "../../basic/Timer.h"



StandardGoalieRole::StandardGoalieRole() : Role(1, "StandardGoalieRole")
{
  addIntention( new DummyIntention() );
  current = theIntentions.begin();

  setHomePosition( globalWorldModel->basePosition(WorldModelInterface::PLAY_ON, OUR_TEAM_ID) );
  setBeamPositionOurKickOff( globalWorldModel->initialPosition(WorldModelInterface::KICK_OFF, OUR_TEAM_ID) );
  setBeamPositionEnemyKickOff( globalWorldModel->initialPosition(WorldModelInterface::KICK_OFF, THEIR_TEAM_ID) );
  
  needNewIntentionList = YES;
  theGameState = -1; 
}

StandardGoalieRole::~StandardGoalieRole()
{
}

void StandardGoalieRole::deliberate()
{

  INFO( Log::ROLE,  6,  name() , globalLog->infoStream << name() << "::deliberate\n");
//cout << "SGR::deliberate()\n";
  // die Analyzer-Komponente soll vorberechnen ob bestimmte Intentions mglich sind
  // und evtl. schon Targets berechnen
  globalAnalyzer->analyze();

	const ModelledPlayer& myself = globalWorldModel->player();

  if( ( theGameState != globalWorldModel->gameState() ) 
  	   || (globalWorldModel->simStepOfLastRefereeDecision() == globalWorldModel->simStep() ) ){
    INFO( Log::ROLE,  6,  name() , globalLog->infoStream << name() << ": Der Gamestate hat sich geaendert\n" );
    needNewIntentionList = YES;
    sitClassChange = YES;
    theGameState = globalWorldModel->gameState();
  }
  else {
    sitClassChange = NO;
  }

//if( (globalWorldModel->simStep() > 0) && (needNewIntentionList == YES) ) {
//  cout << "needNewIntentionList\n";
//}

  setNeckBehaviour();

  if( globalWorldModel->ball().posConfidence().reliability() < 0.6 ){ // HACK! muss mit dem HACK im Neck konform sein!
    // wenn der Ball zu lange nicht mehr gesehen wurde, mu Scanfield ihn suchen
    if( lastIntentionId() != ScanFieldIntention::theId ){
      INFO( Log::ROLE,  6,  name() , globalLog->infoStream << name() << ": ScanfieldIntention ausloesen\n" );
      eraseAllIntentions();
      addIntention( new ScanFieldIntention() );
      current = theIntentions.begin();
    }
    needNewIntentionList = NO;
  }

  /** wenn die alte Intentionliste abgearbeitet ist, oder Grnde fr eine Neuberechnung sprechen
      wird eine neue Intentionliste angelegt */
  if( needNewIntentionList == YES ){
    INFO( Log::ROLE,  6,  name() , globalLog->infoStream << name() << ": Neue Intentionliste generieren\n" );
    // gamestate = before_kick_off oder kick_off => Anstoss
	if( (globalWorldModel->gameState() & (WorldModelInterface::KICK_OFF | WorldModelInterface::BEFORE_KICK_OFF | WorldModelInterface::GOAL) ) != 0 ){
      INFO( Log::ROLE,  6,  name() , globalLog->infoStream << name() << ": Situation vor dem Anstoss\n" );
      eraseAllIntentions();
      Vector beamPosition = (globalWorldModel->initiative() == OUR_TEAM_ID) ? beamPositionOurKickOff() : beamPositionEnemyKickOff();
      addIntention( new MoveIntention( beamPosition ) );
      addIntention( new TurnIntention( Vector( 0, 0 ) ) );
      addIntention( new WaitIntention( new GameStateCondition( WorldModelInterface::PLAY_ON ) ) );
    }
    else if( ( ( theGameState & ( WorldModelInterface::CATCH_BALL | WorldModelInterface::CATCH_BALL_AND_FREE_KICK ) ) != 0 )
	     && ( globalWorldModel->initiative() == OUR_TEAM_ID ) )
	{
      eraseAllIntentions();
//      cout << "StandardGoalieRole: bin in catchBall" << endl;
	// und jetzt zum Abstoss schreiten...   
	  addIntention( new GoalieGoalKickIntention() );
      // addIntention( new PassIntention() );
      //addIntention( new KickIntention( Target(Target::GOALIE_EMERGENCY_KICK) ) );
    }
    else if( ( ( theGameState & WorldModelInterface::GOAL_KICK ) != 0 )
	      && ( globalWorldModel->initiative() == OUR_TEAM_ID ) ){
	  eraseAllIntentions();
      addIntention( new GetBallIntention( globalWorldModel->player().staminaIncMax() ) );
      addIntention( new PassIntention() );
      addIntention( new KickIntention( Target(Target::GOALIE_EMERGENCY_KICK) ) );
    }
    else if( ( theGameState & ( WorldModelInterface::CORNER_KICK | WorldModelInterface::KICK_IN ) ) != 0 ) {
      eraseAllIntentions();
      addIntention( new GoaliePositionIntention() );
    }
    else{
      // normale Spielsituation
      eraseAllIntentions();
      addIntention( new GoalieInterceptIntention( NORMAL ) );
      addIntention( new GoalieCatchIntention() );
      addIntention( new PassIntention() );
      addIntention( new KickIntention( Target(Target::GOALIE_EMERGENCY_KICK) ) );
      addIntention( new GoaliePositionIntention() );
    }

    // Iterator auf aktuell auszufhrende Intention an den Anfang setzen
    current = theIntentions.begin();
    needNewIntentionList = NO;
//    sitClassChange = YES; 
    // HACK
    setLastIntentionId( DummyIntention::theId /*(*current)->id()*/ );
  }
  INFO( Log::ROLE,  6,  name() , globalLog->infoStream	<< "LEAVE " << name() <<"::deliberate()\n" );
}// end of deliberate()
