
/*
 * Aktueller Stand
 *
 *   $RCSfile: Skills.cc,v $
 *   $Revision: 1.1.1.1 $
 *   $Date: 2001/07/31 22:23:09 $
 *   $Author: rami $ 
 *
 */
#include <iomanip> 
#include "Skills.h"

#include "../../basic/Configuration.h"
#include "../../basic/Common.h"
#include "../../basic/Action.h"

#include "../../analyzer/Analyzer.h"

#include "../../worldmodel/MovingObject.h"


Skills *globalSkills; 

Skills::Skills() {
  resetSituation(); 
  theIdString = "Skills";

  // cerr << globalTimer->getAbsoluteTime() << "\n";
  createBallTables();
  // cerr << globalTimer->getAbsoluteTime() << "\n" ;
  //printBallTables();
  createNewBallDistanceTable();
  //printNewBallDistanceTable();

}




/********************/
/*****   KICK   *****/
/********************/

Action Skills::kickOnce( const Vector& aTarget ) {
  Vector toTarget = aTarget - ball().pos() - ball().speed();
  // cerr << "Target: " << aTarget << endl;
  double power = 100;
  if( player().id() == UNDEF_INT ){
    return Action( EMPTY_ACTION, situation().simStep() );
  }
  double angle = normalizeAngle( toTarget.angle() - player().bodyDir() );
  return Action( KICK_ACTION, situation().simStep(), power, angle );
}



Action Skills::kick(const Vector& aTarget, double desiredTargetSpeed, int maxSteps)
{
  const ModelledPlayer& mySelf = player();
  const ModelledBall& theBall = ball();
  Vector targetVector = aTarget - theBall.pos();
  Vector relBallPos = theBall.pos() - mySelf.pos();

  desiredTargetSpeed = max( desiredTargetSpeed, 0.01 );
  
  double dist = targetVector.length();
 		
  double initialBallSpeed = desiredTargetSpeed;
  while( ( dist > 0.0 ) && ( initialBallSpeed < globalConf->ballSpeedMax ) ) {
  	initialBallSpeed /= globalConf->ballDecay;
  	dist -= initialBallSpeed;
  }
 // cout << "Skills:kick: initial " << initialBallSpeed <<endl;
  initialBallSpeed = min( initialBallSpeed, globalConf->ballSpeedMax );
  Vector initialBallAcceleration = Vector( initialBallSpeed, targetVector.angle(), POLAR, RADIAN );
  Vector kickVector = initialBallAcceleration - theBall.speed();
  // ServerManual 7.alpha  S.40
  double kickEffort = 1
 					  - 0.25 * fabs( deviationBetween( mySelf.bodyDir(), theBall.pos().angle() ) ) / Ath98_PI
  					  - 0.25 * (relBallPos.length() - mySelf.size() - globalConf->ballSize) / mySelf.kickableMargin() ;
  double neededKickPower = kickVector.length() / ( kickEffort * globalConf->kickPowerRate ) ;
  neededKickPower = min( neededKickPower, globalConf->maxPower );
  bool kickOnceIsPossible = (  neededKickPower * 0.9 <= globalConf->maxPower); // safety

  ModelledPlayer simPlayer = mySelf;
  ModelledBall simBall = ball();
  simBall.accelerate(neededKickPower, kickVector.angle() - mySelf.bodyDir(), mySelf);
  simBall.simulateSteps(1);
  simPlayer.simulateSteps(1);
  bool collision = ( (simBall.pos() - simPlayer.pos()).length() < mySelf.size() + theBall.size() );
  if( kickOnceIsPossible && collision == NO ) {
    return Action( KICK_ACTION, situation().simStep() , neededKickPower , kickVector.angle() - mySelf.bodyDir() );
  } 
// ab hier mindestens 2 kicks noetig --------------------------------------------------------------
  double newTargetDir = ( aTarget - simPlayer.pos() ).angle();
  Vector newTarget = simPlayer.pos() + Vector( mySelf.size() + 0.10, newTargetDir, POLAR, RADIAN );
  targetVector = newTarget - theBall.pos();
  
  initialBallSpeed = targetVector.length();
  initialBallSpeed = min( initialBallSpeed, globalConf->ballSpeedMax );
  initialBallAcceleration = Vector( initialBallSpeed, targetVector.angle(), POLAR, RADIAN );
  kickVector = initialBallAcceleration - theBall.speed();
  neededKickPower = kickVector.length() / ( kickEffort * globalConf->kickPowerRate ) ;
  neededKickPower = min( neededKickPower, globalConf->maxPower );
  kickOnceIsPossible = (  neededKickPower * 0.9 <= globalConf->maxPower ); // safety

  simBall = theBall;
  simBall.accelerate(neededKickPower, kickVector.angle() - mySelf.bodyDir(), mySelf);
  simBall.simulateSteps(1);
  collision = ( (simBall.pos() - simPlayer.pos()).length() < mySelf.size() + theBall.size() );

  if( kickOnceIsPossible && collision == NO ) {
    return Action( KICK_ACTION, situation().simStep() , neededKickPower , kickVector.angle() - mySelf.bodyDir() );
  }
  return Action( KICK_ACTION, situation().simStep() , neededKickPower , kickVector.angle() - mySelf.bodyDir() );
}// end of Skills::kick()

  


Action Skills::dribble( const Vector aTarget )  {
//cout << "<SKILLS::DRIBBLE>\n";
//cout << "  Target = " << aTarget << endl;

  const ModelledBall& dBall = ball();
  const ModelledPlayer& mySelf = player();
  Action theAction;
//cout << "  distToBall = " << mySelf.distTo(dBall.pos()) << endl;
  
  // zum Ball laufen
  if( mySelf.distTo(dBall.pos()) > mySelf.kickRange() - 0.20 ) {
    vector<Action> plan = (gotoRegion( globalAnalyzer->getMyInterceptPosition(), /*dBall.pos(),*/
                        mySelf.kickRange() - 0.20,
                        globalConf->dribbleStaminaLimit) );
    if( plan.size() > 0 ) {
      theAction = plan[0];
    }
    else {
      theAction = Action( EMPTY_ACTION, globalWorldModel->simStep() );
    }
  }
  else { // kicken
    double staminaLeft = mySelf.stamina() - globalConf->dribbleStaminaLimit;
    double staminaFactor = 1.0 + (100 / staminaLeft);
    staminaFactor = max( staminaFactor, 1 );
    staminaFactor = min( staminaFactor, 3 );
    Vector targetVector = aTarget - dBall.pos();
    double kickLength = 4.0 * log( 1 + targetVector.length() ) / staminaFactor;
    double kickDir = targetVector.angle();
  
  	const ModelledPlayer& nearestEnemy = globalWorldModel->situation().nearestPlayer( THEIR_TEAM_ID );
    Vector enemyVector = nearestEnemy.pos() - dBall.pos();
    double enemyDist = enemyVector.length();

    double dev = deviationBetween( targetVector.angle(), enemyVector.angle() );

    if( (nearestEnemy.id() != UNDEF_ID) && (nearestEnemy.id() != UNDEF_PLAYER_ID) && ( fabs(dev) < radFromDegree( 30 ) )
        && ( (enemyDist < 1.0 * kickLength) || (nearestEnemy.distTo(aTarget) < kickLength) )  ) {
      kickLength = min(kickLength, 1.0 * enemyDist);
      kickLength = max(kickLength / staminaFactor, 4.0);
      if( (fabs( dev ) < radFromDegree(90)) && (enemyVector.length() < 10.0) ) {
        double dirKorrFactor = 1 + enemyDist * enemyDist * 0.1;
        dirKorrFactor = min(dirKorrFactor, 8.0);
        kickDir += (dev > 0.0 ? -1.0 : 1.0 ) * ( radFromDegree(90) - fabs(dev) ) / dirKorrFactor;
      }
    }
    Vector kickVector = mySelf.pos() + Vector( kickLength, kickDir, POLAR, RADIAN );
    theAction = kick( kickVector, 0.25, 2 );
  }
	return theAction;
}  // end dribble





/********************/
/*****   TURN   *****/
/********************/

Action Skills::simpleTurn( const double angle )
{
  INFO( Log::SKILLS ,  10 ,  idString(), globalLog->infoStream << "Skills::simpleTurn\n"  );
  ModelledPlayer player( player() );
  player.turn( angle );
  situation().setPlayer( player );
  situation().simulateSteps();
  return Action( TURN_ACTION, situation().simStep() - 1, angle );
}


vector<Action> Skills::turnBy( double turnAngle )
{
  INFO( Log::SKILLS ,  6 ,  idString(), globalLog->infoStream << " ENTER Skills::turnBy\n"  ) ;

  vector<Action> plan;

  INFO( Log::SKILLS ,  8 ,  idString(), globalLog->infoStream << " turnAngle vor normalisierung: " 
    << degFromRadiant( turnAngle  ) << endl ) ;

  turnAngle = player().normalizeTurnAngle( turnAngle );

  INFO( Log::SKILLS ,  10 ,  idString(), globalLog->infoStream
    << " turnAngle nach normalisierung: " << degFromRadiant( turnAngle ) << endl ) ;

  double maxAngle = player().maxEffectiveTurnAngle();

  INFO( Log::SKILLS ,  10 ,  idString(), globalLog->infoStream
    << " maxAngle: " << degFromRadiant( maxAngle  )  << endl  ) ;

  while( Ath98_PI - fabs(Ath98_PI - turnAngle) > maxAngle ) {
    double simpleTurnAngle = (turnAngle > Ath98_PI) ? 2.0 * Ath98_PI - maxAngle : maxAngle;
    INFO( Log::SKILLS ,  10 ,  idString(), globalLog->infoStream
      << " toTurn: " << degFromRadiant( turnAngle  )  << endl
      << " maxAngle: " << degFromRadiant( maxAngle  )  <<  endl
      << " simpleTurnAngle: " << degFromRadiant( simpleTurnAngle  ) << endl ) ;

    plan.push_back( simpleTurn( player().turnAngleForSend( simpleTurnAngle ) ) );
    turnAngle = normalizeAngle( turnAngle - simpleTurnAngle );
    maxAngle = player().maxEffectiveTurnAngle();
  }
  
  INFO( Log::SKILLS ,  10 ,  idString(), globalLog->infoStream 
    << " toTurn: " << degFromRadiant( turnAngle ) << endl ) ;

  if( Ath98_PI - fabs(Ath98_PI - turnAngle) >= radFromDegree( MIN_TURN_ANGLE_IN_DEGREE ) ) {
    plan.push_back(simpleTurn( player().turnAngleForSend( turnAngle ) ));
  }

  INFO( Log::SKILLS ,  6 ,  idString(), globalLog->infoStream 
    << " LEAVE Skills::turnBy\n"  ) ;

  return plan;
}


vector<Action> Skills::turnTo( const Vector target )
{
  return turnBy( globalWorldModel->player().angleOfBodyTo( target ) ) ;
}



/********************/
/*****   DASH   *****/
/********************/

Action Skills::simpleDash( const double power )
{
  ModelledPlayer player( player() );
  player.accelerate( power );
  situation().setPlayer( player );
  situation().simulateSteps();
  return Action( DASH_ACTION, situation().simStep() - 1, power );
}


vector<Action> Skills::gotoRegion(
    const Vector& centerOfTarget,
    double targetRadius,
    const double staminaLimit,
    const int specialBehaviors,
    double desiredMaxPower,
    int maxDesiredSteps)
{
  INFO( Log::SKILLS , 8 , idString(), globalLog->infoStream << " at simStep " << globalWorldModel->simStep() << endl ) ;
  Situation s = Situation( globalWorldModel->simStep() );
  s.setPlayer( player() );
  s.setBall( ball() );
  resetSituation( s );

//cout << "------ GOTOREGION() ------\n";
//cout << "      player = " << player() << endl;

  bool runBackwards = ( specialBehaviors & RUN_BACKWARDS ) ? YES : NO;

  INFO( Log::SKILLS ,  8 ,  idString(), globalLog->infoStream
    << " centerOfTarget: " <<    centerOfTarget  
    << " \ntargetRadius: " <<     targetRadius   
    << " \nstaminaLimit: " <<     staminaLimit   
    << " \nrunBackwards: " <<    runBackwards <<  endl ) ;

  vector<Action> plan;

  // if( targetRadius < TRIANGULATION_ACCURACY ) {
  //   targetRadius = TRIANGULATION_ACCURACY;
  // }

  Vector intermediateTarget = centerOfTarget;
  double deviationFromTarget;
  double staminaLeft;

  while( player().distTo( centerOfTarget ) > targetRadius ) {
    INFO( Log::SKILLS ,  8 ,  idString(), globalLog->infoStream << " Berechne Aktion " << plan.size()+1 << endl ) ;

    staminaLeft = player().stamina() - staminaLimit;
//cout << "      staminaLeft = " << staminaLeft << endl;

    INFO( Log::SKILLS, 9, idString(), globalLog->infoStream
      << "player stamina: " << player().stamina() << " staminaLimit: "
      << staminaLimit << " staminaLeft: " << staminaLeft << endl );

    deviationFromTarget = player().angleOfBodyTo( intermediateTarget );
    
    if ( runBackwards == YES ) {
      deviationFromTarget = normalizeAngle( deviationFromTarget - Ath98_PI );
    }
    
    INFO( Log::SKILLS , 9 , idString(), globalLog->infoStream << " deviationFromTarget: " << degFromRadiant( deviationFromTarget ) << endl ) ;

    if( Ath98_PI - fabs( Ath98_PI - deviationFromTarget ) > radFromDegree( 4.0*log(5.0+player().distTo(centerOfTarget)) ) || staminaLeft <= 0 ) {
      // drehen
      INFO( Log::SKILLS , 8 , idString(), globalLog->infoStream << " Es wird gedreht weil "  ) ;
      if ( staminaLeft <= 0.0 ) {
        INFO( Log::SKILLS , 8 , idString(), globalLog->infoStream << " Stamina zu niedrig ist \n" ) ;
//cerr << " turn, weil stamina\n";
      }
      else {
       INFO( Log::SKILLS , 8 , idString(), globalLog->infoStream << " Richtungsabweichung zu hoch ist\n" ) ;
//cerr << " turn, weil deviation\n";
      }
      vector<Action> turnPlan = turnBy( deviationFromTarget );
//cerr << "GOTO::deviation = " << deviationFromTarget << endl;
//cerr << "GOTO::turnplan = ";
//for( int m=0; m < (int) turnPlan.size(); ++m) cerr << turnPlan[m];
//cerr << endl;

      if ( turnPlan.size() == 0 ) {
        // turn nicht notwendig, weil Richtung bereits genau stimmt
        plan.push_back( simpleEmptyAction() );
      }
      else {
        plan.insert( plan.end(), turnPlan.begin(), turnPlan.end() );
      }
    }
    else { // beschleunigen
      INFO( Log::SKILLS , 8 , idString(), globalLog->infoStream << " Es muss gedasht werden\n") ;
      double distToTarget = player().distTo( centerOfTarget );
      double neededAcceleration = distToTarget - player().speed().length();
      double power;
//cout << "      distToTarget = " << distToTarget << endl;

      if( neededAcceleration < player().maxEffectiveDashPower() * player().dashPowerRate() )
      {
        // wir knnen mit einem Dash das Ziel erreichen
        double neededPower = neededAcceleration / player().dashPowerRate();
        power = neededPower / player().effort() ;
        if( power < 0.0 ) {
          power = max( power, player().minPower() );
          //power = max( power, player().stamina() / -2.0 );
          //staminaLeft sollte nur verwendet werden, wenn in den elementaren Skills die
          //stamina-abhngigen Werte richtig simuliert werden
          power = max( power, staminaLeft / -2.0 );
          power = max( power, -1.0 *  desiredMaxPower );
        }
        else {
          power = min( power, player().maxPower() );
          //power = min( power, player().stamina() );
          power = min( power, staminaLeft );
          power = min( power, desiredMaxPower );
        }
      }
      else{
        // wir dashen voll
        if( runBackwards == YES ) {
          power = max( -1.0 * desiredMaxPower, player().minPower() );
          power = max( power, staminaLeft / -2.0 );
        }
        else {
          power = min( desiredMaxPower, player().maxPower() );
          power = min( power, staminaLeft );
        }
      }

      INFO( Log::SKILLS, 9, idString(), globalLog->infoStream
        << "Computed power for dash: " << power << endl );

      plan.push_back( simpleDash( power ) );
    }

    if( (signed) plan.size() >= maxDesiredSteps ) {
      // es ist nicht sinnvoll noch grssere Plne zu verfolgen
//      cerr << "Skills::gotoRegion plan.size >= 50\n";
      break;
    }
  }// end of while
  INFO( Log::SKILLS ,  3 ,  idString(), globalLog->infoStream << " LEAVE Skills::gotoRegion\n"  ) ;
//cout << "------ GOTOREGION() ------\n";
  return plan;
}



/*********************/
/*****   CATCH   *****/
/*********************/

Action Skills::simpleCatch( const double angle )
{
  // Mehr kann bisher im Weltmodell nicht registriert werden
  situation().simulateSteps();
  return Action( CATCH_ACTION, situation().simStep() - 1, angle );
}



/*******************/
/*****   SAY   *****/
/*******************/

Action Skills::simpleSay( const Ath98String& message )
{
  return Action( SAY_ACTION, situation().simStep(), message );
}

Action Skills::saySituationAction( const Situation& aSituation )
{
  INFO( Log::SKILLS ,  4 ,  idString(), globalLog->infoStream <<
  " ENTER Skills::sayKickSituationAction\n"  ) ;
  return simpleSay( globalWorldModel->saySituation( aSituation ) );
}



/**********************/
/*****   COMMON   *****/
/**********************/

void Skills::resetSituation( Situation scenario )
{
  if( globalWorldModel->gameState() == WorldModelInterface::FREEZE ) {
    // cout << "resetSituation: simstep" <<globalWorldModel->simStep() << "\n";
  }
  theSituation = scenario;
}

Action Skills::simpleEmptyAction( )
{
  situation().simulateSteps();
  return Action( EMPTY_ACTION, situation().simStep() - 1 );
}



void Skills::createBallTables() {

  int bwa = 0;
  int bva = 0;
  double ballway =  0.0;
  double ballvelocity =  0.0;

 for ( bwa=0; bwa<theBallTableMaxIndex; bwa++) {
   theBallDistanceTable[bwa] = 0;
 }

 for ( bva=0; bva<theBallTableMaxIndex; bva++) {
   theBallVelocityTable[bva] = 0;
 }

 ballvelocity = 2.7;

 theBallDistanceTable[0] = ballway;
 theBallVelocityTable[0] = ballvelocity;
 bwa=1;
 bva=1;
//  cout << "*** forward simulation started ***\n 000 " <<
//    " ballway : " << ballway <<
//    " ballvelocity : " << ballvelocity << endl;
 // Vorwrtssimulation
 for (int i=1; i<theBallTableMaxIndex; i++) {
   ballvelocity = ballvelocity *  theBallVelocityFactor;
   ballway = ballway + (ballvelocity * 1.00);      // 1.06
   theBallDistanceTable[bwa] = ballway;
   theBallVelocityTable[bva] = ballvelocity;
   bwa++;
   bva++;
//    cout << i <<
//      " ballway : " << ballway <<
//      " ballvelocity : " << ballvelocity << endl;
 }


 for( int d=0; d < theStartVelocityTableMaxDistance; ++d ) {
   for( int s=0; s < theStartVelocityTableMaxSteps; ++s ) {
     theBallStartVelocityTable[d][s] = 0;
   }
 }


 int bwabegin = 0;
 int bwaend = theBallTableMaxIndex;
 int wstart = 0;
 int wend = 0;
 int wsize = 0;
 double distsumme = 0;
 int dist=0;
 int lastdist=999;
 double prozabweichung;
 // anzsteps == wieviele Schritte sind Zeit,
 // um die Entfernung zu berbrcken
 for(int anzsteps=1; anzsteps<=bwaend; anzsteps++) {
   // sliding window mit der Gre == anzsteps verschieben
   wsize = anzsteps;
   // cout << " **** \n anzahl_takte : " << wsize << " \n";
   wstart = bwabegin;
   wend = wstart + wsize;
   distsumme = 0;
   prozabweichung = 0.1;

   for (int offset=0; offset<bwaend-wsize; offset++) {
     // cout << "** move sliding window ***\n";
     // cout << " o:" << offset;
     distsumme = 0;
     for (int w=wstart+offset; w < wstart+offset+wsize; w++) {
       // cout << " w=" << w;
       distsumme = distsumme + theBallDistanceTable[w+1]
	 - theBallDistanceTable[w];
     }
     // cout << " ballentfernung : " << distsumme <<
     //  " startgeschw. : " << theBallVelocityTable[offset] << "\n";

     // Eintrag in ballStartVelocityTable
     dist = (int)rint(distsumme);
     // gerundeter Wert liefert index
     // auf dieselbe dist nur eintragen, wenn prozabweichung noch kleiner
     // ist als die vom vorigen Eintrag
     if (dist>=0 && dist < theStartVelocityTableMaxDistance) {
       if (dist == lastdist ) {
	 if ( (fabs(distsumme-dist) / dist) < prozabweichung) {
	   //  prozabweichung ist kleiner
	   prozabweichung = fabs(distsumme-dist)/dist;
	   lastdist = dist;
	   theBallStartVelocityTable[dist][wsize] =
	     theBallVelocityTable[offset];
// 	   cerr << " t=" << wsize << " | Entf. :"
// 		<< distsumme << " zugeordnet zu: " << dist
// 		<< "  velocity : "<<  theBallVelocityTable[offset] << endl;
	 } // else  { cerr << "skipped";}
	
       } else
	 {
	   prozabweichung = fabs(distsumme-dist)/dist;
	   lastdist = dist;
	   theBallStartVelocityTable[dist][wsize] =
	     theBallVelocityTable[offset];
// 	   cerr << " t=" << wsize << " | Entf. :"
// 		<< distsumme << " zugeordnet zu: " << dist
// 		<< "  velocity : "<<  theBallVelocityTable[offset] << endl;
	 }
     } // dist ist gltiger index
   } // offset
 } // anzsteps

 // Lcken suchen und fllen
 for (int d=1; d<theStartVelocityTableMaxDistance; d++) {
   for (int s=1; s<theStartVelocityTableMaxSteps; s++) {
     if (theBallStartVelocityTable[d][s] == 0) {
       // sind modifizierte indizes gltig ?
       if ( (d>2) && (d<theStartVelocityTableMaxDistance-1) &&
	    (s>1) && (s<theStartVelocityTableMaxSteps) ) {
	 // Durchschnitt aus den beiden benachbarten Entfernungen
	 // mit derselben Taktzahl bilden und eintragen
	 // sofern diese nicht auch null sind == Variante A
	
	 // Variante A
	 if ( (theBallStartVelocityTable[d-1][s] != 0)  &&
	      (theBallStartVelocityTable[d+1][s] != 0) ) {
	   theBallStartVelocityTable[d][s] =
	     (theBallStartVelocityTable[d-1][s] +
	      theBallStartVelocityTable[d+1][s]) / 2.0;
// 	   cout << "filling_b"
// 		<< " d:" << d
// 		<< " s:" << s
// 		<< " with :" << theBallStartVelocityTable[d][s]
// 		<< endl;
	 }
	 else {
	   // Variante B
	 if ( (theBallStartVelocityTable[d-1][s] != 0)  &&
	      (theBallStartVelocityTable[d+1][s] == 0) &&
	      (s>0) &&
	      (theBallStartVelocityTable[d][s-1] != 0) ) {
	   theBallStartVelocityTable[d][s] =
	     (theBallStartVelocityTable[d-1][s] +
	      theBallStartVelocityTable[d][s-1]) / 2.0;
// 	   cout << "filling_a"
// 		<< " d:" << d
// 		<< " s:" << s
// 		<< " with :" << theBallStartVelocityTable[d][s]
// 		<< endl;
	 }
	 } // else
       }
     }
   }
 }

}

void Skills::printBallTables() {
  cout << " theBallDistanceTable & theBallVelocityTable \n";
  for (int i=0; i<theBallTableMaxIndex; i++) {
    cout << " step:" <<  setw(3) << i
	 <<  " theBallDistance: " << setw(5) << setprecision(4)
	 <<  theBallDistanceTable[i]
	 <<  " theBallVelocity: " << setw(5) << setprecision(4)
	 <<  theBallVelocityTable[i] << endl;
  }
  cout << " theBallStartVelocityTable \n";
  cout << "waagerecht : Takte, senkrecht : Entfernungen" << endl;
  cout << "Verzoegerung : " << theBallVelocityFactor << endl;

  for (int d=0; d<theStartVelocityTableMaxDistance; d++) {
    cout  << endl << endl << "Entfernung : " << d
	  <<  " Meter " << endl;
    for (int s=1; s<theStartVelocityTableMaxSteps; s++) {
      if (s==11 || s==21 || s==31 || s==41 || s==51 ) {
	cout << endl << s << " Takte ";
      }
      if (s==1 ) {
	cout << endl << s << "  Takt  ";
      }
      cout << setw(5) << setprecision(3)
	   << theBallStartVelocityTable[d][s] << " ";
    }
  }
}
