#include "Target.h"

#include "../../basic/StraightLine.h"
#include "../../basic/Timer.h"
#include "../skills/Skills.h"
#include "../../worldmodel/Situation.h"

#include "../../basic/Formation.h"
#include "../../basic/Formation2.h"


Target::Target() :
  thePos( Vector() ), theId( Target::NOTHING ),
  theTime( UNDEF_INT ), theTimeOfLastCalc( UNDEF_INT ) {}

Target::Target( const Vector& aPos, int aTime ) :
  thePos( aPos ), theId( Target::NOTHING ),
  theTime( aTime ), theTimeOfLastCalc( UNDEF_INT ) {} 


Target::Target( TargetId aId ) :
  theId( aId ), theTimeOfLastCalc( UNDEF_INT ) {}

Target::Target( TargetId aId, int aPlayerId, int aTime ) :
  theId( aId ), thePlayerId( aPlayerId ), theTime( aTime ) {}

  
Vector Target::pos() const{
  if( theTimeOfLastCalc == globalWorldModel->simStep() ){
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	<< "Target::pos: lastcalc in act. Simstep->skip\n" );
    return thePos;
  }
  INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	<< "Target::pos with Flag: " );
  theTimeOfLastCalc = globalWorldModel->simStep();
  switch( theId ){

  case NOTHING:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	<< "NOTHING\n" );
    return thePos;

  case BALL:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	  << "BALL\n" );
    thePos = globalWorldModel->situation().ball().pos();
    return thePos;

  case PLAYER:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	  << "PLAYER\n" );
    if( globalWorldModel->situation().playerWithId( thePlayerId ).id() == UNDEF_ID ){
      cerr << "Target::pos()     player.id() ist UNDEF !!!\n";
      WARNING( Log::TARGET, "player.id() ist UNDEF !!!\n" );
	  thePos = globalWorldModel->player().pos();
      return thePos;
    }
	thePos = globalWorldModel->situation().playerWithId( thePlayerId ).pos();
    return thePos;

  case GOALIE_EMERGENCY_KICK:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream << "GOALIE_EMERGENCY_KICK\n" );
    return Vector ( globalWorldModel->player().pos().x() ,
		                (globalWorldModel->player().pos().y() > 0.0 ? 0.5 : -0.5) * globalWorldModel->field().fieldWidth() );

  case GOAL_KICK_AFTER_CATCH_PASS_TARGET:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	<< "GOAL_KICK_AFTER_CATCH_PASS_TARGET\n" );
	// wenn UNDEF_TEAMMATE_ID ist der angepasste Spieler wohl weit weg... 
	// zumindest nehmen wir das hier an - deshalb setzen wir den Abstand zwischen
	// Ball und Spieler auf 25m
	if( (globalAnalyzer->getGoalKickAfterCatchPassReceiverId()) == UNDEF_TEAMMATE_ID ) {
		thePos = globalAnalyzer->getGoalieMoveTarget() +  Vector( 25 ,
				( globalAnalyzer->getGoalKickAfterCatchPassTarget() - globalAnalyzer->getGoalieMoveTarget() ).angle(), POLAR, RADIAN );
	}
	else {	
		thePos = globalAnalyzer->getGoalieMoveTarget() +  
				Vector( globalWorldModel->situation().playerWithId( globalAnalyzer->getGoalKickAfterCatchPassReceiverId()).distTo( globalAnalyzer->getGoalieMoveTarget() ),
				( globalAnalyzer->getGoalKickAfterCatchPassTarget() - globalAnalyzer->getGoalieMoveTarget() ).angle(), POLAR, RADIAN );				
	}
    return thePos;

  case INTERCEPT:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream	<< "INTERCEPT\n" );
    thePos = globalAnalyzer->getMyInterceptPosition();
    return thePos;

  case KICK_FREE_KICK:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	  << "KICK_FREE_KICK\n" );
    thePos = calculatePositionToKickFree();
    return thePos;

  case GOALIE_STANDARD_POSITION:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	  << "GOALIE_STANDARD_POSITION\n" );
//    return calculateGoalieStandardPosition();
    thePos = globalAnalyzer->getGoaliePositionTarget();
    return thePos;

  case PASS:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "PASS\n" );
	
    thePos = globalAnalyzer->getPassTarget();
    return thePos;	
	// wenn UNDEF_TEAMMATE_ID ist der angepasste Spieler wohl weit weg...
	// zumindest nehmen wir das hier an - deshalb setzen wir den Abstand zwischen
	// Ball und Spieler auf 25m
	if( globalAnalyzer->getPassReceiverId() == UNDEF_TEAMMATE_ID ) {
		thePos = globalWorldModel->ball().pos() 
                         +  Vector( 25 , ( globalAnalyzer->getPassTarget() -  globalWorldModel->ball().pos() ).angle(),
                                   POLAR, RADIAN );				
	}
	else {	
      thePos = globalWorldModel->ball().pos() +  
				Vector( globalWorldModel->situation().playerWithId( globalAnalyzer->getPassReceiverId()).distTo( globalWorldModel->ball().pos() ),
				( globalAnalyzer->getPassTarget() - globalWorldModel->ball().pos()).angle(), POLAR, RADIAN );				
	}
    //cout << "  PASS_RECEIVER_ID : " << globalAnalyzer->getPassReceiverId() << endl;
    //cout << "  PASS_TARGET : " << thePos << endl;
    return thePos;

  case DP_PASS:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream << "DP_PASS\n" );
//    thePos = calculateDPPass();
//    zu Testzwecken wurde das Doppelpasstarget verbogen --> liefert jetzt Ppositionierung
    thePos = SBSPPosition( globalWorldModel->playerNumber() );
    return thePos;

  case SCORE_GOAL:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "SCORE_GOAL\n" );
    thePos = globalAnalyzer->getScoreGoalTarget();
    return thePos;

  case POSITION:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "POSITION\n" );
  	thePos = globalAnalyzer->getPositionTarget();
    return thePos;

  case COVER:
//    zu Testzwecken wurde das Covertarget verbogen --> liefert jetzt Ppositionierung2
    thePos = SBSPPosition2( globalWorldModel->playerNumber() );
    return thePos;

  case GOALIE_INTERCEPT_POSITION:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	    << "GOALIE_INTERCEPT_POSITION\n" );
	thePos = globalAnalyzer->getOurGoalieInterceptPosition();
    return thePos;

  case GOALIE_KICK_TO_THE_OTHER_SIDE:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "GOALIE_KICK_TO_THE_OTHER_SIDE\n" );
    thePos = Vector( 25.0, 34.0 );
    return thePos;

  case GOALIE_MOVE:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "GOALIE_MOVE\n" );
	thePos = globalAnalyzer->getGoalieMoveTarget();
    return thePos;

  case OUR_FREEKICK:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "OUR_FREEKICK\n" );
    calculateOurFreekick();
    return thePos;
    break;

  case DRIBBLE:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream << "DRIBBLE\n" );
    thePos = globalAnalyzer->getDribbleTarget();
    return thePos;
    break;    
    
  case CHAIN:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream << "CHAIN\n" );
    thePos = calculateChain();
    return thePos;
    break;    

  default:
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	<< "DEFAULT\n" );
    return thePos;
  }
  return thePos;
}


// der Ppositionierungsalgo
Vector Target::SBSPPosition( int playerNumber ) const
{
  Vector position( globalFormation->formationGetPosition( playerNumber ) );
  Vector attraction( globalFormation->formationGetAttraction( playerNumber ) );

  position.setX( position.x() + globalWorldModel->ball().pos().x() * attraction.x() );
  position.setY( position.y() + globalWorldModel->ball().pos().y() * attraction.y() );
  if (fabs(position.y()) > 24.0)
    position.setY( position.y() - 0.5*attraction.y()*globalWorldModel->ball().pos().y() );

  position = positionToOnsidePosition( position );
  if ( globalFormation->formationGetBehindBall( playerNumber ) &&
       globalWorldModel->ball().pos().x() < position.x() )
    position.setX( globalWorldModel->ball().pos().x() );
  if (position.x() > globalFormation->formationGetMaxX( playerNumber ))
    position.setX( globalFormation->formationGetMaxX( playerNumber ) );
  if (position.x() < globalFormation->formationGetMinX( playerNumber ))
    position.setX( globalFormation->formationGetMinX( playerNumber ) );

  if( position.x() > globalWorldModel->situation().offsideLineX() - 0.7 ) {
    position.setX( globalWorldModel->situation().offsideLineX() - 0.7 );
  }

  if( ( (globalWorldModel->gameState() & WorldModelInterface::GOAL_KICK) != 0)
      && (globalWorldModel->initiative() == OUR_TEAM_ID)
      && (position.x() < -35.0) ) {
    position.setX( -35.0 );
  }

  return position;
}


// der 2. Ppositionierungsalgo
Vector Target::SBSPPosition2( int playerNumber ) const
{
  Vector position( globalFormation2->formationGetPosition( playerNumber ) );
  Vector attraction( globalFormation2->formationGetAttraction( playerNumber ) );

  position.setX( position.x() + globalWorldModel->ball().pos().x() * attraction.x() );
  position.setY( position.y() + globalWorldModel->ball().pos().y() * attraction.y() );
  if (fabs(position.y()) > 24.0)
    position.setY( position.y() - 0.5*attraction.y()*globalWorldModel->ball().pos().y() );

  position = positionToOnsidePosition( position );
  if ( globalFormation2->formationGetBehindBall( playerNumber ) &&
       globalWorldModel->ball().pos().x() < position.x() )
    position.setX( globalWorldModel->ball().pos().x() );
  if (position.x() > globalFormation2->formationGetMaxX( playerNumber ))
    position.setX( globalFormation2->formationGetMaxX( playerNumber ) );
  if (position.x() < globalFormation2->formationGetMinX( playerNumber ))
    position.setX( globalFormation2->formationGetMinX( playerNumber ) );

  if( position.x() > globalWorldModel->situation().offsideLineX() - 0.7 ) {
    position.setX( globalWorldModel->situation().offsideLineX() - 0.7 );
  }

  if( ( (globalWorldModel->gameState() & WorldModelInterface::GOAL_KICK) != 0)
      && (globalWorldModel->initiative() == OUR_TEAM_ID)
      && (position.x() < -35.0) ) {
    position.setX( -35.0 );
  }

  return position;
}


Vector Target::calculatePositionToKickFree() const{
  
  const ModelledBall& ball = globalWorldModel->ball();
  Vector target =
    globalWorldModel->preferredKickDirection( ball.pos() );
  target.normalize();
  target = 0.6 * target + ball.pos();
  return target;
}
    

void Target::calculateOurFreekick() const{
  
  Vector normalTarget( 0,0 );//hack
  INFO( Log::TARGET  , 12,  "" , globalLog->infoStream <<
	"normalTarget: " << normalTarget << '\n' );
  if( ( normalTarget - globalWorldModel->ball().pos() ).length_sqr() < 150 ){
    Situation scenario( globalWorldModel->situation() );
    ModelledBall simBall = scenario.ball();
    ModelledPlayer self( scenario.playerWithId( globalWorldModel->id() ) );
    scenario.removePlayerWithId( globalWorldModel->id() );
    self.setPos( normalTarget );
    scenario.addPlayer( self );
    
    Vector vel( simBall.pos() - self.pos() );
    vel.normalize();
    
    simBall.setSpeed( 2.0 * vel );
    scenario.setBall( simBall );
    
    int ourSteps = scenario.indexOfNearestPlayer( OUR_TEAM_ID );
    
    if( ourSteps > scenario.stepsOfNearestPlayer( THEIR_TEAM_ID ) + 3 ){
      thePos = normalTarget - 3 * vel;
      INFO( Log::TARGET  ,  12,  "" , globalLog->infoStream <<
	    "newTarget: " << thePos << '\n' );
    }
  }
  thePos = normalTarget;  
}


Vector Target::calculateDPPass() const{
  INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream <<
	" ENTER Target::calculateDPPass\n" );
  //cerr << "##1\n";
  Situation scenario = globalWorldModel->situation();
  ModelledBall ball = scenario.ball();
  Vector target;
  int i = scenario.playerCount() - 1;
  //cerr << "##2\n";
  while( i >= 0 ){
    //cerr << "##2.02\n";
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "Test Player: " << scenario.playerWithIndex( i ).id()
	  << "\n" );
    if( scenario.playerWithIndex( i ).id() == globalWorldModel->id() ){
      --i;
      //       cerr << "##2.03\n";
      INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	    << "is self->continue\n" );
      continue;
    }
    if( scenario.playerWithIndex( i ).team() != OUR_TEAM_ID ){
      --i;
      //       cerr << "##2.03\n";
      INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	    << "is enemy->continue\n" );
      continue;
    }      
    //cerr << "##2.1\n";
//     if( scenario.playerWithIndex( i ).distTo( ball.pos() ) > 15 ){
//       --i;
//       //cerr << "##2.11\n";
//       continue;
//     }
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "find TargetPlayer id: "
	  << scenario.playerWithIndex( i ).id() << "\n" );
    //cerr << "##3\n";
    Vector prefDir( globalWorldModel->
		    preferredKickDirection( scenario.
					    playerWithIndex( i ).pos() ) );
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	  << "prefDir = " << prefDir << "\n" );
    StraightLine line( scenario.playerWithIndex( i ).pos(),
		      prefDir,
		      DIRECTION );
    //cerr << "##4\n";
    double distBallToLine = line.distTo( ball.pos() );
    double distBallToLine_SQR = distBallToLine*distBallToLine;
    prefDir.normalize();
    double distBallToPlayer_SQR =
      ( ball.pos() - scenario.playerWithIndex( i ).pos() ).length_sqr();
    double distPlayerToPerpendicular;
    //cerr << "##5\n";
    if( distBallToLine_SQR > distBallToPlayer_SQR ){
      ERROR( "negativ SQRT!" );
      distPlayerToPerpendicular = 0;
    }
    else{
      distPlayerToPerpendicular =
	sqrt( distBallToPlayer_SQR - distBallToLine_SQR );
    }
    int pToPerpSteps = (int)ceil( distPlayerToPerpendicular );
    int PerpToTargetSteps = (int)ceil(distBallToLine/1.7);
    int playerSteps = pToPerpSteps + PerpToTargetSteps;
    
    target = scenario.playerWithIndex( i ).pos() +
      ( playerSteps * prefDir );
    // Raender vorsichtiger, muss spaeter genauer/besserer Algo her
    if( target.x() > 51 ) target.setCartesian( 50, target.y() );
    if( target.x() < -51 ) target.setCartesian( -50, target.y() );
    if( target.y() > 31 ) target.setCartesian( target.x(), 30 );
    if( target.y() < -31 ) target.setCartesian( target.x(), -30 );
      
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream 
	  << "Line: " << line.toString()
	  << "\n");
    break;
    --i;
  }
  INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream <<
	" LEAVE Target::calculateDPPass with: " << target << "\n" );
  //cerr << "##7\n";
  return target;
}

Vector Target::calcScoreGoal() const{
  INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream <<
	"ENTER Target::calcScoreGoal\n" );
  // Vorbereitung
  Situation scenario( globalWorldModel->situation() );
  Vector targetAtLine( globalWorldModel->field().enemyGoal() );
  // Situation bereinigen: alle Spieler raus, die weiter als
  // 30m in x-Richtung vom Tormittelpunkt entfernt sind
  for( int i = scenario.playerCount()-1; i >= 0; --i ){
    // selbst bleibt drin
    if( scenario.playerWithIndex( i ).id() ==
	globalWorldModel->id() ) continue;
    // nahe bleiben drin: 30*30 = 900
    if( ( scenario.playerWithIndex( i ).pos() -
	  targetAtLine ).length_sqr() < 900 ) continue;
    // Gegnertorwart immer drin
    if( scenario.playerWithIndex( i ).isGoalie() &&
	scenario.playerWithIndex( i ).team() != OUR_TEAM_ID ) continue;
  }
  INFO( Log::TARGET  ,  6  ,  "" ,
	globalLog->infoStream << "Scenario bereinigt\n:"
	<< scenario << "\n" );
  ModelledBall ball( scenario.ball() );
  Vector ballPos( ball.pos() );

  // Simulation auf Gegnertor
  // Schrittweite: Torbreite/20 entspricht ca. 70cm
  double lineStep = globalWorldModel->field().goalWidth() / ( 2 * 10 );
  Vector plusVector( 0, lineStep );
  Vector minusVector( 0, -lineStep );
  int steps = 0;
  double bestAngle = UNDEF_DOUBLE;
  int bestStepDiff = 0;
  int bestSteps = 0;
  int ballSteps = 0;
  bool negFlag = false;
  INFO( Log::TARGET  ,  6  ,  "" ,
	globalLog->infoStream << "Starting Simulation\n" );
  while( steps < 10 ){
    INFO( Log::TARGET  ,  6  ,  "" ,
	  globalLog->infoStream << "steps: " << steps << "\n" );
    // Ball setzen in positive Richtung
    Vector ballGoal( targetAtLine +
		     ((double)steps * plusVector ) -  ballPos );
    double angle = ballGoal.angle();

    ball.setSpeed( Vector( 2.0, angle, POLAR ) );
    scenario.setBall( ball );
    INFO( Log::TARGET  ,  6  ,  "" ,
	  globalLog->infoStream << "positiv: ball set\n" ); 
    int theirSteps = scenario.stepsOfNearestPlayer( THEIR_TEAM_ID );
    try{
      if( scenario.nearestPlayer( THEIR_TEAM_ID ).isGoalie() ){
	// Goalie schneller am Ball (1 Step)
	--theirSteps;
      }
    }
    catch( Situation::NoPlayerWithThisIdException ){}
    double distBallGoal = ballGoal.length();
    ballSteps = estimatedBallSteps( distBallGoal );
    INFO( Log::TARGET  ,  6  ,  "" ,
	  globalLog->infoStream << "positiv: nearest is estimated: "
	  << "theirSteps: " << theirSteps
	  << " ballSteps: " << ballSteps
	  //<< " at:\n" << scenario
	  << "\n" ); 
    
    if( theirSteps - ballSteps > bestStepDiff ){
      INFO( Log::TARGET  ,  6  ,  "" ,
	    globalLog->infoStream << "positiv: good Kick\n" );
      bestStepDiff = theirSteps - ballSteps;
      bestAngle = angle;
      negFlag = false;
      bestSteps = steps;
    }
    if( steps != 0 ){
      // Ball setzen in negative Richtung
      Vector ballGoal( targetAtLine +
		       ((double)steps * minusVector ) -  ballPos );
      double angle = ballGoal.angle();

      ball.setSpeed( Vector( 2.0, angle, POLAR ) );
      scenario.setBall( ball );
      INFO( Log::TARGET  ,  6  ,  "" ,
	    globalLog->infoStream << "negativ: ball set\n" );     
      int theirSteps = scenario.stepsOfNearestPlayer( THEIR_TEAM_ID );
      try{
	if( scenario.nearestPlayer( THEIR_TEAM_ID ).isGoalie() ){
	  // Goalie schneller am Ball (1 Step)
	  --theirSteps;
	}
      }
      catch( Situation::NoPlayerWithThisIdException ){}
      INFO( Log::TARGET  ,  6  ,  "" ,
	    globalLog->infoStream << "negativ: nearest is estimated: "
	    << "theirSteps: " << theirSteps
	    << " ballSteps: " << ballSteps
	    //<< " at:\n" << scenario
	    << "\n" ); 
      double distBallGoal = ballGoal.length();
      ballSteps = estimatedBallSteps( distBallGoal );
    
      if( theirSteps - ballSteps > bestStepDiff ){
	INFO( Log::TARGET  ,  6  ,  "" ,
	    globalLog->infoStream << "negativ: good Kick\n" );
	bestStepDiff = theirSteps - ballSteps;
	bestAngle = angle;
	negFlag = true;
	bestSteps = steps;
      }
    }
    ++steps;
    if( bestStepDiff > 5 ) break;
  }
  if( negFlag ){
    thePos =  targetAtLine +
      ((double)bestSteps * minusVector );
  }
  else{
    thePos =  targetAtLine +
      ((double)bestSteps * plusVector );
  }
  INFO( Log::TARGET  ,  6  ,  "" ,
	globalLog->infoStream
	<< "LEAVE Target::calcScoreGoal with target: " << thePos
	<<"\n" );
  return thePos;
}

int Target::estimatedBallSteps( double dist ) const{int ballSteps = 0;
  if( dist > 20 ){
    ballSteps = (int)rint( dist * 0.75 );
  }
  else if( dist > 15 ){
    ballSteps = (int)rint( dist * 0.666 );
  }
  else if( dist > 10 ){
    ballSteps = (int)rint( dist * 0.6 );
  }
  else{
    ballSteps = (int)rint( dist * 0.5 );
  }
  return ballSteps;
}
  
Vector Target::calculateChain() const{
  Situation situation( globalWorldModel->situation() );
  situation.removeOwnGoalie();
  /* Der naechste Spieler zum Ball macht den Kick */
  situation.removePlayerWithIndex( situation.getIndexOfXNearestPlayer(1) );

  Vector ballPos( globalWorldModel->ball().pos() );
  Vector firstDir( globalWorldModel->preferredKickDirection( ballPos ) );
  firstDir.normalize();
  Vector firstPos( ballPos + ( 15.0 * firstDir ) );
  
  unsigned int index = situation.indexOfNearestPlayerToPos( firstPos, OUR_TEAM_ID );
  if( (index != UNDEF_INT) && (situation.index2Id( index ) ==
      globalWorldModel->id()) ){
    // Spieler sollte auf firstPos gehen
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream << "firstPos: " 
      << firstPos << '\n' );
    return firstPos;
  }
  if( index != UNDEF_INT ){
    situation.removePlayerWithIndex( index );
  }
   
  Vector secondDir( globalWorldModel->preferredKickDirection( firstPos ) );
  secondDir.normalize();
  Vector secondPos( firstPos + ( 17.5 * secondDir ) );
   
  index = situation.indexOfNearestPlayerToPos( secondPos, OUR_TEAM_ID );
  if( (index != UNDEF_INT) && (situation.index2Id( index ) ==
      globalWorldModel->id()) ){    
    // Spieler sollte auf secondPos gehen
    INFO( Log::TARGET  ,  6  ,  "" , globalLog->infoStream
	    << "secondPos: " << secondPos << '\n' );
    return secondPos;
  }
  return Vector();
}

Vector Target::calcCover() const{
  Situation scenario( globalWorldModel->situation() );
  int myIndex = scenario.id2Index( globalWorldModel->id() );
  // 1. Schritt: 1 Spieler geht vor den ball in prefkickdir des
  //             Gegners
  Vector mirroredBall( -scenario.ball().pos().x(),
		       -scenario.ball().pos().y() );
  Vector prefDir( mirroredBall );
  prefDir.normalize();
  Vector target( scenario.ball().pos() + 10.0 * prefDir );
  if( scenario.indexOfNearestPlayerToPos( target, OUR_TEAM_ID ) == myIndex ){
    // Spieler ist naechster zum Prefdir-Punkt
    return target;
  }
  // 2. Schritt: Spieler in der Naehe zum Ball (30m) zw. Ball und naechstem Gegner
  for( int i = scenario.playerCount() - 1; i >= 0 ; --i ){
    const ModelledPlayer& aPlayer = scenario.playerWithIndex( i );
    if( aPlayer.team() != OUR_TEAM_ID ){
      // Gegnerspieler gefunden -> ist nah?
      if( aPlayer.distTo( scenario.ball().pos() ) < 30 &&
	  aPlayer.pos().x() < scenario.ball().pos().x() + 10.0 ){
	// wenn Gegner naeher als 30m am Ball und
	// vor bzw. seitlich vom Ball (10.0m dahinter)
	int index = scenario.indexOfNearestPlayerToPos( aPlayer.pos(),
							OUR_TEAM_ID );
	if( index == myIndex ){
	  // ist es der Spieler selbst, dann deckt er den Gegenspieler
	  return Vector( aPlayer.pos() - 2.0 * prefDir );
	}
	else{
	  // den naechsten eigenen Spieler rausnehmen, damit er nicht
	  // mehr beruecksichtigt wird
	  scenario.removePlayerWithIndex( index );
	  // Gegner rausnehmen, da er schon gedeckt wird
	  scenario.removePlayerWithIndex( i );
	}
      }
    }
  }
  // 3. Schritt: naechsten gegnerischen Spieler locker (Distanz zw. 5 und 10 m )
  // decken
  for( int i = scenario.playerCount() - 1; i >= 0 ; --i ){
    const ModelledPlayer& aPlayer = scenario.playerWithIndex( i );
    if( aPlayer.team() != OUR_TEAM_ID ){
      // Gegnerspieler gefunden -> ist es der naechste Gegner?
      int index = scenario.indexOfNearestPlayerToPos( aPlayer.pos(),
						      OUR_TEAM_ID );
      if( index == myIndex ){
	// ist es der Spieler selbst, dann deckt er den Gegenspieler locker
	return Vector( aPlayer.pos() - 7.0 * prefDir );
      }
      else{
	// den naechsten eigenen Spieler rausnehmen, damit er nicht
	// mehr beruecksichtigt wird
	scenario.removePlayerWithIndex( index );
	// Gegner rausnehmen, da er schon gedeckt wird
	scenario.removePlayerWithIndex( i );
      }
    }
  }
  // 4. Schritt: Normale Home-Position
  return globalWorldModel->basePosition();
}
