
#include <stdio.h>

#include "BodyInfos.h"
#include "../basic/Exception.h"
#include "../basic/Log.h"

// HACK fuer GoalKickAfterCatchPositionAnalyzer.
int BodyInfos::staticMoves = 0;
int BodyInfos::staticCatches = 0;

BodyInfos::BodyInfos( const Ath98String& information ) {
  parseBodyInfos(information);
  theCount = -1;
}

BodyInfos::BodyInfos() {
  theTime = UNDEF_TIME;
  theCount = -1;
}

void BodyInfos::parseBodyInfos( const Ath98String& information ) {
  if( !(information.beginsWith( "(sense_body") ) ) {
    throwException( ( Exception("BodyInfos were incorrect used") ) );
  }
  static char viewQuality[10];
  static char viewWidth[10];

  sscanf( information.cString(),
	  "(sense_body %ld (view_mode %s %s (stamina %lf %lf) (speed %lf %lf) "
	  "(head_angle %lf) (kick %d) (dash %d) (turn %d) (say %d) (turn_neck %d) "
	  "(catch %d) (move %d) (change_view %d))",
	  &theTime, viewQuality, viewWidth, &theStamina, &theEffort,
	  &theAmountOfSpeed, &theDirectionOfSpeed, &theHeadAngle, &theKicks,
          &theDashs, &theTurns, &theSays, &theTurnNecks,
          &theCatches, &theMoves, &theChangeViews );
   
   // Hack fuer GoalKickAfterCatchPositionAnalyzer!
   staticMoves = theMoves;
   staticCatches = theCatches;
   
  theDirectionOfSpeed = normalizeAngle( -radFromDegree( theDirectionOfSpeed ) );
  theHeadAngle = normalizeAngle( -radFromDegree( theHeadAngle ) );
  switch( viewQuality[0] ) { 
  case 'l':  // low 
    theViewQuality = VIEW_QUALITY_LOW;  
    break;  
  case 'h':  // high 
    theViewQuality = VIEW_QUALITY_HIGH;  
    break;
  default:
    theViewQuality = VIEW_UNDEF;
    WARNING( Log::SENSORS, "Unknown viewQuality received" );
  }
  switch( viewWidth[1] ) { // Ath98Strings schliessen mit ')'
  case 'a': // narrow
    theViewWidth = VIEW_WIDTH_NARROW;
    break;
  case 'o': // normal
    theViewWidth = VIEW_WIDTH_NORMAL;
    break;
  case 'i': // wide
    theViewWidth = VIEW_WIDTH_WIDE;
    break;
  default:
    theViewWidth = VIEW_UNDEF;
    WARNING( Log::SENSORS, "Unknown viewWidth received" );
  }
}

BodyInfos* BodyInfos::copy() const {
  return new BodyInfos( *this );  
} 
 
 
Ath98String BodyInfos::toString() const  
{ 
  Ath98String output = Ath98String( "( BodyInfos time: " ) +  Ath98String( time() ) +  
    Ath98String( ", viewQuality: " ) +  Ath98String( viewQuality() ) +  
    Ath98String( ", viewWidth: " ) +  Ath98String( viewWidth() ) + 
    Ath98String( ", stamina: " ) +  Ath98String( stamina() ) + 
    Ath98String( ", effort: " ) +  Ath98String( effort() ) + 
    Ath98String( ", amountOfSpeed: " ) +  Ath98String( amountOfSpeed() ) + 
    Ath98String( ", directionOfSpeed: " ) +  
    Ath98String( degFromRadiant( directionOfSpeed() ) ) +
    Ath98String( ", headAngle: " ) +  
    Ath98String( degFromRadiant( headAngle() ) ) + 
    Ath98String( ", kicks: " ) +  Ath98String(  kicks() ) + 
    Ath98String( ", dashs: " ) +  Ath98String( dashs() ) + 
    Ath98String( ", turns: " ) +  Ath98String( turns() ) + 
    Ath98String( ", says: " ) +  Ath98String( says() ) + 
    Ath98String( ", turnNecks: " ) +  Ath98String( turnNecks() ) +  
    Ath98String( ", number: " ) + Ath98String( count() ) +
    Ath98String( " )" );  
  return output;   
}
