#include "ConfiguredBall.h"
#include "../basic/Log.h" 
#include <math.h> 
 
/** 
 * Aktueller Stand 
 * 
 *   $RCSfile: ConfiguredBall.cc,v $ 
 *   $Revision: 1.1.1.1 $ 
 *   $Date: 2001/07/31 22:23:09 $ 
 *   $Author: rami $  
 * 
 */ 
double ConfiguredBall::theBallSize;  
double ConfiguredBall::theBallDecay;  
double ConfiguredBall::theBallMaxSpeed;  
double ConfiguredBall::theKickPowerRate;  
double ConfiguredBall::theMaxPower;  
double ConfiguredBall::theMinPower;  
double ConfiguredBall::theMaxMoment;  
double ConfiguredBall::theMinMoment;  
double ConfiguredBall::theKickMargin;  
short ConfiguredBall::isSetOnce = NO;  
 
  
ConfiguredBall::ConfiguredBall( int id,
			        const Vector& aPos ,
			        const Vector& aSpeed,
                                const Confidence& aPosConfidence,
                                const Confidence& aSpeedConfidence )
  : ModelledObject( id, aPos, aSpeed, aPosConfidence, aSpeedConfidence )
{
  if (isSetOnce==NO){ 
    setClassVars();  
  } 
} 

 
 
void ConfiguredBall::setClassVars()  
{ 
  setSize(globalConf->ballSize);  
  setDecay(globalConf->ballDecay);  
  setMaxSpeed(globalConf->ballSpeedMax);  
  setKickPowerRate(globalConf->kickPowerRate);  
  setMaxPower(globalConf->maxPower);  
  setMinPower(globalConf->minPower);  
  setMaxMoment(radFromDegree(globalConf->maxMoment));  
  setMinMoment(radFromDegree(globalConf->minMoment));  
  setKickMargin(globalConf->kickMargin);  
  isSetOnce=YES;  
} 
 

 
double ConfiguredBall::effectiveKickPower( double power, 
    const ModelledPlayer& kickingPlayer ) const  
{
  double dist = (kickingPlayer.pos() - pos()).length() - size() - kickingPlayer.size();
  double deviation = fabs( deviationBetween( kickingPlayer.bodyDir(), pos().angle() ) );
  double kickEffort = 1 - 0.25 * deviation / Ath98_PI - 0.25 * dist / kickingPlayer.kickableMargin() ;
  power = normalizeKickPower(power);
  return power * kickEffort; 
}
  
double ConfiguredBall::kickPowerForSend( double power, 
    const ModelledPlayer& kickingPlayer ) const  
{ 
  double dist = (kickingPlayer.pos() - pos()).length() - size() - kickingPlayer.size();
  double deviation = fabs( deviationBetween( kickingPlayer.bodyDir(), pos().angle() ) );
  double kickEffort = 1 - 0.25 * deviation / Ath98_PI - 0.25 * dist / kickingPlayer.kickableMargin() ;
  power = power / kickEffort;  
  return normalizeKickPower(power);  
} 
 
double ConfiguredBall::maxEffectiveKickPower(  
    const ModelledPlayer& kickingPlayer) const  
{ 
  return effectiveKickPower( maxPower(), kickingPlayer );  
} 
