#include "ModelledBall.h"

#include "../basic/Log.h"
#include <math.h> 
 
  
/** 
 * Aktueller Stand 
 * 
 *   $RCSfile: ModelledBall.cc,v $ 
 *   $Revision: 1.2 $ 
 *   $Date: 2001/08/02 08:09:42 $ 
 *   $Author: rami $  
 * 
 */ 

ModelledBall::ModelledBall( int id,
			    const Vector& aPos ,  
			    const Vector& aSpeed,
                            int aLastTouchedBy,
	                    int aLastTouchedAtTime,
                            const Confidence& aPosConfidence,
			    const Confidence& aSpeedConfidence )
  : ConfiguredBall( id, aPos, aSpeed, aPosConfidence, aSpeedConfidence ),
  theLastTouchedBy( aLastTouchedBy ),   
  theLastTouchedAtTime( aLastTouchedAtTime ) 
{ 
  if (isSetOnce == NO){ 
    setClassVars();  
  } 
} 
 
ModelledBall::ModelledBall( const MovingObject& aMovingObject,
			    int aLastTouchedBy,
			    int aLastTouchedAtTime,
                            const Confidence& aPosConfidence,
                            const Confidence& aSpeedConfidence )
  :
  ConfiguredBall( aMovingObject.id(),  
		  aMovingObject.pos(), 
		  aMovingObject.speed(),
		  aPosConfidence,
		  aSpeedConfidence ),
  theLastTouchedBy( aLastTouchedBy),
  theLastTouchedAtTime( aLastTouchedAtTime)

{
  if (isSetOnce==NO){
    setClassVars();
  }
}

StableObject* ModelledBall::copy() const
{
  return new ModelledBall( *this );
}

void ModelledBall::simulateSteps( unsigned int simSteps )
{
  // cerr << "ModBall::simulateSteps with : " << simSteps << "\n";
  // kann evtl. auch ueber eine Tabelle veraendert und effektiviert werden

	simSteps = max( (int) simSteps, 0);
	simSteps = min( (int) simSteps, 100);

  while( simSteps > 0 ) {
    if( posConfidence().reliability() > 0.000001 &&
        speedConfidence().reliability() > 0.000001 ) {
      setPos( pos() + speed() );  
      setSpeed( decay() * speed() );  
    }
    else {
      WARNING( Log::MODELLED_OBJECTS, 
               "ModelledBall::simulateSteps: pos or speed of ball invalid." );
    }
    thePosConfidence.decrease();
    theSpeedConfidence.decrease();
    --simSteps;
  }
}
 
void ModelledBall::accelerate( double power, double angle,  
    const ModelledPlayer& byPlayer ) 
{ 
  power = effectiveKickPower( power, byPlayer );  
  angle = normalizeKickAngle( angle );  
   
  if( speedConfidence().reliability() > 0.001 ) {
    
    Vector myAcceleration( power, normalizeAngle(angle + byPlayer.bodyDir()) ,POLAR);  
    
    setSpeed( speed() + (kickPowerRate() * myAcceleration) );  
    if( speed().length() > maxSpeed() ) { 
      Vector mySpeed = speed();  
      mySpeed.setPolar( maxSpeed(), mySpeed.angle() );  
      setSpeed(mySpeed);  
    } 
  } 
}

bool ModelledBall::isInKickRangeOfPlayer( const ModelledPlayer& aPlayer ) const 
{ 
  double distFromPlayer = (pos() - aPlayer.pos()).length();  
  return ( distFromPlayer <= globalConf->kickRange );
}

Ath98String ModelledBall::toString() const
{
  Ath98String output;
 
  output = Ath98String( "( ID: " ) + Ath98String( id() ) +
    Ath98String(", Pos: ") + pos().toString() +
    Ath98String( ", PosConfidence" ) + posConfidence().toString() +
    Ath98String( ", Speed: " ) + speed().toString() +
    Ath98String( ", SpeedConfidence" ) + speedConfidence().toString() +
    Ath98String( ", LastTouchedBy: " ) + Ath98String( (long)theLastTouchedBy ) +
    Ath98String( ", LastTouchedAtTime: " ) + Ath98String( (long)theLastTouchedAtTime ) +
    Ath98String( " )" );
  return output;
}                                                                                                                      
