#include "Neck.h"

#include "WorldModelInterface.h"
#include "../basic/Common.h"
#include "../basic/Configuration.h"
#include "../basic/Timer.h"
#include "../intentions/skills/Skills.h"
#include "../serverInterface/Sensors.h"

Neck *globalNeck; 

Neck::Neck()  {

  theNeedToSendNeckCommand = NO;
  theNeedToSendViewCommand = NO;

  theTurnNeckCommand = Action( EMPTY_ACTION ,0 );
  theChangeViewCommand = Action( EMPTY_ACTION ,0 );


  theIdOfThePlayerToBeSeen = 0;
  thePosToBeSeen = Vector( 0, 0 );

  theNeckBehaviourPriority = NO_PRIORITY;
  theNeckBehaviourAlignment = ALIGN_CENTER;

  theViewPriority = NO_PRIORITY;
  theViewWidth = VIEW_WIDTH_NORMAL;
  theViewQuality = VIEW_QUALITY_HIGH;
  theIdString = "Neck";

  theMinNeckAngle = radFromDegree(globalConf->minNeckAngle); // -1,57
  theMaxNeckAngle = radFromDegree(globalConf->maxNeckAngle); // +1,57
}


void Neck::setViewBehaviour(const int desiredViewWidth,
    const int desiredViewQuality, const int desiredViewPriority)
{
  INFO( Log::NECK,  3,  idString() , globalLog->infoStream
    << "NECK::setViewBehavior " << desiredViewWidth << "|"
    << desiredViewQuality << "|" << desiredViewPriority << endl );
  INFO( Log::NECK,  3,  idString() , globalLog->infoStream
    << "NECK::lastViewBehavior " << theViewWidth << "|" 
    << desiredViewQuality << "|" << theViewPriority << endl );  
  if (desiredViewWidth < VIEW_WIDTH_NARROW ||
      desiredViewWidth > VIEW_WIDTH_WIDE) {
        WARNING( Log::NECK, "VIEW_WIDTH is out of range" ); 
  }
  
  if( desiredViewPriority <= theViewPriority ) {
    theViewWidth = desiredViewWidth; 
    theViewQuality = desiredViewQuality; 
    theViewPriority = desiredViewPriority; 
  } 
}



void Neck::setNeckBehaviour( const int desiredNeckBehaviour, const int
    desiredNeckBehaviourAlignment, const int desiredNeckBehaviourPriority )
{
  INFO( Log::NECK  ,  6  ,  idString() , globalLog->infoStream
    << "NECK::setNeckBehavior simple with" << desiredNeckBehaviour << "|" 
    << desiredNeckBehaviourAlignment << "|" << desiredNeckBehaviourPriority << endl );
  INFO( Log::NECK  ,  6  ,  idString() , globalLog->infoStream 
    << "old Behaviour: " << theNeckBehaviour << "|" 
    <<	theNeckBehaviourAlignment << "|" << theNeckBehaviourPriority <<	endl );
  
  if( desiredNeckBehaviourPriority <= theNeckBehaviourPriority ) {
    theNeckBehaviour = desiredNeckBehaviour;
    theNeckBehaviourAlignment = desiredNeckBehaviourAlignment;
    theNeckBehaviourPriority = desiredNeckBehaviourPriority;
    theIdOfThePlayerToBeSeen = 0;
    thePosToBeSeen = Vector( 0, 0 );
  }
  INFO( Log::NECK  ,  6  ,  idString() , globalLog->infoStream 
    << "new Behaviour: " << theNeckBehaviour << "|" 
    << theNeckBehaviourAlignment << "|" << theNeckBehaviourPriority << endl );
}



void Neck::setNeckBehaviour( const int desiredNeckBehaviour, const int
    desiredNeckBehaviourAlignment, const int desiredNeckBehaviourPriority,
    const int desiredId )
{
  INFO( Log::NECK, 6,  idString() , globalLog->infoStream 
    << "NECK::setNeckBehavior with Player " << desiredNeckBehaviour << "|" 
    << desiredNeckBehaviourAlignment << "|" << desiredNeckBehaviourPriority << endl );
  INFO( Log::NECK, 6,  idString() , globalLog->infoStream
    << "old Behaviour: " << theNeckBehaviour << "|" 
    << theNeckBehaviourAlignment << "|" << theNeckBehaviourPriority << endl );  
    
  if( desiredNeckBehaviourPriority <= theNeckBehaviourPriority ) {
    theNeckBehaviour = desiredNeckBehaviour;
    theNeckBehaviourAlignment = desiredNeckBehaviourAlignment;
    theNeckBehaviourPriority = desiredNeckBehaviourPriority;

    if( globalWorldModel->situation().playerWithId( desiredId ).id() != UNDEF_ID ){
      theIdOfThePlayerToBeSeen = desiredId;
      INFO( Log::NECK  ,  3  ,  idString() , globalLog->infoStream
        << "NECK::setTheIdOfThePlayerToBeSeen " << theIdOfThePlayerToBeSeen
        << " -> YES\n");
    }
    thePosToBeSeen = Vector( 0, 0 );
  }
}


void Neck::setNeckBehaviour( const int desiredNeckBehaviour, const int
    desiredNeckBehaviourAlignment, const int desiredNeckBehaviourPriority,
    const Vector desiredPos ){

  INFO( Log::NECK, 6,  idString() , globalLog->infoStream 
    << "NECK::setNeckBehavior with Position" << desiredNeckBehaviour << "|" 
    << desiredNeckBehaviourAlignment << "|" << desiredNeckBehaviourPriority << endl );
  INFO( Log::NECK, 6,  idString() , globalLog->infoStream 
    << "old Behaviour: " << theNeckBehaviour << "|" 
    << theNeckBehaviourAlignment << "|" << theNeckBehaviourPriority << endl );
    
  if( desiredNeckBehaviourPriority <= theNeckBehaviourPriority ) {
    theNeckBehaviour = desiredNeckBehaviour;
    theNeckBehaviourAlignment = desiredNeckBehaviourAlignment;
    theNeckBehaviourPriority = desiredNeckBehaviourPriority;
    theIdOfThePlayerToBeSeen = 0;
    if ( desiredPos.isValid() &&
        globalWorldModel->field().isInsideField(desiredPos)) {
      thePosToBeSeen = desiredPos;
    }
    else {
      thePosToBeSeen = Vector( 0, 0 );
    }
  }
}



double Neck::angleAcceptingLimitedNeckPosition( const double angle )
{
  double desiredHeadAngle = normalizeAngle( -bodyDir() + angle );
  if( desiredHeadAngle > Ath98_PI/2 && desiredHeadAngle < 3*Ath98_PI/2 ) {
    desiredHeadAngle= (desiredHeadAngle < Ath98_PI )? Ath98_PI/2 : 3*Ath98_PI/2;
  }
  return normalizeAngle( desiredHeadAngle - faceDir() + bodyDir()); 
}



void Neck::generateViewCommand()
{
  INFO( Log::NECK, 6, idString(), globalLog->infoStream
	  << "ENTER Neck::generateViewCommand\n"); 

  theChangeViewCommand = Action( EMPTY_ACTION ,globalWorldModel->simStep() );

  // pruefen, ob die geforderten Werte bereits eingestellt sind
  if( ( theViewWidth != currentViewWidth() ) ||
      ( theViewQuality != currentViewQuality() ) ) {
    theChangeViewCommand =  Action( CHANGE_VIEW_ACTION, globalWorldModel->simStep(), theViewWidth, theViewQuality );
    theNeedToSendViewCommand = YES;
  }
  theViewPriority = NO_PRIORITY;

  INFO( Log::NECK, 6, idString(), globalLog->infoStream
	  << "LEAVE Neck::generateViewCommand\n"); 
}



void Neck::generateNeckCommand()
{
  INFO( Log::NECK, 6,  idString() , globalLog->infoStream 
    << "ENTER NECK::generateNeckCommand() "<< theNeckBehaviour << endl);
  INFO( Log::NECK, 10,  idString() , globalLog->infoStream
      << "relativeFaceDir : "<< relativeFaceDir() << endl
      << "currentFaceDir  : " << faceDir() << endl);

  double angle;
  vector<int> thePlayers;

  theTurnNeckCommand = Action( EMPTY_ACTION , globalWorldModel->simStep() );

  // HACK 
  // Wenn die Ballconfidence zu weit absinkt, gucken wir uns den Ball genauer an.
  // Damit wird aber alles andere berschrieben. Tjaja.
  // Wenn Ball extrem unsicher, dann wird er aber gar nicht erst angesteuert. Das
  // soll die Scanfieldintention abfangen
  // 
  if( globalWorldModel->ball().posConfidence().reliability() > 0.4 ){ 
    switch( theViewWidth ) {
      case VIEW_WIDTH_NARROW:
        if( globalWorldModel->ball().posConfidence().reliability() < 0.8) theNeckBehaviour = NECK_TO_BALL;
        break;
      case VIEW_WIDTH_NORMAL:
        if( globalWorldModel->ball().posConfidence().reliability() < 0.7) theNeckBehaviour = NECK_TO_BALL;
        break;
      default:
        if( globalWorldModel->ball().posConfidence().reliability() < 0.6) theNeckBehaviour = NECK_TO_BALL;
        break;
    } 
  }

  Action mainAction = globalTimer->lastMainAction();
      ModelledPlayer myself = globalWorldModel->player();
      ModelledBall ball = globalWorldModel->ball();
      double desiredHeadDir;
      double desiredHeadAngle;

  switch( theNeckBehaviour ) {
    case NECK_IN_FRONT:
      angle =	angleAcceptingLimitedNeckPosition( -1.0 * relativeFaceDir() );
      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
    
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_IN_FRONT: \n");
      break;
      
/* HACK ( - radFromDegree(1) )
  vor allem bei ScanField kann sonst folgender Fall eintrten:
   neck ganz nach links -> soll ganz nach rechts gedreht werden -> angle = - (180 + noise)
   normalisiert ist dies 180 - noise , d.h. mitunter <180
   das aber heist nach links drehen (wir sind schon ganz links) */
    case NECK_TO_LEFT:
      angle = normalizeAngle( Ath98_PI/2 - radFromDegree(1) - faceDir() + bodyDir() );
      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
    
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_LEFT: \n");
      break;
    case NECK_TO_RIGHT:
      angle =	normalizeAngle( -Ath98_PI/2 + radFromDegree(1) - faceDir() + bodyDir());
      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
    
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_RIGHT: \n");
      break;

    case NECK_TO_BALL:
      if( mainAction.id() == DASH_ACTION ) {
        myself.accelerate( mainAction.firstParameter() );
      }
      else if( mainAction.id() == TURN_ACTION ) {
        myself.turn( mainAction.firstParameter() );
      }
      else if( mainAction.id() == KICK_ACTION ) {
        ball.accelerate( mainAction.firstParameter(), mainAction.secondParameter(), myself );
      }
      ball.simulateSteps(1);
      myself.simulateSteps(1);

// HACK - sollte eigentlich von headAngleToTurnTo() gemacht werden !!!

      desiredHeadDir = (ball.pos() - myself.pos()).angle();
      desiredHeadAngle = normalizeAngle( -myself.bodyDir() + desiredHeadDir );
      if( desiredHeadAngle >Ath98_PI/2 && desiredHeadAngle < 3*Ath98_PI/2 ) {
        desiredHeadAngle= (desiredHeadAngle < Ath98_PI )? Ath98_PI/2 : 3*Ath98_PI/2;
      }
      angle = normalizeAngle( desiredHeadAngle - myself.faceDir() + myself.bodyDir()); 

      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
       
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_BALL \n");
      break;  

    case NECK_TO_OWN_GOAL:
      angle = angleAcceptingLimitedNeckPosition( headAngleToTurnTo( globalWorldModel->field().ourGoal() ) );

      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_OWN_GOAL: " << angle << endl);
      // if( isAngleGreatEnough( angle ) == NO ) break;

      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES; 
      break;

    case NECK_TO_ENEMY_GOAL:
      angle = angleAcceptingLimitedNeckPosition ( headAngleToTurnTo( globalWorldModel->field().enemyGoal() ) );
      
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_ENEMY_GOAL: " << angle << endl);
      // if( !isAngleGreatEnough( angle ) ) break;

      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES; 
      break;


    case NECK_OSCILLATE:
      // war vllig falsch implementiert
      // Funktionalitt ist nicht trival implementierbar
      // Probleme mit unterschiedlichen Sichttakten etc
      // msste ggf geschrieben werden
      break;

    case NECK_TO_MIDDLE:
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_MIDDLE \n");

      angle = angleAcceptingLimitedNeckPosition( headAngleToTurnTo( Vector( 0, 0 ) ) ); 
      // if( isAngleGreatEnough( angle ) == NO ) break;

      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
      break;

    case NECK_TO_POS:
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream << "NECK::NECK_TO_POS " << thePosToBeSeen << endl);

      angle = angleAcceptingLimitedNeckPosition( headAngleToTurnTo( thePosToBeSeen ) );
      // if( isAngleGreatEnough( angle ) == NO ) break;

      if( isAngleGreatEnough( angle ) == NO ) {
        theNeedToSendNeckCommand = NO;
        break;
      }
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
      break;

    case NECK_TO_PREFERRED_KICK_DIRECTION:
      angle = angleAcceptingLimitedNeckPosition( globalWorldModel->preferredKickDirection(
          globalWorldModel->player().pos() ).angle() - faceDir() );
      // if( isAngleGreatEnough( angle ) == NO ) break;

      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
      break;

    case NECK_TO_BACK_KICK_DIRECTION:
      angle =  angleAcceptingLimitedNeckPosition( globalWorldModel->backKickDirection(
          globalWorldModel->player().pos() ).angle() - faceDir() );
      // if( isAngleGreatEnough( angle ) == NO ) break;

      if( isAngleGreatEnough( angle ) == NO ) break;
      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
      break;

    case NECK_TO_PLAYER_WITH_ID:
      INFO( Log::NECK, 10,  idString() , globalLog->infoStream 
        << "NECK::NECK_TO_PLAYER_WITH_ID " << theIdOfThePlayerToBeSeen << endl);

      if ( (theIdOfThePlayerToBeSeen < MIN_PLAYER_ID || theIdOfThePlayerToBeSeen > MAX_PLAYER_ID ) &&
           (theIdOfThePlayerToBeSeen < MIN_OPPONENT_ID || theIdOfThePlayerToBeSeen > MAX_OPPONENT_ID ) ) {
        WARNING( Log::NECK, "NECK::NECK_TO_PLAYER_WITH_ID \n");
        break;
      }

      if( globalWorldModel->situation().playerWithId( theIdOfThePlayerToBeSeen ).id() == UNDEF_ID ){
        cerr << "Neck::generateNeckCommand()     player.id() ist UNDEF !!!\n";
        WARNING( Log::NECK, "playerSelf.id() ist UNDEF !!!\n" );
        break;
      }

      angle = angleAcceptingLimitedNeckPosition(
        headAngleToTurnTo( globalWorldModel->situation().playerWithId( theIdOfThePlayerToBeSeen ).pos() ) );
      if( isAngleGreatEnough( angle ) == NO ) break;

      theTurnNeckCommand = Action( TURN_NECK_ACTION, globalWorldModel->simStep(), angle );
      theNeedToSendNeckCommand = YES;
      theIdOfThePlayerToBeSeen = 0;

      break;

    default:
    break;  
  }

  theNeckBehaviourAlignment = ALIGN_CENTER;
  theNeckBehaviourPriority = NO_PRIORITY;

  INFO( Log::NECK, 6,  idString() , globalLog->infoStream << "LEAVE NECK::generateNeckCommand()\n");
  INFO( Log::NECK, 6,  idString() , globalLog->infoStream << "with NeedToSend: " << theNeedToSendNeckCommand << endl);
  if( theNeedToSendNeckCommand ){
      INFO( Log::NECK, 6,  idString() , globalLog->infoStream << "with Action: " << theTurnNeckCommand << endl);
  }
}// end of generateNeckCommand()



bool Neck::isAngleGreatEnough( double anAngle ){
  INFO( Log::NECK, 10,  idString() , globalLog->infoStream 
    << "ENTER NECK::isAngleGreatEnough with angle: " << anAngle << endl );  

  anAngle = normalizeAngle(anAngle);

  // berechnet den Winkelabstand von 0 aus (0 - 360 --> 0 - 180)
  anAngle = fabs( deviationBetween( 0.0, anAngle ) );

  if( anAngle < radFromDegree(1) ) {
    return NO;
  }
  else {
    return YES;
  }
  
/*  if( theNeckBehaviourAlignment == ALIGN_CENTER ){
    if( fabs( anAngle ) < viewWidthAngleWithId( currentViewWidth() ) / 18.0 )
      return NO;
    else
      return YES;
   }
   else {
    if( fabs( anAngle ) < viewWidthAngleWithId( currentViewWidth() ) / 28.0 )
      return NO;
    else
      return YES;
   }
*/   
}
