#ifndef _PLAYERTYPES_H_
#define _PLAYERTYPES_H_

#include <map.h>
#include "../basic/Ath98String.h"
#include "../basic/Log.h"


class PlayerTypes
{
public:
  PlayerTypes();
  ~PlayerTypes();
  
  void parsePlayerTypeMessage(const Ath98String& message);
  void parseSubstituteMessage(const Ath98String& message);
  
  double playerMaxSpeed(int i);
  double staminaIncMax(int i);
  double playerDecay(int i);
  double inertiaMoment(int i);
  double dashPowerRate(int i);
  double playerSize(int i);
  double kickableMargin(int i);
  double kickRand(int i);
  double extraStamina(int i);
  double effortMax(int i);
  double effortMin(int i);
  double kickRange(int i);

  void setTypeOfPlayerWithId(int aPlayerId, int aPlayerType);
  int typeOfPlayerWithId(int aPlayerId);

  double getMaxDistForPlayerAfterStep(int playerType, int steps, int offset) const;

  static const int numberOfPlayerTypes = 7; 

private:
  void calculateDependentParameters(int aId);

  // Werte, die der server fr die heterogenen Spieler wrfelt
  double thePlayerMaxSpeed[numberOfPlayerTypes];
  double theStaminaIncMax[numberOfPlayerTypes];
  double thePlayerDecay[numberOfPlayerTypes];
  double theInertiaMoment[numberOfPlayerTypes];
  double theDashPowerRate[numberOfPlayerTypes];
  double thePlayerSize[numberOfPlayerTypes];
  double theKickableMargin[numberOfPlayerTypes];
  double theKickRand[numberOfPlayerTypes];
  double theExtraStamina[numberOfPlayerTypes];
  double theEffortMax[numberOfPlayerTypes];
  double theEffortMin[numberOfPlayerTypes];

  // Werte, die sich mit den obigen verndern
  double theKickRange[numberOfPlayerTypes];

  // Tabelle, die die Spieler <-> Spielertypen verwaltet
  // Index: 0-10 unsere Spieler mit Offset MIN_TEAMMATE_ID, Goalie = 0
  // 11-21: gegnerische Spieler mit Offset MIN_OPPONENT_ID, Goalie = 11
  int thePlayerMap[22];

  // verwaltet die schon geparseten Player
  // diese werden aus dem Vector rausgeschmissen !
  bool parsedPlayers[numberOfPlayerTypes];

  // Tabelle die fr die verschiedenen Player, die Maximaldistanzen
  // beim Dashing nach n Takten hlt
  // HACK
  // in Zukunft soll noch ein Staminalimit und ein MaxDesiredDashPower
  // bergeben werden knnen
  double theMaxPlayerDist[numberOfPlayerTypes][101][9];

  void generatePlayerDistTable();

  void printPlayerDistTable() const;
};

  inline double PlayerTypes::playerMaxSpeed(int i){
    return thePlayerMaxSpeed[i];
  }
  inline double PlayerTypes::staminaIncMax(int i){
    return theStaminaIncMax[i];
  }
  inline double PlayerTypes::playerDecay(int i){
    return thePlayerDecay[i];
  }
  inline double PlayerTypes::inertiaMoment(int i){
    return theInertiaMoment[i];
  }
  inline double PlayerTypes::dashPowerRate(int i){
    return theDashPowerRate[i];
  }
  inline double PlayerTypes::playerSize(int i){
    return thePlayerSize[i];
  }
  inline double PlayerTypes::kickableMargin(int i){
    return theKickableMargin[i];
  }
  inline double PlayerTypes::kickRand(int i){
    return theKickRand[i];
  }
  inline double PlayerTypes::extraStamina(int i){
    return theExtraStamina[i];
  }
  inline double PlayerTypes::effortMax(int i){
    return theEffortMax[i];
  }
  inline double PlayerTypes::effortMin(int i){
    return theEffortMin[i];
  }
  inline double PlayerTypes::kickRange(int i){
    return theKickRange[i];
  }

  inline void PlayerTypes::setTypeOfPlayerWithId(int aPlayerId, int aPlayerType){
    if( (aPlayerId >= MIN_TEAMMATE_ID) && (aPlayerId <= MAX_EXISTING_TEAMMATE_ID) ){
      thePlayerMap[aPlayerId - MIN_TEAMMATE_ID] = aPlayerType;
    }
    else if( (aPlayerId >= MIN_OPPONENT_ID) && (aPlayerId <= MAX_EXISTING_OPPONENT_ID) ){
      thePlayerMap[aPlayerId - MIN_OPPONENT_ID + 11] = aPlayerType;
    }
    else {
      INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream <<
            "referenced thePlayerMap with undef PlayerId\n" );
    }
  }
  
  inline int PlayerTypes::typeOfPlayerWithId(int aPlayerId){
    if( (aPlayerId >= MIN_TEAMMATE_ID) && (aPlayerId <= MAX_EXISTING_TEAMMATE_ID) ){
      return thePlayerMap[aPlayerId - MIN_TEAMMATE_ID];
    }
    else if( (aPlayerId >= MIN_OPPONENT_ID) && (aPlayerId <= MAX_EXISTING_OPPONENT_ID) ){
      return thePlayerMap[aPlayerId - MIN_OPPONENT_ID + 11];
    }
    else{
      INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream <<
            "referenced thePlayerMap with undef PlayerId\n" );
    }
    // in Fehlerfall soll noch vernnftig weitergearbeitet werden
    return 0;
  }

  inline double PlayerTypes::getMaxDistForPlayerAfterStep(int playerType, int steps, int offset = 0) const {
    if( ( (steps < 0) || (steps > 100) ) ){
      cerr << "PlayerTypes::getMaxDistForPlayerAfterStep  steps out of range  --> returning: theMaxPlayerDist[playerType][100] !!!\n";
      steps = 100;
    }
    return theMaxPlayerDist[playerType][steps][offset];
  }
  
extern PlayerTypes *globalPlayerTypes;

#endif
