#include "WorldModelCore.h"
#include "WorldModelIntegrator.h"
#include "WorldModelCommunicator.h"
#include "ModelledPlayer.h"
#include "ModelledBall.h"

#include "../basic/Timer.h"

#include "../intentions/skills/Skills.h"


WorldModelIntegrator::WorldModelIntegrator( )
{

}

void WorldModelIntegrator::setWorldModelReference( WorldModelCore*
						   aWorldModelCore  )
{
  theCore = aWorldModelCore;
}

void WorldModelIntegrator::setSensorsReference( Sensors*
						   aSensor  )
{
  theSensors = aSensor;
}

void WorldModelIntegrator::integrateSensorInfo()
{
  int startTime = globalTimer->getAgentTime();
  // HACK !!!
  bool oldSeeInfoFlag = NO;
  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream <<
	"    ENTER WorldModelIntegrator::integrateSensorInfo\n" );

  if( theSensors->isNewBodyInfo() ) {
    //schauen ob noch alte Sicht-Infos vorliegen, dann vorher auswerten
    if( theSensors->isNewSeeInfo() &&
        theSensors->simStep() < theSensors->simStepOfSoccerServer() ) {
//cerr << "+++++++++ Michaels Error +++++++++++\n";
      theCore->simStepOfLastMichaelsError = theSensors->simStepOfSoccerServer();
      oldSeeInfoFlag = YES;
      INFO( Log::WORLD_MODEL, 8, idString(), globalLog->infoStream <<
            "Got old see info and new body info.\n" );
      BodyInfos bodyInfosForSee = theSensors->lastBodyInfos();

      // versuche BodyInfos in der BodyInfoList zu finden, die vom selben Takt,
      // wie die letzten SeeInfos stammen (Konsistenz)
      for( int i = theSensors->bodyInfos().size() - 1; i >= 0; --i ) {
        if( theSensors->bodyInfos()[i].time() == theSensors->simStep() ) {
          bodyInfosForSee = theSensors->bodyInfos()[i];
          processBodyInfo( bodyInfosForSee );
          break;
        }
      }
/*      if( theSensors->simStep() != theCore->simStep() ) {
        synchronizeSituations( theSensors->simStep() );
      }*/
      processSeeInfo( bodyInfosForSee );
    }
    processBodyInfo( theSensors->lastBodyInfos() );

  }

  if( theSensors->simStepOfSoccerServer() != theCore->simStep() ) {
    synchronizeSituations( theSensors->simStepOfSoccerServer() );
  }
  if( theSensors->isNewRefereeInfo() ) {
    processRefereeInfo();
  }
  if( theSensors->isNewHearInfo() ) {
    processHearInfo();
  }
  if( theSensors->isNewSeeInfo() &&
      theSensors->simStep() == theSensors->simStepOfSoccerServer() &&
      oldSeeInfoFlag == NO) {
    processSeeInfo( theSensors->lastBodyInfos() );
  }

  improveConsistency();

//  cout << "########  Ball nach improve: " << globalWorldModel->ball() << endl;

  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream <<
	"    LEAVE WorldModelIntegrator::integrateSensorInfo, time needed: " <<
        ( globalTimer->getAgentTime() - startTime ) << '\n' );
}



void WorldModelIntegrator::integrateCorrectSensorInfo()
{
  if( theSensors->isNewFullstateInfo() ) {
    theCore->theCorrectSituation = theSensors->correctSituation();
    INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream
      << " correctSituation from Fullstate-Info = "
   	  << theCore->theCorrectSituation  << endl);
  }
}



void WorldModelIntegrator::registerExecution( const Action& anAction )
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream <<
	"    ENTER WorldModel::registerExecution\n" );
  // Alle Kommandos die im SoccerServer sofort ausgefuehrt werden -> ausfuehren
  if( anAction.id() == MOVE_ACTION || anAction.id() == CATCH_ACTION ||
      anAction.id() == SAY_ACTION || anAction.id() == CHANGE_VIEW_ACTION ||
      anAction.id() == SENSE_BODY_ACTION ) {
    updateWithAction( anAction );
    if( anAction.id() == MOVE_ACTION || anAction.id() == CATCH_ACTION ) {
      // Hauptaktionen in EMPTY_ACTION umwandeln, da bereits simuliert
      Action emptyAction = Action( EMPTY_ACTION, anAction.simStep() );
      theCore->theExecutedActions.push_back( emptyAction );
    }
  }
  else {
    // Dies Kommandos werden erst am Ende des Taktes im SoccerServer
    // simuliert also werden sie in eine Queue gepackt und spaeter mit
    // updateWithAction simuliert.
    theCore->theExecutedActions.push_back( anAction );
  }
  INFO( Log::WORLD_MODEL, 8, idString(), globalLog->infoStream <<
        " Registere action: " << anAction << " \n" );

  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream <<
	"    LEAVE WorldModelIntegrator::registerExecution, time needed: " <<
        ( globalTimer->getAgentTime() - startTime ) << '\n' );
}


void WorldModelIntegrator::processSeeInfo( const BodyInfos& bodyInfosForSee )
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<
	"    ENTER WorldModel::processSeeInfo\n" );

  // Situation anlegen und gesehene Objekte einfgen
  Situation seenSituation;
  setSimStepOfLastSee( theCore->simStepOfSee() );
  seenSituation.setSimStep( theCore->simStepOfSee() );
  seenSituation.addPlayer( modelOfSelf( bodyInfosForSee ) );
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<
   	" seenSituation before adding seen Objects = "
   	<< seenSituation  << "\n");
  addSeenObjects( seenSituation );

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<
	" seenSituation after adding seen Objects = "
	<< seenSituation  << "\n");

  theCore->theSituations[theCore->currentSituationIndex] = 
// das Vertauschen der beiden Situationen bringt konsistenteres WM, evtl. aber zuviele Spieler
//    mergeSituations( theCore->theSituations[theCore->currentSituationIndex], seenSituation, YES );
    mergeSituations( seenSituation, theCore->theSituations[theCore->currentSituationIndex], YES );

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<
	" seenSituation after merge simulated  Objects = "
	<< seenSituation  << "\n");

  INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream <<
	"New see situation:\n" << seenSituation );
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<
	"    LEAVE WorldModel::processSeeInfo, time needed: " <<
        ( globalTimer->getAgentTime() - startTime ) << '\n' );
 //cout << theCore->simStepOfSee() << "   WMI::processSeeInfos  - new SeeInfos:\n";
}

/******** INLINES FROM WMI *************/

 void WorldModelIntegrator::addCurrentSituation( const Situation& aSituation )
{
  theCore->currentSituationIndex = ( theCore->currentSituationIndex +
					   1 + theCore->situationFieldSize) %
    theCore->situationFieldSize;
  theCore->theSituations[theCore->currentSituationIndex] = aSituation;
}

 void WorldModelIntegrator::removeCurrentSituation()
{
  theCore->theSituations[theCore->currentSituationIndex].setSimStep( 0 );
  theCore->currentSituationIndex = ( theCore->currentSituationIndex - 1
					   + theCore->situationFieldSize) %
    theCore->situationFieldSize;
}
 void WorldModelIntegrator::setSimStepOfLastCatch( int newSimStep )
{
  theCore->theSimStepOfLastCatch = newSimStep;
}

 void WorldModelIntegrator::setSimStepWhenCatchedBallIsReleased(
				                      int newSimStep ){
  theCore->theSimStepWhenCatchedBallIsReleased = newSimStep;
}

 void WorldModelIntegrator::setSimStepOfLastSay( int newSimStep )
{
  theCore->theSimStepOfLastSay = newSimStep;
}

 void WorldModelIntegrator::setSimStepOfLastRefereeDecision( int newSimStep )
{
  theCore->theSimStepOfLastRefereeDecision = newSimStep;
}

 void WorldModelIntegrator::setSimStepOfLastFreekick( int newSimStep )
{
  theCore->theSimStepOfLastFreekick = newSimStep;
}

 void WorldModelIntegrator::setSimStepOfLastSee( int newSimStep )
{
  theCore->theSimStepOfLastSee = newSimStep;
}

void WorldModelIntegrator::processBodyInfo( const BodyInfos& bodyInfos )
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << " ENTER WorldModelIntegrator::processBodyInfo\n" );
  if( theCore->simStep() > bodyInfos.time() || ( theCore->simStep() == bodyInfos.time() && theCore->isTimeElapsingGameState() ) ) {
    // Weltmodell bereits weiter als letzte BodyInfos
    WARNING( Log::WORLD_MODEL, "Worldmodel simstep is in the future.\n" );
    return;
  }

  for( int j = 0; j < (int) theCore->theExecutedActions.size(); ++j ) {
    INFO(  Log::WORLD_MODEL, 8, idString(), globalLog->infoStream << "processing queded action nr: " << j << " action: " 
      << theCore->theExecutedActions[j] << endl );
      
    if( theCore->theExecutedActions[j].simStep() < theCore->simStep() && theCore->theExecutedActions[j].id() != TURN_NECK_ACTION ) {
      // Aktion ist zu alt -> nicht simulieren
      // aeltere turnNecks fallen nicht darunter
    }
    else if( theCore->theExecutedActions[j].simStep() > theCore->simStep() ) {
      // keine Aktion in diesem Takt abgesendet -> EMPTY_ACTION simulieren
      updateWithAction( Action( EMPTY_ACTION, theCore->simStep() ) );
      --j;
    }
    else {
      // schauen ob Aktion bestaetigt wurde
      if( theCore->theExecutedActions[j].simStep() <= bodyInfos.time() ) {
        int bodyInfosActionCount = bodyInfos.countForActionId( theCore->theExecutedActions[j].id() );
        if( bodyInfosActionCount == UNDEF_INT || theCore->theExecutedActions[j].actionCount() < bodyInfosActionCount ) {
          updateWithAction( theCore->theExecutedActions[j] );
        }
        else {
//          WARNING( Log::WORLD_MODEL, "Action was lost:\n" + theCore->theExecutedActions[j].toString() << endl );
          cerr << "Action was lost: " << theCore->theExecutedActions[j].toString() << endl;
        }
      }
      else {
        WARNING( Log::WORLD_MODEL, "Action with wrong timestamp:\n" + theCore->theExecutedActions[j].toString() << endl );
      }
    }
  }// end for
  
  int noActionsCount = bodyInfos.time() - theCore->simStep();
  while( noActionsCount > 0 ) {
    updateWithAction( Action( EMPTY_ACTION, theCore->simStep() ) );
    --noActionsCount;
  }

  theCore->theExecutedActions.clear();

  ModelledPlayer playerSelf = theCore->theSituations[theCore->currentSituationIndex].playerWithId( theCore->id() );
  if( playerSelf.id() == UNDEF_ID ){
    cerr << "WorldModelIntegrator::processBodyInfo     playerSelf.id() ist UNDEF !!!\n";
    WARNING( Log::WORLD_MODEL, "playerSelf.id() ist UNDEF !!!\n" ); 
  }
  updateOwnPlayerWithBodyInfos( playerSelf, bodyInfos );
  theCore->theSituations[theCore->currentSituationIndex].setPlayer(playerSelf);

  INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream << "New situation after bodyInfos:\n" 
    << theCore->theSituations[theCore->currentSituationIndex] << endl);
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << "LEAVE WorldModelIntegrator::processBodyInfo, time needed: " 
    << ( globalTimer->getAgentTime() - startTime ) << endl );
// cout << bodyInfos.time() << "   WMI::processBodyInfos  - new situation after bodyInfos:\n";
}


void WorldModelIntegrator::synchronizeSituations( int aSimStep ) {
  int startTime = globalTimer->getAgentTime();

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<	"ENTER WorldModelIntegrator::synchronizeSituations\n" );
  INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream << "SimStep Sensoren (gewuenscht): " << aSimStep 
    << " SimStep Weltmodell: " << theCore->simStep() << endl );

  int stepDifference = aSimStep - theCore->simStep();

  if( fabs(stepDifference) > theCore->historyCount ) {
    WARNING( Log::WORLD_MODEL, "Current WorldModel is corrupt.\n" );

    // wir erzeugen eine neue Situation in die wir uns selber integrieren
    // --> alles andere bleibt unbekannt, da die alten Daten nicht verlsslich sind
    ModelledPlayer myself = theCore->situation().playerWithId( theCore->id() );
    if( myself.id() == UNDEF_ID ){
      cerr << "WorldModelIntegrator::synchronizeSituations     playerSelf.id() ist UNDEF !!!\n";
      WARNING( Log::WORLD_MODEL, "playerSelf.id() ist UNDEF !!!\n" ); 
    }
    theCore->theSituations[theCore->currentSituationIndex] = Situation(aSimStep );
    theCore->theSituations[theCore->currentSituationIndex].addPlayer( myself );
    return;
  }

  if( stepDifference > 0 ) {
    // ich muss noch simulieren, sind wohl bodyInfos verloren gegangen
    WARNING( Log::WORLD_MODEL, "BodyInfos was lost for simStep:" << theCore->simStep() << " stepDifference: "  << stepDifference << endl );
    if( stepDifference <= 3 ) {
      // versuchen zu korrigieren
      for( unsigned int j = 0; j < theCore->theExecutedActions.size(); ++j ) {
        if( theCore->theExecutedActions[j].simStep() < aSimStep ) {
          updateWithAction( theCore->theExecutedActions[j] );
        }
        else {
          break;
        }
      }
      theCore->theExecutedActions.clear();
      stepDifference = aSimStep - theCore->simStep();
      while( stepDifference > 0 ) {
        updateWithAction( Action( EMPTY_ACTION, aSimStep - stepDifference ) );
        stepDifference--;
      }
    }
    theCore->theSituations[theCore->currentSituationIndex].setSimStep( aSimStep );
  }
  else {
    WARNING( Log::WORLD_MODEL, "Current WorldModel situation is in the future!\n" );
    theCore->theSituations[theCore->currentSituationIndex].setSimStep( aSimStep );
  }

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << "LEAVE WorldModelIntegrator::synchronizeSituations, time needed: " 
    << ( globalTimer->getAgentTime() - startTime ) << '\n' );
}


void WorldModelIntegrator::processRefereeInfo()
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << "ENTER WorldModelIntegrator::processRefereeInfo\n" );

  const vector<Message>& messages = theSensors->heardMessages();
  for( unsigned int i = 0; i < messages.size(); ++i ) {
    if( messages[i].senderInfo() == REFEREE_ID ) {
      processRefereeMessage( messages[i] );
    }
  }
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<	" LEAVE WorldModelIntegrator::processRefereeInfo, time needed: "
    << ( globalTimer->getAgentTime() - startTime ) << endl );
}


void WorldModelIntegrator::processHearInfo()
{

//cout << "WMCORE::HEARSITUATION ENTER\n";

  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << "ENTER WorldModelIntegrator::processHearInfo\n" );
  const vector<Message>& messages = theSensors->heardMessages();

  // heardSituation wird gefllt wenn processOurTeamMessage gerufen wird
  Situation heardSituation;

  for( unsigned int i = 0; i < messages.size(); ++i ) {
//    if( messages[i].senderInfo() == OUR_TEAM_ID ) {
//      if( heardSituation.simStep() != UNDEF_INT ) {
        // bereits ein Mitspielerruf enthalten und ausgwertet
//        heardSituation = Situation( UNDEF_INT );
//      }
      if( theCore->theWorldModelCommunicator->understandYourSituation( heardSituation, messages[i].message().cString() ) == 1 ) {
    	// es muss festgestellt werden, ob die zurueckgelieferte Situation
    	// ueberhaupt gueltig ist.
//cout << theCore->simStep() << "   WMCORE::HEARSITUATION = " << heardSituation << endl << endl;
     	  theCore->theSituations[theCore->currentSituationIndex] = 
     	    mergeSituations(theCore->theSituations[theCore->currentSituationIndex], heardSituation );
      }
//    }
  }

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream <<	"LEAVE WorldModelIntegrator::processHearInfo, time needed: "
    << ( globalTimer->getAgentTime() - startTime ) << endl );
}


void WorldModelIntegrator::improveConsistency()
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream << "ENTER WorldModelIntegrator::improveConsistency\n" );

  if( theCore->gameState() == WorldModelInterface::BEFORE_KICK_OFF ||
      theCore->gameState() == WorldModelInterface::GOAL ||
      theCore->gameState() == WorldModelInterface::KICK_OFF ) {
    /* In diesen Situationen kenn wir die Position des Balles genau
       --> Ball auf den Mittelpunkt zuruecklegen --> Speed ist 0
       die Confidence kann auf 1 gesetzt werden
    */
    theCore->theSituations[theCore->currentSituationIndex].
      setBall( ModelledBall( BALL_ID, Vector(0, 0), Vector(0, 0),
                             theCore->ball().lastTouchedBy(),
                             theCore->ball().lastTouchedAtTime(),
			     Confidence( theCore->simStep(), 0.0, 1.0 ),
			     Confidence( theCore->simStep(), 0.0, 1.0 ) ) );

    /* Wir kennen natrlich auch die Positionen unserer Mitspieler genau,
       da sie sich an ihre Homepositionen beamen.
       --> Alle Spieler unseres Teams setzen mit Confidence 1 */
  }

  if( theCore->gameState() == WorldModelInterface::KICK_IN ||
      theCore->gameState() == WorldModelInterface::FREE_KICK ||
      theCore->gameState() == WorldModelInterface::DROP_BALL ||
      theCore->gameState() == WorldModelInterface::CORNER_KICK ||
      theCore->gameState() == WorldModelInterface::GOAL_KICK ||
      theCore->gameState() == WorldModelInterface::OFFSIDE ||
      theCore->gameState() == WorldModelInterface::CATCH_BALL ) {
    /* In diesen Situationen knnen wir mit Confidence 1 sagen
       das die Ballgeschwindigkeit 0 ist.
    */
    theCore->theSituations[theCore->currentSituationIndex].
      setBall( ModelledBall( BALL_ID, theCore->ball().pos(), Vector(0, 0),
                             theCore->ball().lastTouchedBy(),
                             theCore->ball().lastTouchedAtTime(),
			     theCore->ball().posConfidence(),
			     Confidence( theCore->simStep(), 0.0, 1.0 ) ) );
  }

  // Wenn wir den gegnerischen Goalie einmal identifiziert haben, wollen wir
  // dieses Wissen natrlich nicht verlieren
  int anId = theSensors->opponentGoalieId();
  if( (anId >= MIN_OPPONENT_ID) && (anId <= MAX_EXISTING_OPPONENT_ID) ){
    ModelledPlayer goalie = theCore->theSituations[theCore->currentSituationIndex].playerWithId( anId );
    goalie.setIsGoalie( YES );
    theCore->theSituations[theCore->currentSituationIndex].setPlayer( goalie );
  }

  INFO( Log::WORLD_MODEL, 3, idString(), globalLog->infoStream
    << " LEAVE WorldModelIntegrator::improveConsistency, time needed: "
    << ( globalTimer->getAgentTime() - startTime ) << endl );
}


void WorldModelIntegrator::updateWithAction( const Action& anAction )
{
  int startTime = globalTimer->getAgentTime();
  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream << " ENTER WorldModelIntegrator::updateWithAction\n" );
  INFO( Log::WORLD_MODEL, 8, idString(), globalLog->infoStream <<	"Updating WorldModel with action" << anAction << endl );

  if( anAction.id() == DASH_ACTION || anAction.id() == TURN_ACTION ||
      anAction.id() == KICK_ACTION || anAction.id() == EMPTY_ACTION ) {

    // neue Situation anlegen, da neuer Simulationstakt erzeugt wird
    Situation newSituation = theCore->situation();
    
    if( anAction.id() == DASH_ACTION ) {
      // ich simuliere meinen dash
      ModelledPlayer myself = newSituation.playerWithId( theCore->id() );
      myself.accelerate( anAction.firstParameter() );
      // Confidence mu verringert werden, da dash nur simuliert
      myself.posConfidence().decrease();
      myself.speedConfidence().decrease();
      // unklar, ob ntig
      // myself.staminaConfidence().decrease();
      newSituation.setPlayer( myself );
    }
    else if( anAction.id() == TURN_ACTION ) {
      // den turn simulieren
      ModelledPlayer myself = newSituation.playerWithId( theCore->id() );
      myself.turn( anAction.firstParameter() );
      // Confidence mu verringert werden, da turn nur simuliert
      myself.bodyDirConfidence().decrease();
      myself.faceDirConfidence().decrease();
      newSituation.setPlayer( myself );
    }
    else if( anAction.id() == KICK_ACTION ) {
      const ModelledPlayer& myself = newSituation.playerWithId(theCore->id());
      ModelledBall ball = newSituation.ball();
      ball.accelerate( anAction.firstParameter(), anAction.secondParameter(), myself );
      // TM ball.setLastTouchedBy( myself.id() );
      // TM ball.setLastTouchedAtTime( newSituation.simStep() );
      // Confidence des Balles verringern
      ball.posConfidence().decrease();
      ball.speedConfidence().decrease();
      newSituation.setBall( ball );
    }
    else {
      // EMPTY_ACTION
    }

    newSituation.simulateSteps( 1 );

    if( theCore->isTimeElapsingGameState() == NO ) {
      newSituation.setSimStep( newSituation.simStep() - 1 );
    }
    
    addCurrentSituation( newSituation );

    INFO( Log::WORLD_MODEL, 9, idString(), globalLog->infoStream << "New update situation :\n" << theCore->situation() << endl );
  }

  else if( anAction.id() == TURN_NECK_ACTION ) {
    ModelledPlayer myself;
    myself = theCore->theSituations[theCore->currentSituationIndex].playerWithId( theCore->id() );
    myself.turnNeck( anAction.firstParameter() );
    // Confidence verringern
    myself.faceDirConfidence().decrease();
    theCore->theSituations[theCore->currentSituationIndex].setPlayer( myself );
  }

  // ab hier Aktionen die sofort ausgefuehrt werden
  else if( anAction.id() == MOVE_ACTION ) {
    ModelledPlayer myself;
    myself = theCore->theSituations[theCore->currentSituationIndex].playerWithId( theCore->id() );
    myself.setPos( Vector( anAction.firstParameter(),anAction.secondParameter() ) );
    // wir mutmassen, dass die Aktion ausgefhrt wurde
    myself.setPosConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
    myself.setSpeedConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
    theCore->theSituations[theCore->currentSituationIndex].setPlayer( myself );
  }
  else if( anAction.id() == CATCH_ACTION ) {
    theCore->theSimStepOfLastCatch = anAction.simStep();
  }
  else if( anAction.id() == SAY_ACTION ) {
    theCore->theSimStepOfLastSay = anAction.simStep();
  }
  else if( anAction.id() == CHANGE_VIEW_ACTION ) {
    ModelledPlayer myself;
    myself = theCore->theSituations[theCore->currentSituationIndex].playerWithId( theCore->id() );
    myself.setViewWidth( anAction.viewWidth() );  
    theCore->theSituations[theCore->currentSituationIndex].setPlayer( myself );
  }
  else if( anAction.id() == SENSE_BODY_ACTION ) {
    // nichts zu tun
  }
  else {
    throwException( Action::ActionException( "Unknown ActionId in WorldModelIntegrator::updateWithAction." ) );
  }

  INFO( Log::WORLD_MODEL, 2, idString(), globalLog->infoStream 
    << " LEAVE WorldModelIntegrator::updateWithAction, time needed: " 
    << ( globalTimer->getAgentTime() - startTime ) << endl );
} 


Situation WorldModelIntegrator::mergeSituations( const Situation& firstSituation ,  
			    const Situation& secondSituation, bool mergeWithSeenSituation ) { 
  // Version vom 22.05.2001
  // 
  // wenn eine seenSituation bergeben wird, sollte diese unbedingt! 
  //   als firstSituation bergeben werden!


bool GGG = globalWorldModel->player().isGoalie();

//cout << "-----------------------------------------------------------------" << endl;
//cout << "FIRST : " << firstSituation << endl;
//cout << "SECOND : " << secondSituation << endl;
  if( firstSituation.simStep() != secondSituation.simStep() ) { 
    cerr << "ERROR different Simsteps in Merge \n";
  }
  // hier ist unklar, ob dies wirklich notwendig / sinvoll ist
  /*if( ( theCore->gameState() & 
			(WorldModelInterface::BEFORE_KICK_OFF | WorldModelInterface::GOAL | WorldModelInterface::CATCH_BALL_AND_FREEKICK) ) != 0 ) {
    // Spieler in diesen Zustaenden nicht mergen, da beamen erlaubt
    INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream 
      << "LEAVE WorldModel::mergeSituations\n" );  
    return;  
  }*/

  int firstPlayerCount = firstSituation.playerCount();  
  int secondPlayerCount = secondSituation.playerCount();  

  INFO( Log::WORLD_MODEL, 5, idString(), globalLog->infoStream 
    << " start WorldModelIntegrator::mergeSituations with " << firstPlayerCount 
    << " first Players and " << secondPlayerCount << " second Players\n" );

  // resultSituation anlegen
  Situation resultSituation = Situation( firstSituation.simStep() );
//cout << "RESULT 0 : " << resultSituation << endl;

  // uns selber mssen wir noch in die Situation reinsetzen
  // RB: wahrscheinlich nicht
  // wenn, msste man aus secondSit noch den richtigen MergePartner raussuchen
  /*resultSituation.setPlayer( firstSituation.playerWithId( theCore->id() ) );
  firstSituation.removePlayerWithId( theCore->id() );*/
  
  

  // GHOSTBUSTERS
  if(   ( mergeWithSeenSituation == YES )
     && ( firstSituation.ball().posConfidence().reliability() < 0.1 )
     && ( theCore->player().isInFieldOfView( secondSituation.ball() ) == YES ) ) {
    ModelledBall ghostBall = secondSituation.ball();
    ghostBall.setPosConfidence( firstSituation.ball().posConfidence() );
    ghostBall.setSpeedConfidence( firstSituation.ball().speedConfidence() );
    resultSituation.setBall( ghostBall );
  }
  else if( firstSituation.ball().posConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY <
           secondSituation.ball().posConfidence().reliability() ) {
    resultSituation.setBall( secondSituation.ball() );
  }
  else{
    resultSituation.setBall( firstSituation.ball() );
  }

  
//cout << "RESULT 1 : " << resultSituation << endl;
  // Listen mit den Playern aus first- und secondSituation anlegen
  vector<ModelledPlayer> firstPlayers, secondPlayers;
  for( int i = 0; i < firstPlayerCount; ++i ){
    firstPlayers.push_back(firstSituation.playerWithIndex(i));
  }
  for( int i = 0; i < secondPlayerCount; ++i ){
    if(secondSituation.playerWithIndex(i).id() != UNDEF_ID ) {
      secondPlayers.push_back(secondSituation.playerWithIndex(i));
    }
  }

  // 1. Regel
  // Spieler, die in first und second eine " sichere" Nummer haben, werden bernommen
  for( int i = 0; i < (int) firstPlayers.size(); ++i ){
    if( firstPlayers[i].playerIdConfidence().reliability() > 0.3 ){
      if( secondSituation.playerWithId( firstPlayers[i].id() ).playerIdConfidence().reliability() > 0.3 ){
        for( int j = 0; j < (int) secondPlayers.size(); ++j ){
          if( secondPlayers[j].id() == firstPlayers[i].id() ){
            ModelledPlayer resultPlayer = 
                ( firstPlayers[i].posConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[j].posConfidence().reliability() ) ?
                  firstPlayers[i] : secondPlayers[j];
            // Problem: wenn ein Spieler nie sieht wer Goalie ist, kann er es durch Hoerinfos auch nie erfahren
            resultPlayer.setIsGoalie( firstPlayers[i].isGoalie() );
            resultSituation.setPlayer( resultPlayer );
            secondPlayers.erase( &secondPlayers[j] );
            firstPlayers.erase( &firstPlayers[i] );
            --i;
            break;
          }
        }
      }
    }
  }
//cout << "RESULT 2 : " << resultSituation << endl;
//if( secondPlayerCount > (int) secondPlayers.size()+resultSituation.playerCount() ) cout << "PROBLEM BEIM MERGE\n";
  // 2. Regel
  // Spieler mit sicherer TeamId werden 'gemaped' bzgl. minimalem Abstand
  vector<ModelledPlayer> firstPlayersSorted;
  // firstPlayers nach Teamconfidence sortieren
  for( int i = 0; i < (int) firstPlayers.size(); ++i ){
    int maxIndex = 0;
    for( int j = 0; j < (int) firstPlayers.size(); ++j ){
      if( firstPlayers[maxIndex].teamIdConfidence().reliability() < firstPlayers[j].teamIdConfidence().reliability() ){
        maxIndex = j;
      }
    }
    firstPlayersSorted.push_back( firstPlayers[maxIndex] );
    firstPlayers.erase( &firstPlayers[maxIndex] );
    --i;
  }

  int i = 0;
  while( ((int) firstPlayersSorted.size() > i) && (firstPlayersSorted[i].teamIdConfidence().reliability() > 0.3) ){
    int minIndex = -1;
    double minDistance = 100000.0;
    for( int j = 0; j < (int) secondPlayers.size(); ++j ){
      if( firstPlayersSorted[i].team() == secondPlayers[j].team() && secondPlayers[j].teamIdConfidence().reliability() > 0.3 ){
        double distance = (firstPlayersSorted[i].pos() - secondPlayers[j].pos()).length_sqr();
        if( (distance < minDistance) && 
            // Hack : sind die beiden maxError zusammen grer als die Distanz (berschneiden der Fehlerbereiche)
            //        knnte es sich um dieselben Spieler handeln
            (distance < 2.0 * (firstPlayersSorted[i].posConfidence().maxError() + secondPlayers[j].posConfidence().maxError()) + 5.0 ) ){
          minIndex = j;
          minDistance = distance;
        }
      }
    }
    if( minIndex >= 0 ){
      ModelledPlayer resultPlayer =
        ( firstPlayersSorted[i].posConfidence().reliability() > secondPlayers[minIndex].posConfidence().reliability() ) ?
        firstPlayersSorted[i] : secondPlayers[minIndex];

      if(  firstPlayersSorted[i].teamIdConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].teamIdConfidence().reliability() ){
        resultPlayer.setTeamIdConfidence( firstPlayersSorted[i].teamIdConfidence() );
      }
      else{
        resultPlayer.setTeamIdConfidence( secondPlayers[minIndex].teamIdConfidence() );
      }
      
      if( firstPlayersSorted[i].playerIdConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].playerIdConfidence().reliability() ){
        resultPlayer.setId( firstPlayersSorted[i].id() );
        resultPlayer.setPlayerIdConfidence( firstPlayersSorted[i].playerIdConfidence() );
      }
      else{
        resultPlayer.setId( secondPlayers[minIndex].id() );
        resultPlayer.setPlayerIdConfidence( secondPlayers[minIndex].playerIdConfidence() );
      }
      
      resultSituation.setPlayer( resultPlayer );
      firstPlayersSorted.erase( &firstPlayersSorted[i] );
      secondPlayers.erase( &secondPlayers[minIndex] );
    }
    else{
      ++i;
    }
  }
//cout << "RESULT 3 : " << resultSituation << "  first = " << firstPlayersSorted.size() << "  second = " << secondPlayers.size() << endl;
//if( secondPlayerCount > (int) secondPlayers.size()+resultSituation.playerCount() ) cout << "PROBLEM BEIM MERGE\n";
  // 3. Regel
  // Spieler mit unsicherer TeamId werden 'gemapt' bzgl. minimalem Abstand
  i = 0;
  while( ((int) firstPlayersSorted.size() > i) && (firstPlayersSorted[i].teamIdConfidence().reliability() > 0.49) ){
    int minIndex = -1;
    double minDistance = 100000.0;
    for( int j = 0; j < (int) secondPlayers.size(); ++j ){
      if( firstPlayersSorted[i].team() == secondPlayers[j].team() || secondPlayers[j].teamIdConfidence().reliability() < 0.2 ){
        double distance = (firstPlayersSorted[i].pos() - secondPlayers[j].pos()).length();
        if( (distance < minDistance) && 
            // Hack : sind die beiden maxError zusammen grer als die Distanz (berschneiden der Fehlerbereiche)
            //        knnte es sich um dieselben Spieler handeln
            (distance < 2.0 * (firstPlayersSorted[i].posConfidence().maxError() + secondPlayers[j].posConfidence().maxError()) + 5.0 ) ){
          minIndex = j;
          minDistance = distance;
        }
      }
    }
    if( minIndex >= 0 ){
      if( firstPlayersSorted[i].playerIdConfidence().reliability() < 0.1 ){
        firstPlayersSorted[i].setId( secondPlayers[minIndex].id() );
        firstPlayersSorted[i].setPlayerIdConfidence( secondPlayers[minIndex].playerIdConfidence() );
      }
      resultSituation.setPlayer( firstPlayersSorted[i] );
      firstPlayersSorted.erase( &firstPlayersSorted[i] );
      secondPlayers.erase( &secondPlayers[minIndex] );
    }
    else{
      ++i;
    }
  }
//cout << "RESULT 4 : " << resultSituation << "  first = " << firstPlayersSorted.size() << "  second = " << secondPlayers.size() << endl;
//if( secondPlayerCount > (int) secondPlayers.size()+resultSituation.playerCount() ){
 //cout << "PROBLEM BEIM MERGE\n"; // ACHTUNG! NOCH NICHT BEHOBEN!
//}
  // 4. Regel 
  // die 2 Spieler mit dem geringsten Abstand werden aufeinander 'gemuppet'
  for(int i = 0; i < (int) firstPlayersSorted.size() && resultSituation.playerCount() < 22; ++i ){
    int minIndex = -1;
    double minDistance = 100000.0;
    for( int j = 0; j < (int) secondPlayers.size(); ++j ){
      double distance = (firstPlayersSorted[i].pos() - secondPlayers[j].pos()).length();
        if( (distance < minDistance) && 
            // Hack : sind die beiden maxError zusammen grer als die Distanz (berschneiden der Fehlerbereiche)
            //        knnte es sich um dieselben Spieler handeln
            (distance < 2.0 * (firstPlayersSorted[i].posConfidence().maxError() + secondPlayers[j].posConfidence().maxError()) + 5.0 ) ){
        minIndex = j;
        minDistance = distance;
      }
    }
    if( minIndex >= 0 ){
      ModelledPlayer resultPlayer = 
        ( firstPlayersSorted[i].posConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].posConfidence().reliability() ) ?
        firstPlayersSorted[i] : secondPlayers[minIndex];
      if( firstPlayersSorted[i].team() == secondPlayers[minIndex].team() ){
        if( firstPlayersSorted[i].playerIdConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].playerIdConfidence().reliability() ){
          resultPlayer.setId( firstPlayersSorted[i].id() );
          resultPlayer.setPlayerIdConfidence( firstPlayersSorted[i].playerIdConfidence() );
        }
        else{
          resultPlayer.setId( secondPlayers[minIndex].id() );
          resultPlayer.setPlayerIdConfidence( secondPlayers[minIndex].playerIdConfidence() );
        }
        resultPlayer.setTeamIdConfidence(
          (firstPlayersSorted[i].teamIdConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].teamIdConfidence().reliability()) ?
          (firstPlayersSorted[i].teamIdConfidence()) : (secondPlayers[minIndex].teamIdConfidence()) );
      }
      else{
        // von dem Spieler von dem ich die TeamId bernehme mu ich auch die PlayerId bernehmen
        if( firstPlayersSorted[i].teamIdConfidence().reliability() + DEFAULT_COMPARISON_ACCURACY >= secondPlayers[minIndex].teamIdConfidence().reliability() ){
          resultPlayer.setId( firstPlayersSorted[i].id() );
          resultPlayer.setTeamIdConfidence( firstPlayersSorted[i].teamIdConfidence() );
          resultPlayer.setPlayerIdConfidence( firstPlayersSorted[i].playerIdConfidence() );
        }
        else{
          resultPlayer.setId( secondPlayers[minIndex].id() );
          resultPlayer.setTeamIdConfidence( secondPlayers[minIndex].teamIdConfidence() );
          resultPlayer.setPlayerIdConfidence( secondPlayers[minIndex].playerIdConfidence() );
        }
      }
        
      resultSituation.setPlayer( resultPlayer );
      firstPlayersSorted.erase( &firstPlayersSorted[i] );
      secondPlayers.erase( &secondPlayers[minIndex] );
      --i;
    }
  }
//cout << "RESULT 5 : " << resultSituation << endl;
//if( secondPlayerCount > (int) secondPlayers.size()+resultSituation.playerCount() ) cout << "PROBLEM BEIM MERGE\n";
  // 5. Regel
  // verbleibende Spieler aus firstSituation bernehmen
  // Beispielsweise am Anfang gibt es noch keine simulierten Spieler, d.h. die vorhergehenden Regeln
  // haben versagt, da sie die Spieler aus First nicht auf Spieler in Second mappen konnten
  while( firstPlayersSorted.size() > 0 && resultSituation.playerCount() < 22 ){
    resultSituation.setPlayer( firstPlayersSorted[0] );
    firstPlayersSorted.erase( &firstPlayersSorted[0] );
  }
//cout << "RESULT 6 : " << resultSituation << endl;
//if( secondPlayerCount > (int) secondPlayers.size()+resultSituation.playerCount() ) cout << "PROBLEM BEIM MERGE\n";
  // 6. Regel
  // verbleibende Spieler aus secondSituation bernehmen
  while( secondPlayers.size() > 0 && resultSituation.playerCount() < 22 ){
    if( mergeWithSeenSituation == YES && resultSituation.playerWithId( theCore->id() ).isInFieldOfView( secondPlayers[0] ) == YES ){
      // ich mte den Spieler eigentlich sehen, er befindet sich aber nicht im Sichtfeld
      // --> Winkeldifferenz des Spielers vom linken und rechten Sichtfeldrand bestimmen 
      const ModelledPlayer& mySelf = resultSituation.playerWithId( theCore->id() );
      if( (mySelf.angleOfFaceTo( secondPlayers[0].pos() ) > mySelf.visibleAngle() / 2.0 - Ath98_PI_10_DEGREE) &&
          (mySelf.angleOfFaceTo( secondPlayers[0].pos() ) < normalizeAngle(Ath98_PI_10_DEGREE - mySelf.visibleAngle() / 2.0)) ){
        resultSituation.setPlayer( secondPlayers[0] );
      }
    }
    else{
      resultSituation.setPlayer( secondPlayers[0] );
    }
    secondPlayers.erase( &secondPlayers[0] );
  

 
/*
   Verschieben bringt Probleme
      // --> Spieler an den nheren Sichtfeldrand schieben (distance konst. und winkel der nhere Sichtwinkelstrahl)
      //     Confidence 0.1 fr pos
      const ModelledPlayer& mySelf = resultSituation.playerWithId( theCore->id() );
      Vector movedPos = mySelf.pos() + Vector( mySelf.distTo( secondPlayers[0].pos() ), 
                                               ( mySelf.angleOfFaceTo( secondPlayers[0].pos() ) < Ath98_PI ) ? 
                                                   normalizeAngle( mySelf.faceDir() + mySelf.visibleAngle() / 2.0 ) :
                                                   normalizeAngle( mySelf.faceDir() - mySelf.visibleAngle() / 2.0 ),
                                               POLAR, RADIAN );
      secondPlayers[0].setPosConfidence( 
        Confidence( resultSituation.simStep(), 
                    (secondPlayers[0].pos() - movedPos).length() + secondPlayers[0].posConfidence().maxError(), 0.1 ) );
      secondPlayers[0].setPos( movedPos );

    }
    resultSituation.setPlayer( secondPlayers[0] );
    secondPlayers.erase( &secondPlayers[0] );
*/
  }
//cout << "RESULT 7 : " << resultSituation << endl;
//if( secondPlayerCount > secondPlayers.size()+resultSituation.playerCount() ) cout << "PROBLEM BEIM MERGE\n";
//cout << "-------------------------------------------------------" << endl << endl;

  if( (GGG==YES) && (resultSituation.playerWithId(theCore->id()).isGoalie() == NO ) ){
    cerr << "Oups, ich hab vergessen, wer ich bin ?!\n";
  }
  return resultSituation;
}


const ModelledPlayer WorldModelIntegrator::modelOfSelf( const BodyInfos& bodyInfos ) const
{
  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream 
    << " ENTER WorldModel::modelOfSelf\n" );

  // Spieler holen
  ModelledPlayer playerSelf = theCore->situation().playerWithId( theCore->id() );
  if( playerSelf.id() == UNDEF_ID ){
    cerr << "WorldModelIntegrator::modelOfSelf     playerSelf.id() ist UNDEF !!!\n";
    WARNING( Log::WORLD_MODEL, "playerSelf.id() ist UNDEF !!!\n" ); 
  }

  // Berechnung der Blickrichtung
  double faceDir =  theCore->theField.computeFaceDir( theSensors->sensedLines() ) ;

  if ( theCore->theSide == LEFT_SIDE_ID ) {
    playerSelf.setFaceDir( faceDir );
  }
  else if ( theCore->theSide == RIGHT_SIDE_ID ) {
    playerSelf.setFaceDir( normalizeAngle( faceDir - Ath98_PI ) );
  }
  else {
    throwException( InvalidIdException( "Undef side Id" ) );
  }
  
  // die Confidence neu setzen
  playerSelf.setFaceDirConfidence( Confidence( theCore->simStep(), 0.5, 1.0 ) );

  Vector position( theCore->theField.calculatePosition( theSensors->sensedFlags() ) );

  playerSelf.setPos( position );

  playerSelf.setPosConfidence( Confidence( theCore->simStep(), 0.20, 1.0 ) );

  // eigene Koerperrichtung bestimmen 
  double bodyDir = normalizeAngle ( playerSelf.faceDir() - bodyInfos.headAngle() );
  playerSelf.setBodyDir( bodyDir );
  // Confidence auf 1 setzen und Maxerror auf 1 Grad
  playerSelf.setBodyDirConfidence( Confidence( theCore->simStep(), 1.0, 1.0 ) );

  // eigene Geschwindigkeit bestimmen
  // Alternative A : aus den BodyInfos
  // Alternative B : aus den Positionsdifferenzen

  // HACK
  //diese speedDir schwankt ziemlich stark
  //Zur Verbesserung kann ber die speedDir der alten Situation gemittelt werden (sofern kein DASH)
  //oder man summiert ber die BodyDirs bei den DASH-Kommandos
  double speedDir = normalizeAngle( bodyInfos.directionOfSpeed() + playerSelf.faceDir() );

  double speedDist = bodyInfos.amountOfSpeed();
  playerSelf.setSpeed( Vector( speedDist, speedDir, POLAR, RADIAN ) );
  playerSelf.setSpeedConfidence( Confidence( theCore->simStep(), 0.1, 1.0 ) );

  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream 
    << " LEAVE WorldModel::modelOfSelf\n" );
     
  return playerSelf;    
}// end WMI::modelOfSelf()
 

void WorldModelIntegrator::updateOwnPlayerWithBodyInfos( 
				   ModelledPlayer& playerSelf,
				   const BodyInfos& bodyInfos ) const
{
  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream 
    << " ENTER WorldModelIntegrator::updateOwnPlayerWithBodyInfos\n" );

  if( bodyInfos.time() == theCore->simStep() ) { 
    // nur auswerten, wenn aktuelle BodyInfos vorhanden. 
    // VisibleAngle, Stamina und Effort setzen 
    playerSelf.setViewWidth( bodyInfos.viewWidth() );
    playerSelf.setStamina( bodyInfos.stamina() );
    // Confidence auf 1 setzen und Maxerror auf 0
    playerSelf.setStaminaConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
    playerSelf.setEffort( bodyInfos.effort() );

    playerSelf.setTeamIdConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
    playerSelf.setPlayerIdConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
  }
  else {
    WARNING( Log::WORLD_MODEL, "No current bodyInfos available.\n" );  
  }

  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream <<
        "    LEAVE WorldModelIntegrator::updateOwnPlayerWithBodyInfos\n" );
}

 
void WorldModelIntegrator::addSeenObjects( Situation& newSituation )
{
  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream
    << "ENTER WorldModel::addSeenObjects\n" );

  const ModelledPlayer& myself = newSituation.playerWithId( theCore->id() );
  if( myself.id() == UNDEF_ID ){
    cerr << "WorldModelIntegrator::addSeenObjects     myself.id() ist UNDEF !!!\n";
    WARNING( Log::WORLD_MODEL, "myself.id() ist UNDEF !!!\n" );
    return;
  }

  if(theCore->simStep() > 0) {
    INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream
      << " WorldModel::addSeenObjects\n"
      << theSensors->lastBodyInfos()
      << "myself at : " << newSituation.simStep()
      << myself.pos().x() << " " << myself.pos().y()
      << " " << myself.speed().x() << " " << myself.speed().y()
      << " FaceDir: " << myself.faceDir()
      << " BodyDir: " << myself.bodyDir() << endl );
  }

  // den Ball vom Sensor holen
  // falls ein See ohne Ballinformation vorliegt
  // ist Ball.isValid() == FALSE
  ModelledBall sensedBall = ModelledBall( theSensors->sensedBall() );

  if ( (sensedBall.isValid()) && (sensedBall.pos().isValid() == YES) ) {
    if(theCore->simStep() > 0) {
      INFO( Log::WORLD_MODEL, 10, idString(), globalLog->infoStream
        << "sensedBall " << sensedBall.pos().x()
        << " " << sensedBall.pos().y()
        << " " << sensedBall.speed().x() << " " << sensedBall.speed().y()
        << " Ball.simStep = " << sensedBall.posConfidence().simStep()
        << " GWM.simStep = " << theCore->simStep() << endl );
    }
    // relative Koordinaten berechnen
    Vector relativeBallPos = relativePos( sensedBall.pos(), myself.faceDir() );

    // absolute Koordinaten berechnen
    ModelledBall absoluteSeenBall = sensedBall;
    absoluteSeenBall.setPos( myself.pos() + relativeBallPos );
    /* ! Fr die Confidence des Balles wird nur der ballspezifische Fehler (relativ zu mir) bewertet,
         da in den meisten Fllen die absolute Position des Balles nicht exakt bentigt wird.
         Wohl aber ist es sinnvoll, bei nahen Bllen die Confidence nicht unntig zu verringern
         - uns interessiert nur die relative Position zu mir selbst.*/
    absoluteSeenBall.setPosConfidence( Confidence( theCore->simStep(),
                     myself.distTo(sensedBall.pos()) * 0.05 + 0.05, // ermittelter Fehler fr bewegte Objekte
                     1.0 ) ); 

    if( sensedBall.speed().isValid() == YES ){
      // absoluten Speed berechnen (Variablen benannt nach Serverfunktion im ServerManual 7alpha)
	    double vx0 = myself.speed().x();
	    double vy0 = myself.speed().y();
	    double prx = absoluteSeenBall.pos().x() - myself.pos().x();
	    double pry = absoluteSeenBall.pos().y() - myself.pos().y();
	    double dist = (absoluteSeenBall.pos() - myself.pos()).length();
	    double erx = prx / dist;
	    double ery = pry / dist;
	    double distchng = sensedBall.speed().x();
	    double dirchng = sensedBall.speed().y();
	    double vry = (distchng*ery/erx - dirchng*dist) / (erx+ery*ery/erx);
	    double vrx = (distchng - vry * ery) / erx;
	    double vxt = vrx + vx0;
	    double vyt = vry + vy0;
    
      absoluteSeenBall.setSpeed( Vector(vxt, vyt, CARTESIAN) );

      // speed ggf. deckeln (kann hchstens mit 2.7 * 0.94 + ServerFehler gesehen werden, da im Server erst decay, dann see)
      // ServerNoise mit maximal 0.16 empisch bestimmt
      double ts = globalConf->ballSpeedMax * globalConf->ballDecay + 0.16;
      if( absoluteSeenBall.speed().length() > ts ) {
        cerr << theCore->simStep() << "  Ball seen with speed of : " << absoluteSeenBall.speed().length() << endl;
        absoluteSeenBall.setSpeed( Vector( ts, absoluteSeenBall.speed().angle(), POLAR, RADIAN ) );
        absoluteSeenBall.setSpeedConfidence( Confidence( theCore->simStep(),
                                             2.0*globalConf->ballSpeedMax,
                                             1 / ( 1 + absoluteSeenBall.speed().length() - globalConf->ballSpeedMax ) ) );
      }
      else{
        // Annahmen wie bei posConfidence
        absoluteSeenBall.setSpeedConfidence( Confidence( theCore->simStep(),
                                             absoluteSeenBall.speed().length() * (2.3 / (2.0 * absoluteSeenBall.speed().length() + 1) - 0.3 ), // empirisch bestimmt.
                                             1.0 ) );
      }
    }
    else{
     // Algorithmus fr das Balltracking rufen
     // ist nur noch notwendig, wenn nur die Position (ohne speed) gesehen wurde
     absoluteSeenBall = trackBall( absoluteSeenBall, myself );
    }

    if(theCore->simStep() > 0) {
      INFO( Log::WORLD_MODEL, 10, idString(), globalLog->infoStream
        << "absolutSeenBall " << absoluteSeenBall.pos().x()
        << " " << absoluteSeenBall.pos().y()
        << " " << absoluteSeenBall.speed().x()
        << " " << absoluteSeenBall.speed().y()
        << " Ball.simStep = " << absoluteSeenBall.posConfidence().simStep()
        << " GWM.simStep = " << theCore->simStep() << endl );
    }

    // Ball setzen
    newSituation.setBall( absoluteSeenBall );
  }
  else {
    // See enthielt keinen Ball --> wir setzen einen ungltigen
    // im merge sollte dieser dann durch einen weitersimulierten ersetzt werden
    newSituation.setBall( ModelledBall( BALL_ID, Vector(0.0, 0.0), Vector(0.0, 0.0),
                          theCore->ball().lastTouchedBy(),
                          theCore->ball().lastTouchedAtTime(),
                          Confidence( theCore->simStep(), 50.0, 0.0 ),
                          Confidence( theCore->simStep(), globalConf->ballSpeedMax, 0.0 ) ) );
  }

  // Der Reihe nach alle gesehenen Spieler hinzufuegen
  ModelledPlayer newPlayer;
  int playerCount = theSensors->sensedPlayers().size();
  for( int i = 0; i < playerCount; ++i ) {
    newPlayer = ModelledPlayer( theSensors->sensedPlayers()[i] );
    // setzen der Team- und IdConfidence
    if( newPlayer.team() == UNDEF_TEAM_ID ){
      newPlayer.setTeamIdConfidence( Confidence( theCore->simStep(), 1.0, 0.0 ) );
      newPlayer.setPlayerIdConfidence( Confidence( theCore->simStep(), 1.0, 0.0 ) );
    }
    else{
      newPlayer.setTeamIdConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
      
      if( newPlayer.isPlayerNumberKnown() == YES ){
        newPlayer.setPlayerIdConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
      }
      else{
        newPlayer.setPlayerIdConfidence( Confidence( theCore->simStep(), 1.0, 0.0 ) );
      }
    }

    // Position und hnliche Daten setzen
    if( newPlayer.pos().isValid() ) { 
      // Blickwinkel wegrechnen 
      Vector relativePlayerPos = relativePos( newPlayer.pos(), myself.faceDir() );
      // absolute Position aus relativer Position errechnen
      newPlayer.setPos( myself.pos() + relativePlayerPos );
      // aus o.g. Grnden wird nur die objekteigene Unsicherheit bercksichtigt
      newPlayer.setPosConfidence(
        Confidence( theCore->simStep(),
                    myself.distTo(newPlayer.pos()) * 0.05 + 0.05, // ermittelter Fehler fr bewegte Objekte
                    1.0 ) );

      if ( newPlayer.speed().isValid() ) { 

	      // absoluten Speed berechnen (Variablen benannt nach Serverfunktion im ServerManual 7alpha)
		    double vx0 = myself.speed().x();
		    double vy0 = myself.speed().y();
		    double prx = newPlayer.pos().x() - myself.pos().x();
		    double pry = newPlayer.pos().y() - myself.pos().y();
		    double dist = (newPlayer.pos() - myself.pos()).length();
		    double erx = prx / dist;
		    double ery = pry / dist;
		    double distchng = newPlayer.speed().x();
		    double dirchng = newPlayer.speed().y();
		    double vry = (distchng*ery/erx - dirchng*dist) / (erx+ery*ery/erx);
		    double vrx = (distchng - vry * ery) / erx;
		    double vxt = vrx + vx0;
		    double vyt = vry + vy0;
	    
	      newPlayer.setSpeed( Vector(vxt, vyt, CARTESIAN) );

	      // speed ggf. deckeln   HACK (hart gesetzte Maximalwerte)
	      if( newPlayer.speed().length() > 1.2 /*newPlayer.maxSpeed()*/ * 0.6 /*newPlayer.decay*/ ) {
	        cerr << theCore->simStep() << "  Player seen with speed of : " << newPlayer.speed().length() << endl;
	        newPlayer.setSpeed( Vector( 1.2 * 0.6, newPlayer.speed().angle(), POLAR, RADIAN ) );
	        newPlayer.setSpeedConfidence( Confidence( theCore->simStep(), 2.0 * 1.2 * 0.6,
	                                             1 / ( 1 + newPlayer.speed().length() - 1.2 * 0.6 ) ) );
	      }
	      else{
	        // Annahmen wie bei posConfidence
	        newPlayer.setSpeedConfidence( Confidence( theCore->simStep(),
	                                                  newPlayer.speed().length() * (2.2 / 
	                                                  ( (5.0/newPlayer.maxSpeed()) * newPlayer.speed().length() + 1) - 0.2 ), // empirisch bestimmt.
	                                                  1.0 ) );
	      }
      }
	    else{
	      newPlayer.setSpeed( Vector( 0.0, 0.0, CARTESIAN ) );
	      newPlayer.setSpeedConfidence( Confidence( theCore->simStep(), 1.2 , 0.0 ) );
	    }

      if( newPlayer.bodyDir() != UNDEF_DOUBLE ) {
        newPlayer.setBodyDir( newPlayer.bodyDir() + myself.faceDir() );  
        newPlayer.setBodyDirConfidence( Confidence( theCore->simStep(), 0.5, 1.0 ) );
      }
      else { 
        newPlayer.setBodyDir( 0 );  
        newPlayer.setBodyDirConfidence( Confidence( theCore->simStep(), Ath98_PI, 0.0 ) );
      }

      if( newPlayer.faceDir() != UNDEF_DOUBLE ) {
        newPlayer.setFaceDir( normalizeAngle(newPlayer.faceDir() + myself.faceDir() ) );
        newPlayer.setFaceDirConfidence(
          Confidence( theCore->simStep(),
                      myself.faceDirConfidence().maxError() + 0.5,
                      myself.faceDirConfidence().reliability() * 1.0 ) );
      }
      else { 
        newPlayer.setFaceDir( 0 );
        newPlayer.setFaceDirConfidence( Confidence( theCore->simStep(), Ath98_PI, 0.0 ) );
      } 
    }
    else { 
      newPlayer.setPos( Vector( 0, 0 ) );
      newPlayer.setPosConfidence( Confidence( theCore->simStep(), 50.0, 0.0 ) );
      newPlayer.setSpeed( Vector( 0, 0 ) );
      newPlayer.setSpeedConfidence( Confidence( theCore->simStep(), 1.2, 0.0 ) );
      newPlayer.setFaceDir( 0 );
      newPlayer.setFaceDirConfidence( Confidence( theCore->simStep(), Ath98_PI, 0.0 ) );
      newPlayer.setBodyDir( 0 );
      newPlayer.setBodyDirConfidence( Confidence( theCore->simStep(), Ath98_PI, 0.0 ) );
      WARNING( Log::WORLD_MODEL, "See a player without a position.\n" );
    }
    newSituation.addPlayer( newPlayer ); 
  } 

  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream
    << " LEAVE WorldModel::addSeenObjects\n" ); 

}// end of WMI::addSeenObjects()



void WorldModelIntegrator::processRefereeMessage( const Message& message )  
{ 
  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream
    << " ENTER WorldModel::processRefereeMessage\n" );

  setSimStepOfLastRefereeDecision( theCore->simStep() );
  bool haveToResetBall = NO;
   
  Ath98String messageString = message.message();  
  if ( messageString == "before_kick_off" ) { 
    theCore->setGameState( WorldModelInterface::BEFORE_KICK_OFF );
    // Links faengt an.
    theCore->setInitiative( LEFT_SIDE_ID );
  }
  else if ( messageString == "time_over" ) {
    theCore->setGameState( WorldModelInterface::TIME_OVER );
    theCore->setInitiative(  NO_TEAM_ID );
  }  
  else if ( messageString == "play_on" ) { 
    theCore->setGameState( WorldModelInterface::PLAY_ON );
    theCore->setInitiative(  BOTH_TEAMS_ID );
  }
  else if ( messageString == "kick_off_l" ) {
    theCore->setGameState( WorldModelInterface::KICK_OFF );
    theCore->setInitiative(  LEFT_SIDE_ID );
  } 
  else if ( messageString == "kick_off_r" ) {
    theCore->setGameState( WorldModelInterface::KICK_OFF );
    theCore->setInitiative(  RIGHT_SIDE_ID );
  }  
  else if ( messageString == "kick_in_l" ) { 
    theCore->setGameState( WorldModelInterface::KICK_IN );
    theCore->setInitiative( LEFT_SIDE_ID );
    haveToResetBall = YES;
  } 
  else if ( messageString == "kick_in_r" ) { 
    theCore->setGameState( WorldModelInterface::KICK_IN );
    theCore->setInitiative(  RIGHT_SIDE_ID );
    haveToResetBall = YES;
  } 
  else if ( messageString == "free_kick_l" ) { 
    theCore->setGameState( WorldModelInterface::FREE_KICK );
    theCore->setInitiative(  LEFT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "free_kick_r" ) {
    theCore->setGameState( WorldModelInterface::FREE_KICK );
    theCore->setInitiative(  RIGHT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "drop_ball" ) {
    theCore->setGameState( WorldModelInterface::DROP_BALL );
    theCore->setInitiative(  BOTH_TEAMS_ID );
    haveToResetBall = YES;
  }
  else if ( messageString == "corner_kick_l" ) {
    theCore->setGameState( WorldModelInterface::CORNER_KICK );
    theCore->setInitiative( LEFT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "corner_kick_r" ) {
    theCore->setGameState( WorldModelInterface::CORNER_KICK );
    theCore->setInitiative( RIGHT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "goal_kick_l" ) {
    theCore->setGameState( WorldModelInterface::GOAL_KICK );
    theCore->setInitiative( LEFT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "goal_kick_r" ) {
    theCore->setGameState( WorldModelInterface::GOAL_KICK );
    theCore->setInitiative( RIGHT_SIDE_ID );
    setSimStepOfLastFreekick( theCore->simStep() );
    haveToResetBall = YES;
  }
  else if ( messageString == "offside_l" ) {
    theCore->setGameState( WorldModelInterface::OFFSIDE );
    theCore->setInitiative( RIGHT_SIDE_ID );
    haveToResetBall = YES;
  }
  else if ( messageString == "offside_r" ) {
    theCore->setGameState( WorldModelInterface::OFFSIDE );
    theCore->setInitiative( LEFT_SIDE_ID );
    haveToResetBall = YES;
  }
  else if ( messageString == "goalie_catch_ball_l" ) {
    theCore->setGameState( WorldModelInterface::CATCH_BALL );
    theCore->setInitiative( LEFT_SIDE_ID );
    haveToResetBall = YES;
  }
  else if ( messageString == "goalie_catch_ball_r" ) {
    theCore->setGameState( WorldModelInterface::CATCH_BALL );
    theCore->setInitiative( RIGHT_SIDE_ID );
    haveToResetBall = YES;
  }
  else if ( messageString.beginsWith( "goal_l" ) ) {
    int newScore = messageString.subString( 7 ).asInt();
    theCore->setGameState( WorldModelInterface::GOAL );
    if ( theCore->side() == LEFT_SIDE_ID ) {
      theCore->setScore( newScore, OUR_TEAM_ID );
    }
    else {
      theCore->setScore( newScore, THEIR_TEAM_ID );
    }
    theCore->setInitiative( RIGHT_SIDE_ID );
  }
  else if ( messageString.beginsWith( "goal_r" ) ) {
    int newScore = messageString.subString( 7 ).asInt();
    theCore->setGameState( WorldModelInterface::GOAL );
    if ( theCore->side() == RIGHT_SIDE_ID ) {
      theCore->setScore( newScore, OUR_TEAM_ID );
    }
    else {
      theCore->setScore( newScore, THEIR_TEAM_ID );
    }
    theCore->setInitiative( LEFT_SIDE_ID );
  }
  else if ( messageString.beginsWith( "foul_l" ) ) {
    theCore->setGameState( WorldModelInterface::FOUL );
    theCore->setInitiative( RIGHT_SIDE_ID );
  }
  else if ( messageString.beginsWith( "foul_r" ) ) {
    theCore->setGameState( WorldModelInterface::FOUL );
    theCore->setInitiative( LEFT_SIDE_ID );
  }
  else if ( messageString == "half_time" ) {
    INFO( Log::WORLD_MODEL ,  8 ,  idString() , globalLog->infoStream  <<
	  " Halbzeit: Setze Stamina-Modell zurueck.\n"  ) ;
    theCore->setGameState( WorldModelInterface::HALF_TIME );
    theCore->setInitiative( NO_TEAM_ID );
    ModelledPlayer myself = theCore->player();
    myself.setStaminaValues( globalConf->playerStaminaMax, 1.0, 1.0 ); /*HACK*/
    myself.setStaminaConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
    theCore->theSituations[theCore->currentSituationIndex].setPlayer(myself);
  }
  else if ( messageString == "time_up" ) {
    theCore->setGameState( WorldModelInterface::TIME_UP );
    theCore->setInitiative( NO_TEAM_ID );
  }
  else if ( messageString == "time_up_without_a_team" ) {
    theCore->setGameState( WorldModelInterface::TIME_UP_WITHOUT_A_TEAM );
    theCore->setInitiative( NO_TEAM_ID );
  }
  else if ( messageString == "time_extended" ) {
    theCore->setGameState( WorldModelInterface::TIME_EXTENDED );
    theCore->setInitiative( NO_TEAM_ID );
  }
  else if ( messageString.beginsWith( "goalie_catch_ball" ) ) {
    // Nichts machen. Gleich darauf kommt ein Freekick. Dann wird
    // die Initiative richtig gesetzt.
  }
  else if ( messageString.beginsWith( "freeze" ) ) {
    theCore->setGameState( WorldModelInterface::FREEZE );
  }
  else {
    WARNING( Log::WORLD_MODEL, "Unknown referee decision '" +
             messageString + "' ignored.\n" );
  }

  if( haveToResetBall == YES ) {
    // in diesen Situationen hat sich die Ballpositionen verndert ohne
    // das wir wissen wie (z.B. Schiri hat den Ball irgendwo hingesetzt)
    // im Gegensatz zu improveConsistency wird hier die Ballposition nur 
    // einmal gesetzt (improveConsistency fr die Dauer des Gamestates)
    ModelledBall ball( theCore->ball() );
    ball.setPosConfidence(Confidence( theCore->simStep(), 100.0, 0.0 ));
    theCore->theSituations[theCore->currentSituationIndex].setBall( ball );
  }

  INFO( Log::WORLD_MODEL, 7, idString(), globalLog->infoStream
    << "New game state: " << theCore->gameState()
    << " New initiative: " << theCore->initiative() << endl );
  INFO( Log::WORLD_MODEL, 4, idString(), globalLog->infoStream
    << " LEAVE WorldModel::processRefereeMessage\n" );
} // end of WMI::processRefereeMessage



Vector WorldModelIntegrator::relativePos( const Vector& seenPos, double faceDir )
{ 
  return Vector( seenPos.length(), normalizeAngle(seenPos.angle() + faceDir), POLAR, RADIAN );
} 


  
ModelledBall WorldModelIntegrator::trackBall(const ModelledBall& seenBall, const ModelledPlayer& myself)
{
  ModelledBall trackedBall = seenBall;
  ModelledBall simBall = theCore->ball();
  ModelledBall oldSeenBall = theCore->situation(-1).ball(); // Initialisierung passt nicht zum Namen

  bool foundOldSeenBall = NO;

  if( globalWorldModel->simStep() > 0 ) {
	  // wir suchen den letzten gesehenen Ball in der situation-history
	  int i;
	  for( i = -1; i > -1 * globalConf->historyCount; --i )
	  {
	    if( theCore->situation(i).ball().posConfidence().reliability() > 0.99 ) {
	      oldSeenBall = theCore->situation(i).ball();
	      foundOldSeenBall = YES;
	      break;
	    }
	  }

	  int stepDiff = trackedBall.posConfidence().simStep() - oldSeenBall.posConfidence().simStep();
	  Vector posDiff = trackedBall.pos() - oldSeenBall.pos();

	  // Speedbestimmung:
	  double oldSpeedLength = posDiff.length() / globalSkills->getNewBallDistance( stepDiff );
	  Vector trackedSpeed = Vector( oldSpeedLength * pow(globalConf->ballDecay, stepDiff),
	                                (trackedBall.pos() - oldSeenBall.pos()).angle(), POLAR, RADIAN );

	  double simSpeedAngleDiff = Ath98_PI - fabs( Ath98_PI - normalizeAngle(trackedSpeed.angle() - simBall.speed().angle()) );
	  // HACK  10000.0 = f(UNDEF)
	  if( ( ( simBall.speedConfidence().reliability() > 0.7) && ( simSpeedAngleDiff < radFromDegree(10) ) && ( simSpeedAngleDiff < 10000.0 ) ) ||
	      foundOldSeenBall == NO ) {
//cout << "trackedBall()  - simSpeedAngleDiff zu gross: " << degFromRadiant(simSpeedAngleDiff) << endl;
//cout << "                 setze simBall.speed()\n";
	    trackedBall.setSpeed( simBall.speed() );
	    trackedBall.setSpeedConfidence( simBall.speedConfidence() );
	  }
	  else {
	    trackedBall.setSpeed( trackedSpeed );
	    // HACK  -  Fehlerfunktion muss noch bestimmt werden
	    trackedBall.setSpeedConfidence( Confidence( theCore->simStep() , -i * 0.3 , 1.0) );
	  }

	  // speed ggf auf ballSpeedMax begrenzen und bei berschreitung Confidence runtersetzen
	  // wegen ServerNoise addieren wir noch 0.16 (empirisch bestimmt)
    double ts = globalConf->ballSpeedMax * globalConf->ballDecay + 0.16;
	  if( trackedBall.speed().length() > ts ){
//cout << "!!! trackedBall()  - ballspeed wurde gedeckelt  ( " << trackedBall.speed().length() << ")\n";
// HACK
          if( trackedBall.speed().length() > 3.5) {
	          trackedBall.setSpeedConfidence( Confidence( theCore->simStep(), 0.0, 1.0 ) );
	          trackedBall.setSpeed( Vector( 0.0, 0.0 ) );
          }
          else {          
	          trackedBall.setSpeedConfidence( Confidence( theCore->simStep(),
	                                          trackedBall.speedConfidence().maxError() + trackedBall.speed().length() - globalConf->ballSpeedMax,
	                                          trackedBall.speedConfidence().reliability() + (0.1) / (0.1 + trackedSpeed.length() - globalConf->ballSpeedMax) ) );
	          trackedBall.setSpeed( Vector( ts, trackedSpeed.angle(), POLAR, RADIAN ) );
	        }
	  }
//cout << "trackedBall() : " << trackedBall.speed().length() << "\n";

	  return trackedBall;
	}
  else {
	  trackedBall.setSpeed( Vector(0.0, 0.0) );
	  trackedBall.setSpeedConfidence( Confidence( theCore->simStep() , 0.0 , 1.0) );

	  return trackedBall;
  }
}
